package com.bocloud.cop.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Script;
import com.bocloud.cop.entity.ScriptVersion;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.TaskParam;
import com.bocloud.cop.intf.ScriptService;
import com.bocloud.cop.intf.ScriptVersionService;
import com.bocloud.cop.model.FastTask;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.model.ScriptBean;
import com.bocloud.cop.service.utils.JobScheduler;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;

/**
 * 脚本控制器
 *
 * @author 王宇
 */
@Slf4j
@RestController
@RequestMapping("/v1/scripts")
@Tag(name = "脚本管理")
public class ScriptController {

	@Autowired
	private ScriptService scriptService;
	@Autowired
	private ScriptVersionService scriptVersionService;
	@Autowired
	private JobScheduler jobScheduler;

	/**
	 * 获取脚本列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本列表")
	public GeneralResult<GridBean<Script>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return scriptService.list(pager, context.getTarget());
	}

	/**
	 * 获取脚本列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping(value = "/simple")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本列表（简单）")
	public GeneralResult<GridBean<SimpleBean>> simple(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return scriptService.simple(pager, context.getTarget());
	}

	/**
	 * 添加脚本
	 *
	 * @param params
	 * @param context
	 * @return 添加结果
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本创建")
	public GeneralResult<Void> create(@RequestBody Script params, @Value(Common.REQ_CONTEXT) RequestContext context) {
		params.setCreatorId(context.getTarget());
		params.setMenderId(context.getTarget());
		return scriptService.create(params, context.getTarget());
	}

	/**
	 * 修改脚本
	 *
	 * @param params
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本修改")
	public GeneralResult<Void> modify(@RequestBody ScriptBean params, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		params.setId(id);
		return scriptService.modify(params, context.getTarget());
	}

	/**
	 * 移除脚本
	 *
	 * @param id
	 * @param context
	 * @return 操作结果
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return scriptService.remove(id, context.getTarget());
	}

	/**
	 * 查看脚本详细信息
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本详情")
	public GeneralResult<Script> detail(@PathVariable(value = Common.ID) Long id) {
		return scriptService.detail(id);
	}

	/**
	 * 导出脚本
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}/export")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本导出")
	public void export(@PathVariable(value = Common.ID) Long id, HttpServletRequest request,
			HttpServletResponse response) {
		GeneralResult<Script> result = scriptService.detail(id);
		if (result.isFailed()) {
			return;
		}
		String content = JSONObject.toJSONString(result.getData());
		JSONObject object = JSONObject.parseObject(content);
		response.setCharacterEncoding("utf-8");
		response.setContentType("multipart/form-data");
		try {
			String fileName = "脚本-" + object.getString("name") + "_" + System.currentTimeMillis() + ".txt";
			final String userAgent = request.getHeader("user-agent").toLowerCase();
			if (userAgent.contains("trident") || userAgent.contains("edge")) {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} else {
				fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
			}
			response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
			OutputStream os = response.getOutputStream();
			byte[] bytes = object.getString("content").getBytes();
			os.write(bytes);
			os.close();
			os.flush();
		} catch (UnsupportedEncodingException e1) {
			log.error("set code failure:", e1);
		} catch (IOException e) {
			log.error("Download fail:", e);
		}
	}

	/**
	 * 脚本操作
	 *
	 * @param id
	 * @param requestAction
	 * @param context
	 */
	@PatchMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本操作")
	public GeneralResult<Void> action(@PathVariable(value = Common.ID) Long id, @RequestBody JSONObject object,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		String action = object.getString("action");
		if ("switch".equalsIgnoreCase(action)) {
			return scriptService.modifyVersion(id, context.getTarget());
		} else if ("apply".equalsIgnoreCase(action)) {
			Script script = scriptService.get(id);
			if (null == script) {
				return new GeneralResult<>(false, "申请上传失败：脚本不存在");
			}
			GeneralResult<Void> result = scriptService.updateStatus(id, ApplyStatus.APPROVED.name(),
					context.getTarget());
			if (result.isFailed()) {
				return result;
			}
			return new GeneralResult<>(true, "申请上传脚本成功");
		}
		return new GeneralResult<>(false, "参数格式错误");
	}

	@PostMapping(value = "/fast")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本快速执行")
	public GeneralResult<Map<String, Object>> fast(@RequestBody FastTask fastTask,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		if (StringUtils.isEmpty(fastTask.getExecuteType())) {
			return new GeneralResult<>(false, "执行方式不能为空");
		}
		if (null == fastTask.getItemScripts() || fastTask.getItemScripts().isEmpty()) {
			return new GeneralResult<>(false, "脚本不能为空");
		}
		if (fastTask.getTarget() == null) {
			return new GeneralResult<>(false, "目标主机不能为空");
		}
		TaskInstance instance = scriptService.fastExecute(fastTask, context);
		if (null == instance) {
			return new GeneralResult<>(false, "执行失败：原因可能是作业节点分类参数格式错误");
		}
		Job job = new Job();
		job.setTaskId(instance.getUuid());
		job.setParam(Common.NAME, instance.getName());
		job.setParam(TaskParam.TASKCODE.name(), instance.getCode());
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
		job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
		job.setParam("QUEUE", fastTask.getCallback());
		boolean result = jobScheduler.insertTask(job);
		if (!result) {
			return new GeneralResult<>(false, "任务启动失败");
		}
		JSONObject data = new JSONObject();
		data.put("instanceId", instance.getId());
		data.put("instanceName", instance.getName());
		data.put("instanceExtension", instance.getExtension());
		return new GeneralResult<>(true, data, "启动成功");
	}

	/**
	 * 脚本版本列表查询
	 *
	 * @param id
	 * @param pager
	 * @return
	 */
	@GetMapping(value = "/{id}/versions")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本版本列表")
	public GeneralResult<GridBean<ScriptVersion>> versions(@PathVariable(value = Common.ID) Long id, Pager pager) {
		return scriptVersionService.list(pager);
	}

	/**
	 * 脚本版本列表查询
	 *
	 * @param id
	 * @param pager
	 * @return
	 */
	@GetMapping(value = "/{id}/versions/simple")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本版本列表（简单）")
	public GeneralResult<GridBean<SimpleBean>> simple(@PathVariable(value = Common.ID) Long id, Pager pager) {
		return scriptVersionService.simple(pager);
	}

	@PatchMapping(value = "/{id}/release")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本发布")
	public GeneralResult<Void> release(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return scriptService.updateStatus(id, ApplyStatus.APPROVED.name(), context.getTarget());
	}

	@PatchMapping(value = "/{id}/recall")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本撤回")
	public GeneralResult<Void> recall(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return scriptService.updateStatus(id, ApplyStatus.DEVELOPING.name(), context.getTarget());
	}
}
