package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.Software;
import com.bocloud.cop.entity.SoftwareTenant;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.intf.SoftwareService;
import com.bocloud.cop.model.SoftwareBean;
import com.bocloud.cop.service.utils.StorageConfig;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.jsonwebtoken.lang.Collections;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.util.Arrays;
import java.util.List;

/**
 * 文件控制器
 *
 * @author dmw
 */
@RestController
@RequestMapping("/v1/softwares")
@Tag(name = "文件管理")
public class SoftwareController {

	@Autowired
	private SoftwareService softwareService;
	@Autowired
	private StorageConfig storageConfig;

	/**
	 * 获取文件文件列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "文件列表")
	public GeneralResult<GridBean<Software>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		if (com.megatron.common.model.RequestContext.Catalog.Tenant == context.getCatalog()
				|| com.megatron.common.model.RequestContext.Catalog.User == context.getCatalog()) {
			return softwareService.listAllot(pager, context);
		}
		return softwareService.list(pager);
	}

	/**
	 * 获取文件文件列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping(value = "/simple")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件列表（简单）")
	public GeneralResult<GridBean<SimpleBean>> simple(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return softwareService.simple(pager);
	}

	/**
	 * 添加文件文件
	 *
	 * @param software
	 * @param context
	 * @return 添加结果
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "文件创建")
	public GeneralResult<Software> create(@RequestBody Software software,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		software.setCreatorId(context.getTarget());
		software.setMenderId(context.getTarget());
		return softwareService.create(software);
	}

	/**
	 * 修改文件文件
	 *
	 * @param bean
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件修改")
	public GeneralResult<SoftwareBean> modify(@RequestBody SoftwareBean bean, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return softwareService.modify(bean, context.getTarget());
	}

	/**
	 * 移除文件文件
	 *
	 * @param id
	 * @param context
	 * @return 操作结果
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件删除")
	public GeneralResult<String> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return softwareService.remove(id, context.getTarget());
	}

	/**
	 * 查看文件文件详细信息
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件详情")
	public GeneralResult<Software> detail(@PathVariable(value = Common.ID) Long id) {
		return softwareService.detail(id);
	}

	/**
	 * 判断是否存在
	 *
	 * @param name
	 * @param context
	 * @return
	 */
	@GetMapping(value = "/{name}/exists")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件名称是否存在")
	public GeneralResult<Software> fileExists(@PathVariable(value = Common.NAME) String name,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return softwareService.fileNotExists(name);
	}

	/**
	 * 检查文件是否存在
	 *
	 * @param paths
	 * @return
	 */
	@GetMapping(value = "/check")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件是否存在")
	public GeneralResult<Void> checkFile(@RequestParam(value = "path") String paths) {
		if (StringUtils.isBlank(paths)) {
			return new GeneralResult<>(false, "检查文件失败,没有需要下载的文件!");
		}
		List<String> pathList = Arrays.asList(paths.split(","));
		for (String path : pathList) {
			String filepath = this.storageConfig.getPath() + path;
			File file = new File(filepath);
			if (!file.exists()) {
				return new GeneralResult<>(false, "检查文件失败,文件[" + path + "]可能不存在!");
			}
		}
		return new GeneralResult<>(true, "检查文件成功！");
	}

	/**
	 * 文件下载
	 *
	 * @param path
	 * @param response
	 * @return
	 */
	@GetMapping(value = "/download")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件下载")
	public void download(@RequestParam(value = "path") String path, HttpServletRequest request,
			HttpServletResponse response) {
		List<String> pathList = Arrays.asList(path.split(","));
		if (!Collections.isEmpty(pathList)) {
			if (pathList.size() == 1) {
				softwareService.singleDownload(path, request, response);
			} else {
				softwareService.compressDownload(pathList, response);
			}
		}
	}

	@PatchMapping(value = "/{id}/release")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件发布")
	public GeneralResult<Void> release(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return softwareService.updateStatus(id, ApplyStatus.APPROVED.name(), context.getTarget());
	}

	@PatchMapping(value = "/{id}/recall")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件撤回")
	public GeneralResult<Void> recall(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return softwareService.updateStatus(id, ApplyStatus.DEVELOPING.name(), context.getTarget());
	}

	@PutMapping(value = "/{tenantId}/allot")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件分配")
	public GeneralResult<Void> allot(@RequestBody SoftwareTenant params,
			@PathVariable(value = "tenantId") Long tenantId, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return softwareService.allot(tenantId, params.getSoftwareIds());
	}

	@GetMapping(value = "/{tenantId}/allots")
	@Operation(tags = { "CMC", "CSC" }, summary = "已分配文件列表")
	public GeneralResult<List<Software>> listAllotSoftwares(@PathVariable(value = "tenantId") Long tenantId,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return softwareService.listAllotSoftwares(tenantId);
	}
}
