package com.bocloud.cop.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.ExecuteMode;
import com.bocloud.cop.enums.TaskParam;
import com.bocloud.cop.intf.TaskHostService;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.model.TaskHostBean;
import com.bocloud.cop.model.TaskHostConfigBean;
import com.bocloud.cop.service.utils.JobScheduler;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/v1/hosts")
@Tag(name = "主机资源管理")
public class TaskHostController {

	@Autowired
	TaskHostService taskHostService;
	@Autowired
	private TaskInstanceService taskInstanceService;
	@Autowired
	private JobScheduler jobScheduler;

	/**
	 * 获取主机资源列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源列表")
	public GeneralResult<GridBean<TaskHost>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskHostService.list(pager, context);
	}

	/**
	 * 获取主机资源列表以及认证账号
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping(value = "/target")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源列表以及认证账号")
	public GeneralResult<GridBean<TaskHost>> listTaskHostAndIdentification(Pager pager,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskHostService.listTaskHostAndIdentification(pager, context);
	}

	/**
	 * 获取主机资源列表(用于配置权限)
	 *
	 * @param pager
	 * @return 数据结果集
	 */
	@GetMapping("/manage")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源列表（用于配置权限）")
	public GeneralResult<GridBean<TaskHost>> listForManage(Pager pager) {
		return taskHostService.listForManage(pager);
	}

	/**
	 * 添加
	 *
	 * @param taskHost
	 * @param context
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源创建")
	public GeneralResult<Void> create(@RequestBody TaskHost taskHost,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskHostService.create(taskHost, context);
	}

	/**
	 * 修改
	 *
	 * @param
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源修改")
	public GeneralResult<Void> modify(@RequestBody TaskHostBean taskHost, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskHostService.modify(taskHost, context);
	}

	/**
	 * 查看主机资源详细信息
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源详情")
	public GeneralResult<TaskHost> detail(@PathVariable(value = Common.ID) Long id) {
		return taskHostService.detail(id);
	}

	/**
	 * 删除主机资源
	 *
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id) {
		return taskHostService.remove(id);
	}

	/**
	 * 主机配置网段信息
	 *
	 * @param
	 * @param segmentId
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}/configure/{segmentId}")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源网段配置")
	public GeneralResult<Void> configure(@PathVariable Long id, @PathVariable Long segmentId,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskHostService.configure(id, segmentId);
	}

	/**
	 * 主机配置worker组操作
	 *
	 * @param hostId
	 * @param context
	 * @return
	 */
	// @PostMapping(value = "/{hostId}/allow/config")
	// @Operation(tags = { "CMC", "CSC" }, summary = "主机配置worker组操作")
	public GeneralResult<?> config(@PathVariable Long hostId, @Value(Common.REQ_CONTEXT) RequestContext context) {
		GeneralResult<Task> result = taskHostService.configHostAllow(hostId);
		if (result.isFailed()) {
			return result;
		}
		Task task = result.getData();
		TaskInstance instance = taskInstanceService.initInstance(task, ExecuteMode.WEB.name(), context);
		if (null == instance) {
			return new GeneralResult<>(false, "执行失败：原因可能是作业节点分类参数格式错误");
		}
		boolean flag = jobScheduler.insertTask(buildJob(instance));
		return flag ? new GeneralResult<>(true, instance.getId(), "主机HostsAllow配置任务下发成功")
				: new GeneralResult<>(false, "主机HostsAllow配置任务下发失败");
	}

	private Job buildJob(TaskInstance instance) {
		Job job = new Job();
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
		job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
		job.setTaskId(instance.getUuid());
		job.setParam(Common.NAME, instance.getName());
		job.setParam(TaskParam.TASKCODE.name(), instance.getCode());
		return job;
	}

	/**
	 * 同步
	 *
	 * @return
	 */
	@PostMapping(value = "/sync")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源同步")
	public GeneralResult<?> sync(@Value(Common.REQ_CONTEXT) RequestContext context) {
		taskHostService.syncHostFromIMS(context, null);
		return new GeneralResult<>(true, "同步任务已下发，请等待……");
	}

	@PostMapping(value = "/upload")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源导入")
	public GeneralResult<Void> upload(HttpServletRequest request, HttpServletResponse response,
			@Value(Common.REQ_CONTEXT) RequestContext context) throws IOException {
		try {
			MultipartHttpServletRequest multipartHttpServletRequest = (MultipartHttpServletRequest) request;
			MultiValueMap<String, MultipartFile> multiFileMap = multipartHttpServletRequest.getMultiFileMap();
			MultipartFile item = null;
			if (multiFileMap.values().size() > 0) {
				List<MultipartFile> multiFileList = multiFileMap.values().iterator().next();
				item = multiFileList.get(0);
			}
			return taskHostService.upload(item, context);
		} catch (Exception e) {
			log.error("上传失败", e);
		}
		return new GeneralResult<>(false, "文件错误");
	}

	@GetMapping(value = "/template/download")
	@Operation(tags = { "CMC", "CSC" }, summary = "下载导入文件模板")
	public void download(@RequestParam(value = Common.PARAMS, required = false) String params,
			HttpServletRequest request, HttpServletResponse response) {
		Map<String, Object> paramsMap = new HashMap<>();
		if (null != params) {
			JSONObject.parseObject(params, HashMap.class);
		}
		GeneralResult<?> result = taskHostService.download(paramsMap);
		if (result.isFailed()) {
			return;
		}
		try {
			@SuppressWarnings("unchecked")
			Map<String, Object> map = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), HashMap.class);
			response.setCharacterEncoding("utf-8");
			response.setContentType("application/vnd.ms-excel");
			String fileName = map.get("fileName").toString();
			final String userAgent = request.getHeader("user-agent").toLowerCase();
			if (userAgent.contains("trident") || userAgent.contains("edge")) {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} else {
				fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
			}
			response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
			OutputStream os = response.getOutputStream();
			String content = (String) map.get("data");
			if (content != null) {
				byte[] bytes = Base64.getDecoder().decode(content);
				os.write(bytes);
				os.close();
				os.flush();
			}
		} catch (Exception e) {
			log.error("下载失败");
		}
	}

	/**
	 * 权限设置
	 *
	 * @param configBean
	 * @return
	 */
	@PostMapping(value = "/permission")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源权限设置")
	public GeneralResult<Void> permission(@RequestBody TaskHostConfigBean configBean) {
		return taskHostService.configPermissionRelation(configBean.getIds(), configBean.getUserGroupIds());
	}

	/**
	 * 关联账号
	 *
	 * @param configBean 权限配置信息
	 * @return 配置结果
	 */
	@PostMapping(value = "/identification")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机资源关联账号")
	public GeneralResult<Void> identification(@RequestBody TaskHostConfigBean configBean) {
		return taskHostService.configIdentificationRelation(configBean.getIds(), configBean.getIdentificationIds());
	}
}
