package com.bocloud.cop.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.ExecuteMode;
import com.bocloud.cop.enums.TaskParam;
import com.bocloud.cop.intf.TaskInstanceApiService;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.intf.TaskPlatformConfigService;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.model.TaskPlatformModel;
import com.bocloud.cop.model.params.ScriptTaskParams;
import com.bocloud.cop.service.utils.DateUtils;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * 作业实例控制器
 *
 * @author weiwei
 */
@Slf4j
@RestController
@RequestMapping("/v1/tasks/instances/api")
@Tag(name = "作业实例Api管理")
public class TaskInstanceApiController {

	@Autowired
	private TaskInstanceApiService taskInstanceApiService;
	@Autowired
	private TaskPlatformConfigService taskPlatformConfigService;
	@Autowired
	private TaskInstanceService taskInstanceService;

	@GetMapping("/result")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例结果")
	public GeneralResult<?> judge(@RequestParam(required = true) String instanceId) {
		return taskInstanceApiService.getResult(instanceId);
	}

	@PostMapping("/task")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例创建执行")
	public GeneralResult<?> task(@RequestBody ScriptTaskParams params) {
		long startTime = System.currentTimeMillis();
		log.info("收到信息时间+++++++++++++++++++++++++" + DateUtils.dateToString(new Date(), "yyyy-MM-dd HH:mm:ss"));
		log.info("调用执行作业操作接口参数：" + JSONObject.toJSONString(params));
		GeneralResult<Task> taskResult = null;
		if ("reboot".equalsIgnoreCase(params.getTaskType())) {
			taskResult = taskInstanceApiService.reboot(params);
		} else if ("template".equalsIgnoreCase(params.getTaskType())) {
			// 根据playbook构建对应任务
			taskResult = taskInstanceApiService.taskTemplate(params);
		} else {
			// 构建脚本类任务
			taskResult = taskInstanceApiService.task(params);
		}

		if (taskResult.isFailed()) {
			log.warn("用户【" + params.getUserNo() + "】请求作业构建失败，失败原因为：" + taskResult.getMessage());
			return taskResult;
		}
		log.info("生成任务task用时：" + (System.currentTimeMillis() - startTime));
		Task task = taskResult.getData();
		if (null != task) {
			task.setPlatform(params.getPlatform());
			task.setPriority(params.getPriority());
			task.setUntreate(params.isUntreate());
		}
		// 创建任务实例
		RequestContext context = new RequestContext(RequestContext.Catalog.Manager, 1L, null, null, null);
		TaskInstance instance = taskInstanceService.initInstance(task, ExecuteMode.API.name(), context);
		if (null == instance) {
			log.warn("用户【" + params.getUserNo() + "】请求作业执行失败，失败原因为：原因可能是作业节点分类参数格式错误");
			return new GeneralResult<>(false, "执行失败：原因可能是作业节点分类参数格式错误");
		}
		log.info("生成实例instance用时：" + (System.currentTimeMillis() - startTime));
		if (!params.isUntreate()) {
			// 校验是否需要限流
			this.putJob(instance, params.getPriority());
		}
		log.info("用户【" + params.getUserNo() + "】请求作业执行下发成功，任务号为【" + instance.getUuid() + "】");
		return new GeneralResult<>(true, instance.getUuid(), "作业任务下发成功");
	}

	private GeneralResult<?> putJob(TaskInstance instance, Integer priority) {
		GeneralResult<?> result;
		// 任务是否放行
		boolean released = true;
		TaskPlatformModel model = null;
		String dealMethod = "";
		Long platformAlertId = null;

		// 平台限流判断
		if (StringUtils.isNotEmpty(instance.getPlatform())) {
			/**
			 * 判断任务是否放行：
			 *
			 * 目前只会读取缓存中的限流配置，又由于缓存的设置是在定时任务中执行的，所以 所以平台告警配置过后立即执行任务即使满足告警规则也并不会触发告警信息，
			 * 因为需要等待一分钟之后执行定时任务才会把告警配置加载到缓存中，此处方法执行才会读取到缓存中的数据并做校验。
			 */
			GeneralResult<TaskPlatformModel> platformResult = taskPlatformConfigService
					.chargeLimit(instance.getPlatform(), instance.getTargetHostNum());
			if (platformResult.isSuccess()) {
				model = platformResult.getData();
				if (null != model && model.isLimit()) {
					released = false;
					dealMethod = model.getDealMethod();
					platformAlertId = model.getPlatformAlertId();
				}
			}
		}
		if (released) {
			// 根据任务实例生成job工作项
			result = taskInstanceService.putJob(buildJob(instance, priority));
		} else {
			// 更新taskInstance.is_limited为1
			result = taskInstanceService.updateLimited(instance.getId(), 1L, dealMethod, platformAlertId);
		}
		return result;
	}

	private Job buildJob(TaskInstance instance, Integer priority) {
		Job job = new Job();
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
		job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
		job.setTaskId(instance.getUuid());
		job.setParam(Common.NAME, instance.getName());
		job.setParam(TaskParam.TASKCODE.name(), instance.getCode());
		if (null != priority) {
			job.setPriority(priority);
		}
		return job;
	}

}
