package com.bocloud.cop.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskTemplate;
import com.bocloud.cop.entity.TaskTemplateTenant;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.intf.TaskTemplateService;
import com.bocloud.cop.model.TaskGraphBean;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.List;

/**
 * 作业模板控制器
 *
 * @author weiwei
 */
@Slf4j
@RestController
@RequestMapping("/v1/tasks/templates")
@Tag(name = "作业模板管理")
public class TaskTemplateController {

	@Autowired
	private TaskTemplateService taskTemplateService;

	/**
	 * 获取作业模板列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "作业模板列表")
	public GeneralResult<GridBean<TaskTemplate>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskTemplateService.list(pager, context);
	}

	/**
	 * (自服务）获取公共作业模板列表
	 *
	 * @param pager
	 * @param context
	 * @return
	 */
	@GetMapping(value = "/allots")
	@Operation(tags = { "CMC", "CSC" }, summary = "公共作业模板列表")
	public GeneralResult<GridBean<TaskTemplate>> listAllot(Pager pager,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskTemplateService.listAllot(pager, context);
	}

	/**
	 * 获取作业模板列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping(value = "/simple")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业模板列表（简单）")
	public GeneralResult<GridBean<SimpleBean>> simple(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskTemplateService.simple(pager);
	}

	/**
	 * 添加作业模板
	 *
	 * @param params
	 * @param context
	 * @return 添加结果
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "作业模板创建")
	public GeneralResult<TaskTemplate> create(@RequestBody Object params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		TaskTemplate template = JSONObject.parseObject(JSONObject.toJSONString(params), TaskTemplate.class);
		return taskTemplateService.create(template, context);
	}

	/**
	 * 修改定时作业模板
	 *
	 * @param params
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业模板修改")
	public GeneralResult<TaskTemplate> modify(@RequestBody TaskTemplate template,
			@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskTemplateService.modify(template, context);
	}

	/**
	 * 移除作业模板
	 *
	 * @param id
	 * @param context
	 * @return 操作结果
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业模板删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskTemplateService.remove(id, context.getTarget());
	}

	/**
	 * 查看作业模板详细信息
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业模板详情")
	public GeneralResult<TaskTemplate> detail(@PathVariable(value = Common.ID) Long id) {
		return taskTemplateService.detail(id);
	}

	/**
	 * 导出作业模板
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}/export")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业模板导出")
	public void export(@PathVariable(value = Common.ID) Long id, HttpServletRequest request,
			HttpServletResponse response) {
		TaskTemplate template = taskTemplateService.basic(id);
		if (null == template) {
			return;
		}
		template.setGraph(JSONObject.parseObject(template.getGraph().toString(), TaskGraphBean.class));
		String content = JSONObject.toJSONString(template);
		response.setCharacterEncoding("utf-8");
		response.setContentType("multipart/form-data");
		try {
			String fileName = "task_" + System.currentTimeMillis() + ".txt";
			final String userAgent = request.getHeader("user-agent").toLowerCase();
			if (userAgent.contains("trident") || userAgent.contains("edge")) {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} else {
				fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
			}
			response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
			OutputStream os = response.getOutputStream();
			byte[] bytes = content.getBytes();
			os.write(bytes);
			os.close();
			os.flush();
		} catch (Exception e) {
			log.error("Download fail:", e);
		}
	}

	@PatchMapping(value = "/{id}/release")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业模板发布")
	public GeneralResult<Void> release(@PathVariable(value = Common.ID) Long id,
			@RequestParam(value = Common.STARTMODE, required = false) String startMode,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		TaskTemplate template = taskTemplateService.basic(id);
		if (null == template) {
			return new GeneralResult<>(false, "发布失败：作业模板不存在");
		}
		GeneralResult<Void> result = taskTemplateService.updateStatus(template.getId(), ApplyStatus.APPROVED.name(),
				context.getTarget());
		return new GeneralResult<>(result.isSuccess(), "申请发布作业模板" + (result.isSuccess() ? "成功" : "失败"));
	}

	@PatchMapping(value = "/{id}/clone")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业模板克隆")
	public GeneralResult<TaskTemplate> clone(@PathVariable(value = Common.ID) Long id) {
		return taskTemplateService.clone(id);
	}

	@PatchMapping(value = "/{id}/recall")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业模板撤回")
	public GeneralResult<Void> recall(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		TaskTemplate template = taskTemplateService.basic(id);
		if (null == template) {
			return new GeneralResult<>(false, "撤回失败：作业模板不存在");
		}
		GeneralResult<Void> result = taskTemplateService.updateStatus(template.getId(), ApplyStatus.DEVELOPING.name(),
				context.getTarget());
		return new GeneralResult<>(result.isSuccess(), "申请撤回作业模板" + (result.isSuccess() ? "成功" : "失败"));
	}

	@PutMapping(value = "/{tenantId}/allot")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业模板分配")
	public GeneralResult<Void> allot(@RequestBody TaskTemplateTenant params,
			@PathVariable(value = "tenantId") Long tenantId, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskTemplateService.allot(tenantId, params.getTemplateIds());
	}

	@GetMapping(value = "/{tenantId}/allots")
	@Operation(tags = { "CMC", "CSC" }, summary = "租户已分配作业模板列表")
	public GeneralResult<List<TaskTemplate>> listAllotTemplates(@PathVariable(value = "tenantId") Long tenantId,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskTemplateService.listAllotTemplates(tenantId);
	}

}
