package com.bocloud.cop.booter.controller.inspection;

import com.bocloud.cop.entity.inspection.InspectReport;
import com.bocloud.cop.intf.inspection.InspectReportService;
import com.bocloud.cop.model.OperationParam;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.web.bind.annotation.*;

/**
 * 巡检报表Controller
 *
 * @author jiaohongtao
 * @version 1.0
 * @since 2020-10-30 16:51:21
 */
@RestController
@RequestMapping("/v1/inspect/reports")
@Tag(name = "巡检报表管理")
public class InspectReportController implements ApplicationListener<ApplicationReadyEvent> {

	@Autowired
	private InspectReportService inspectReportService;

	/**
	 * 查询巡检报表
	 *
	 * @param pager
	 * @return
	 */
	@GetMapping()
	@Operation(tags = { "CMC" }, summary = "巡检报表列表")
	public GeneralResult<GridBean<InspectReport>> list(Pager pager) {
		return inspectReportService.list(pager);
	}

	/**
	 * 创建巡检报表
	 *
	 * @param inspectReport
	 * @param user
	 * @return 属性信息的json字符串，数据转换发生异常时，对外抛出400异常【Bad Request】
	 */
	@PostMapping()
	@Operation(tags = { "CMC" }, summary = "巡检报表创建")
	public GeneralResult<Void> create(@RequestBody InspectReport inspectReport,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectReportService.create(inspectReport, user.getTarget());
	}

	/**
	 * 修改巡检报表
	 *
	 * @param inspectReport
	 * @param id
	 * @param user
	 * @return 属性信息的json字符串，数据转换发生异常时，对外抛出400异常【Bad Request】
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "巡检报表修改")
	public GeneralResult<Void> modify(@RequestBody InspectReport inspectReport,
			@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectReportService.modify(id, inspectReport, user.getTarget());
	}

	/**
	 * 删除巡检报表
	 *
	 * @param id
	 * @param user
	 * @return
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "巡检报表删除")
	public GeneralResult<InspectReport> remove(@PathVariable(Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectReportService.remove(id, user.getTarget());
	}

	/**
	 * 查询巡检报表详情
	 *
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "巡检报表详情")
	public GeneralResult<InspectReport> detail(@PathVariable(Common.ID) Long id) {
		return inspectReportService.detail(id);
	}

	/**
	 * 操作巡检报表
	 *
	 * @param id
	 * @param action
	 * @param param
	 * @param user
	 * @return
	 */
	@PatchMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "巡检报表操作")
	public GeneralResult<Void> start(@PathVariable(Common.ID) Long id,
			@RequestParam(value = Common.ACTION, required = false) String action,
			@RequestBody(required = false) OperationParam param, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectReportService.start(id, action, param, user);
	}

	/**
	 * 为巡检报表添加任务(主服务有效)
	 *
	 * @param id
	 * @return
	 */
	@PostMapping(value = "/{id}/schedule")
	@Operation(tags = { "CMC" }, summary = "巡检报表添加任务(主服务有效)")
	public GeneralResult<String> addScheduleIfLeader(@PathVariable(Common.ID) Long id) {
		return inspectReportService.addScheduleIfLeader(id);
	}

	/**
	 * 为巡检报表删除任务(主服务有效
	 *
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/{id}/schedule")
	@Operation(tags = { "CMC" }, summary = "巡检报表删除任务(主服务有效)")
	public GeneralResult<String> removeScheduleIfLeader(@PathVariable(Common.ID) Long id) {
		return inspectReportService.removeScheduleIfLeader(id);
	}

	/**
	 * 初始化方法
	 *
	 * @param event
	 */
	@Override
	public void onApplicationEvent(ApplicationReadyEvent event) {
		inspectReportService.onApplicationReadyEvent();
	}

}
