package com.bocloud.cop.booter.scheduler;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskPlatform;
import com.bocloud.cop.entity.TaskPlatformAlert;
import com.bocloud.cop.entity.TaskPlatformConfig;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.TaskParam;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.intf.TaskPlatformConfigService;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.repository.TaskInstanceRepository;
import com.bocloud.cop.repository.TaskPlatformAlertRepository;
import com.bocloud.cop.repository.TaskPlatformConfigRepository;
import com.bocloud.cop.repository.TaskPlatformRepository;
import com.bocloud.cop.task.TaskPlatformConfigTask;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.megatron.common.utils.Common;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 定时处理平台任务指标数据
 *
 * @author majiali
 */
@Slf4j
@Component
public class TaskPlatformLimitScheduler implements InitializingBean {

    @Autowired
    private CurrentService service;
    @Autowired
    private TaskPlatformConfigRepository taskInstancePlatformRepository;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private TaskInstanceService taskInstanceService;
    @Autowired
    private TaskInstanceRepository taskInstanceRepository;
    @Autowired
    private TaskPlatformRepository taskPlatformRepository;
    @Autowired
    private TaskPlatformAlertRepository taskPlatformAlertRepository;
    @Autowired
    private TaskPlatformConfigService taskPlatformConfigService;
    private ExecutorService executor;

    @Scheduled(cron = "${task.instance.platform.schedule:0 0/1 * * * ?}")
    private void countIndex() {
        if (!service.isLeader()) {
            return;
        }
        try {
            boolean limitSwitch = taskPlatformConfigService.limitSwitch();
            if (!limitSwitch) {
                log.info("限流任务未开启");
                limitSwitchNotOpen();
                return;
            }
            SimpleDateFormat df = new SimpleDateFormat("HH:mm");
            String currTime = df.format(new Date());
            List<TaskPlatform> platforms = taskPlatformRepository.listAll();
            Set<String> platformSet = platforms.stream().map(TaskPlatform::getName).collect(Collectors.toSet());
            //告警配置
            List<TaskPlatformConfig> list = taskInstancePlatformRepository.getByTime(currTime);
            if (CollectionUtils.isNotEmpty(list)) {
                //'CREATED','PROCESSED' 新创建、已处理 的告警数据
                List<TaskPlatformAlert> listAlert = taskPlatformAlertRepository.getByTime(currTime);
                Map<Long, List<TaskPlatformAlert>> map =
                        listAlert.stream().collect(Collectors.groupingBy(TaskPlatformAlert::getPlatformConfigId));
                //设置每个告警配置对应的告警数据
                for (TaskPlatformConfig config : list) {
                    config.setAlertList(map.get(config.getId()));
                }
                Map<String, List<TaskPlatformConfig>> configMap =
                        list.stream().collect(Collectors.groupingBy(TaskPlatformConfig::getPlatform));
                for (String key : configMap.keySet()) {
                    platformSet.remove(key);
                    TaskPlatformConfigTask task = new TaskPlatformConfigTask(currTime, configMap.get(key), redisTemplate,
                            amqpTemplate, lockFactory, taskInstanceService, taskInstanceRepository);
                    executor.execute(task);
                }
            }
            //如果存在当前时间没有告警配置的平台，需要放行平台排队的任务
            if (CollectionUtils.isNotEmpty(platformSet)) {
                executor.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            for (String key : platformSet) {
                                redisTemplate.delete("platform_" + key);
                                List<TaskInstance> instanceList = taskInstanceRepository.listLimited(key, null);
                                for (TaskInstance instance : instanceList) {
                                    // 还要判断任务数及目标主机数是否超出限制
                                    instance.setIsLimited(false);
                                    instance.setLimitedReleaseTime(new Date());
                                    instance.setMenderId(1L);
                                    taskInstanceRepository.updateLimitedRelease(instance.getId(), 1L);
                                    taskInstanceService.putJob(buildJob(instance, instance.getPriority()));
                                }
                            }
                        } catch (Exception e) {
                            log.error("平台限流当前周期不限流平台任务处理失败", e);
                        }
                    }
                });
            }
        } catch (Exception e) {
            log.error("平台限流信息任务下发失败", e);
        }
    }

    private void limitSwitchNotOpen() throws Exception {
        // 删除缓存中已有限流配置信息，如果有暂缓执行的任务需要放开执行
        List<TaskPlatform> list = taskPlatformRepository.listAll();
        if (CollectionUtils.isNotEmpty(list)) {
            for (TaskPlatform platform : list) {
                redisTemplate.delete("platform_" + platform.getName());
            }
        }
        List<TaskInstance> instanceList = taskInstanceRepository.listLimited(null, null);
        for (TaskInstance instance : instanceList) {
            // 还要判断任务数及目标主机数是否超出限制
            instance.setIsLimited(false);
            instance.setLimitedReleaseTime(new Date());
            instance.setMenderId(1L);
            taskInstanceRepository.updateLimitedRelease(instance.getId(), 1L);
            taskInstanceService.putJob(buildJob(instance, instance.getPriority()));
        }
    }

    private Job buildJob(TaskInstance instance, Integer priority) {
        Job job = new Job();
        job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
        job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
        job.setTaskId(instance.getUuid());
        job.setParam(Common.NAME, instance.getName());
        job.setParam(TaskParam.TASKCODE.name(), instance.getCode());
        if (null != priority) {
            job.setPriority(priority);
        }
        return job;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        int processors = Runtime.getRuntime().availableProcessors();
        executor = new ThreadPoolExecutor(processors + 1, processors * 100, 60L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(1000), new ThreadFactoryBuilder().build());
    }

}
