package com.bocloud.cop.booter.scheduler;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.constant.CacheConstant;
import com.bocloud.cop.core.WorkerCache;
import com.bocloud.cop.entity.NetworkSegment;
import com.bocloud.cop.entity.TaskWorker;
import com.bocloud.cop.repository.NetworkSegmentRepository;
import com.bocloud.cop.repository.TaskWorkerRepository;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Slf4j
@Component
public class WorkerCacheScheduler implements InitializingBean {

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private TaskWorkerRepository taskWorkerRepository;
    @Autowired
    private NetworkSegmentRepository networkSegmentRepository;
    @Autowired
    private CurrentService service;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private WorkerCache workerCache;

    @Scheduled(cron = "0 0 */1 * * ?")
    private void refreshCache() {
        writeCache();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        writeCache();
    }

    private void writeCache() {
        if (!service.isLeader()) {
            return;
        }
        try {
            // 获取worker列表
            List<TaskWorker> ansibleWorkers = taskWorkerRepository.listWorkers(TaskWorker.Category.Ansible.name());
            List<TaskWorker> saltWorkers = taskWorkerRepository.listWorkers(TaskWorker.Category.SaltMaster.name());
            Map<Long, List<String>> workerMap = new HashMap<>();
            Map<Long, List<TaskWorker>> saltWorkerMap = new HashMap<>();
            List<String> superWorkers = new ArrayList<>();
            // 按group存放ansibleInstance
            this.groupMap(workerMap, superWorkers, ansibleWorkers);
            // 按group存放saltInstance
            this.saltGroupMap(saltWorkerMap, saltWorkers);
            // 刷新 ansible 缓存
            this.refreshCache(workerMap);
            // 刷新 salt 缓存
            this.refreshSaltCache(saltWorkers);
            this.refreshSaltGroupCache(saltWorkerMap);
            // 处理超级worker缓存
            if (!superWorkers.isEmpty()) {
                for (String superWorker : superWorkers) {
                    workerCache.addCache(CacheConstant.SUPER_WORKERS, superWorker);
                }
            }
            // 处理网段分组缓存
            List<NetworkSegment> segments = networkSegmentRepository.list();
            JSONObject segObject = new JSONObject();
            for (NetworkSegment segment : segments) {
                segObject.put(segment.getId() + "", segment.getName());
                if (null == segment.getGroupId()) {
                    continue;
                }
                String key = CacheConstant.SEGMENT_ + segment.getId();
                try (AutoCloseLock lock = lockFactory.getACLock(key)) {
                    Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
                    redisTemplate.opsForValue().set(key, String.valueOf(segment.getGroupId()));
                } catch (Exception e) {
                    log.error("配置网段分组缓存失败:", e);
                }
            }
            try (AutoCloseLock lock = lockFactory.getACLock(CacheConstant.SEGMENTS)) {
                Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
                redisTemplate.opsForValue().set(CacheConstant.SEGMENTS, segObject.toJSONString());
            } catch (Exception e) {
                log.error("配置网段缓存失败:", e);
            }
        } catch (Exception e) {
            log.error("配置worker负载缓存失败：", e);
        }
    }

    /**
     * 按group存放数据
     * 
     * @param map
     * @param workers
     */
    private void groupMap(Map<Long, List<String>> map, List<String> superWorkers, List<TaskWorker> workers) {
        for (TaskWorker worker : workers) {
            if (worker.getIsSuper()) {
                superWorkers.add(worker.getInstance());
                continue;
            }
            if (null == worker.getGroupId()) {
                continue;
            }
            if (map.containsKey(worker.getGroupId())) {
                map.get(worker.getGroupId()).add(worker.getInstance());
            } else {
                List<String> instances = new ArrayList<>();
                instances.add(worker.getInstance());
                map.put(worker.getGroupId(), instances);
            }
        }
    }

    /**
     * 分组存放salt数据
     * @param map
     * @param workers
     */
    private void saltGroupMap(Map<Long, List<TaskWorker>> map, List<TaskWorker> workers) {
        for (TaskWorker worker : workers) {
            if (null == worker.getGroupId()) {
                continue;
            }
            if (map.containsKey(worker.getGroupId())) {
                map.get(worker.getGroupId()).add(worker);
            } else {
                List<TaskWorker> instances = new ArrayList<>();
                instances.add(worker);
                map.put(worker.getGroupId(), instances);
            }
        }
    }

    /**
     * 刷新缓存
     * @param map
     * key   是 worker对应的分组id
     * value 是 分组内worker实例
     */
    private void refreshCache(Map<Long, List<String>> map) {
        Iterator<Long> iterator = map.keySet().iterator();
        while (iterator.hasNext()) {
            Long key = iterator.next();
            // 先获取redis上存在的worker负载信息，然后更新负载信息
            List<String> workers = map.get(key);
            for (String worker : workers) {
                workerCache.addCache(CacheConstant.WORKERGROUP_ + key, worker);
            }

        }
    }

    /**
     * 刷新缓存
     * @param workers
     */
    private void refreshSaltCache(List<TaskWorker> workers) {
        for (TaskWorker worker : workers) {
            workerCache.addSaltCache(CacheConstant.SALTMASTER_ + worker.getId(), Arrays.asList(worker));
        }
    }

    /**
     * 刷新缓存
     * @param map
     */
    private void refreshSaltGroupCache(Map<Long, List<TaskWorker>> map) {
        Iterator<Long> iterator = map.keySet().iterator();
        while (iterator.hasNext()) {
            Long key = iterator.next();
            // 先获取redis上存在的worker负载信息，然后更新负载信息
            workerCache.addSaltCache(CacheConstant.WORKERGROUP_SALT_ + key, map.get(key));
        }
    }
}
