package com.bocloud.cop.model.inspection;

import java.math.BigDecimal;

import lombok.Data;

@Data
public class WarningRule {
    private RULE rule;
    private String threshold;
    private int warnLevel;
    private String description;
    private String color;

    public WarningRule(RULE rule, String threshold, int warnLevel, String description, String color) {
        this.rule = rule;
        this.threshold = threshold;
        this.warnLevel = warnLevel;
        this.description = description;
        this.color = color;
    }

    public enum RULE {
        GT("大于", (value, threshold) -> {
            return numberMatch(value, threshold, (x, y) -> x.compareTo(y) > 0, true);
        }), EQ("等于", (value, threshold) -> {
            boolean match = value.equals(threshold);
            if (match) {
                return true;
            } else {
                return numberMatch(value, threshold, (x, y) -> x.compareTo(y) == 0, false);
            }
        }), NEQ("不等于", (value, threshold) -> {
            // 判断字符串是否相等
            boolean notEqual = !value.equals(threshold);
            if (notEqual) {
                // 判断数字是否相等
                return !numberMatch(value, threshold, (x, y) -> x.compareTo(y) == 0, false);
            } else {
                return false;
            }
        }), LT("小于", (value, threshold) -> {
            return numberMatch(value, threshold, (x, y) -> {
                int a = x.compareTo(y);
                return a < 0;
            }, true);
        }), GE("大于等于", (value, threshold) -> {
            return numberMatch(value, threshold, (x, y) -> x.compareTo(y) >= 0, true);
        }), LE("小与等于", (value, threshold) -> {
            return numberMatch(value, threshold, (x, y) -> x.compareTo(y) <= 0, true);
        }), IN("包含于", (value, threshold) -> {
            return value.contains(threshold);
        }), NIN("不包含", (value, threshold) -> {
            return !value.contains(threshold);
        });

        private String name;
        private Validator validator;

        RULE(String name, Validator validator) {
            this.name = name;
            this.validator = validator;
        }

        public WarningRuleResult validate(String value, WarningRule warningRule) {
            if (value == null) {
                value = "";
            }
            String threshold = warningRule.getThreshold();
            if (threshold == null) {
                return WarningRuleResult.error("对应阈值【" + threshold + "】为【null】,请检查阈值设置", warningRule);
            }
            try {
                boolean result = this.validator.verify(value, threshold);
                if (result) {
                    String warningInfo = "结果【" + value + "】" + name + "阈值 【" + threshold + "】";
                    return WarningRuleResult.warning(warningInfo, warningRule);
                } else {
                    return WarningRuleResult.pass();
                }
            } catch (RuntimeException bs) {
                return WarningRuleResult.error(bs.getMessage(), warningRule);
            } catch (Exception e) {
                return WarningRuleResult.error("阈值解析异常", warningRule);
            }
        }

        /**
         * @param value
         * @param threshold
         * @param comparator
         * @param checkNumber
         *            同时校验是否是数字类型，如果不是则抛出异常
         * @return
         * @throws RuntimeException
         */
        private static boolean numberMatch(String value, String threshold, Comparator comparator, boolean checkNumber)
            throws RuntimeException {
            if (value == null) {
                value = "";
            }
            if (threshold == null) {
                // return false;
                throw new RuntimeException(" 对应阈值【" + threshold + "】为【null】,请检查阈值设置");
            }
            BigDecimal valueNum = toNumber(value);
            BigDecimal thresholdNum = toNumber(threshold);

            if (valueNum == null || thresholdNum == null) {
                if (checkNumber) {
                    throw new RuntimeException("结果【" + value + "】无法与阈值【" + threshold + "】比较，请修改脚本或阈值设置");
                } else {
                    return false;
                }
            }
            return comparator.compare(valueNum, thresholdNum);
        }

        interface Comparator {
            boolean compare(BigDecimal d1, BigDecimal d2);
        }

        public String getName() {
            return name;
        }
    }

    /**
     * 请确保 value和threshold 非空
     */
    interface Validator {
        boolean verify(String value, String threshold) throws RuntimeException;
    }

    private static BigDecimal toNumber(String value) {
        boolean isPercent = false;
        value = value.trim();
        if (value.endsWith("%")) {
            value = value.replace("%", "");
            isPercent = true;
        }
        try {
            BigDecimal big = new BigDecimal(value);
            return isPercent ? (big.divide(new BigDecimal("100"))) : big;
        } catch (Exception e) {
            return null;
        }

    }
}
