package com.bocloud.cop.model.inspection;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeSet;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectWarnLevel;

public class WarningRuleEntity {
    private boolean mulitCompare;
    private TreeSet<WarningRule> warningRules; // if mulitCompare == false 表示不匹配key，只匹配值 这个数据使用
    private Map<String, TreeSet<WarningRule>> warningRulesMap; // if mulitCompare == true 表示匹配key 匹配值 这个数据使用

    public boolean isMulitCompare() {
        return mulitCompare;
    }

    /**
     * 单值匹配时使用 mulitCompare = false
     *
     * @return
     * @throws RuntimeException
     */
    public TreeSet<WarningRule> getWarningRules() throws RuntimeException {
        if (mulitCompare) {
            throw new RuntimeException("多值匹配模式，请使用getWarningRulesMap");
        }
        return warningRules;
    }

    /**
     * 多值匹配时使用 mulitCompare = true
     *
     * @return
     * @throws RuntimeException
     */
    public Map<String, TreeSet<WarningRule>> getWarningRulesMap() throws RuntimeException {
        if (!mulitCompare) {
            throw new RuntimeException("单值匹配模式，请使用getWarningRules");
        }
        return warningRulesMap;
    }

    /**
     * 将巡检规则解析为bean
     *
     * @param text
     *            巡检规则字符串
     * @param mulitCompare
     *            该巡检规则是单值匹配还是多值匹配
     * @param warnLevelMap
     *            告警等级数据 用于将告警数据加载进规则数据
     * @return 巡检规则实体
     * @throws RuntimeException
     */
    public static WarningRuleEntity paras(String text, boolean mulitCompare, Map<String, InspectWarnLevel> warnLevelMap)
        throws RuntimeException {
        WarningRuleEntity warningRuleEntity = new WarningRuleEntity();
        warningRuleEntity.mulitCompare = mulitCompare;
        if (mulitCompare) {
            warningRuleEntity.warningRulesMap = parseMap(text, warnLevelMap);
        } else {
            warningRuleEntity.warningRules = parse(text, warnLevelMap);
        }
        return warningRuleEntity;
    }

    /**
     * 用于单值匹配的解析
     *
     * @param text
     * @param warnLevelMap
     * @return
     * @throws RuntimeException
     * @see WarningRuleEntity#paras(String, boolean, Map)
     */
    private static TreeSet<WarningRule> parse(String text, Map<String, InspectWarnLevel> warnLevelMap)
        throws RuntimeException {
        Comparator<WarningRule> comparator = (o1, o2) -> o2.getWarnLevel() - o1.getWarnLevel();
        TreeSet<WarningRule> rules = new TreeSet<>(comparator);
        if (StringUtils.isEmpty(text)) {
            return rules;
        }
        JSONArray array = JSON.parseArray(text);
        if (array == null) {
            return rules;
        }
        for (Object o : array) {
            JSONObject obj = ((JSONObject)o);
            String ruleName = obj.getString("rule");
            if (ruleName == null) {
                throw new RuntimeException("请检查巡检指标，规则不能为空");
            }
            WarningRule.RULE rule = WarningRule.RULE.valueOf(ruleName);
            String target = obj.getString("threshold");
            Integer level = obj.getInteger("warnLevel");
            String remark = obj.getString("description");
            InspectWarnLevel warnLevel = warnLevelMap == null ? null : warnLevelMap.get(level + "");
            String color = warnLevel == null ? null : warnLevel.getColor();
            if (target == null) {
                throw new RuntimeException("请检查巡检指标，阈值不能为空");
            }
            if (level == null) {
                throw new RuntimeException("请检查巡检指标，等级不能为空");
            }
            if (remark == null) {
                remark = "";
            }
            WarningRule warningRule = new WarningRule(rule, target, level, remark, color);
            if (rules.contains(warningRule)) {
                throw new RuntimeException("单值匹配中,存在相同告警等级[" + warningRule.getWarnLevel() + "]的阈值");
            }
            rules.add(warningRule);
        }
        /*Collections.sort(rules, new Comparator<WarningRule>() {
            @Override
            public int compare(WarningRule o1, WarningRule o2) {
                return o2.warnLevel - o1.warnLevel  ;
            }
        });*/
        return rules;
    }

    private static Map<String, TreeSet<WarningRule>> parseMap(String text, Map<String, InspectWarnLevel> warnLevelMap)
        throws RuntimeException {
        // [{"rule":"1","target":"60","level":"1","remark":""}]
        if (StringUtils.isEmpty(text)) {
            return new HashMap<>();
        }
        JSONArray array = JSON.parseArray(text);
        if (array == null) {
            return new HashMap<>();
        }
        Map<String, TreeSet<WarningRule>> map = new HashMap<>();
        for (Object o : array) {
            JSONObject obj = ((JSONObject)o);
            String ruleName = obj.getString("rule");
            if (ruleName == null) {
                throw new RuntimeException("请检查巡检指标，规则不能为空");
            }
            WarningRule.RULE rule = WarningRule.RULE.valueOf(ruleName);
            String target = obj.getString("threshold");
            Integer level = obj.getInteger("warnLevel");
            String remark = obj.getString("description");
            String keyName = obj.getString("keyName");
            String key = obj.getString("key");
            InspectWarnLevel warnLevel = warnLevelMap == null ? null : warnLevelMap.get(level + "");
            String color = warnLevel == null ? null : warnLevel.getColor();
            if (target == null) {
                throw new RuntimeException("请检查巡检指标，阈值不能为空");
            }
            if (level == null) {
                throw new RuntimeException("请检查巡检指标，等级不能为空");
            }
            if (remark == null) {
                remark = "";
            }
            if (StringUtils.isEmpty(key)) {
                throw new RuntimeException("多值匹配，指标不能为空");
            }
            if (StringUtils.isEmpty(keyName)) {
                throw new RuntimeException("多值匹配，指标名称不能为空");
            }
            WarningRule warningRule = new WarningRule(rule, target, level, remark, color);
            TreeSet<WarningRule> warningRules = map.get(key);
            if (warningRules == null) {
                Comparator<WarningRule> comparator = (o1, o2) -> o2.getWarnLevel() - o1.getWarnLevel();
                warningRules = new TreeSet<>(comparator);
                map.put(key, warningRules);
            }
            if (warningRules.contains(warningRule)) {
                throw new RuntimeException("多值匹配中,[" + key + "]存在相同告警等级[" + warningRule.getWarnLevel() + "]的阈值");
            }
            warningRules.add(warningRule);
        }
        return map;
    }
}
