package com.bocloud.cop.schedule.inspection.report;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectReport;
import com.bocloud.cop.entity.inspection.InspectReportDetail;
import com.bocloud.cop.intf.inspection.InspectReportDetailService;
import com.bocloud.cop.repository.inspection.InspectReportRepository;
import com.bocloud.cop.service.utils.EmailSender;
import com.bocloud.cop.service.utils.StorageConfig;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 巡检报表定时作业-发送报表
 */
@Slf4j
@Component
public class InspectReportSchedulerJob {

    @Autowired
    private InspectReportDetailService inspectReportDetailService;
    @Autowired
    private InspectReportRepository inspectReportRepository;
    @Autowired
    private EmailSender emailSender;
    @Autowired
    private StorageConfig storageConfig;
    @Autowired
    private LockFactory lockFactory;

    private void before() {
        log.info("巡检报表任务开始执行");
    }

    public void execute(Long reportId) {
        before();
        try {
            InspectReport report = inspectReportRepository.get(reportId);
            if (null == report) {
                return;
            }
            GeneralResult<InspectReportDetail> result =
                inspectReportDetailService.create(reportId, report.getCreatorId());
            log.info("保存详情" + (result.isSuccess() ? "成功" : "失败：" + result.getMessage()));
            if (result.isSuccess()) {
                InspectReportDetail detail = result.getData();
                GeneralResult<InspectReportDetail> detailResult = inspectReportDetailService.detail(detail.getId());
                if (detailResult.isSuccess()) {
                    setEmail(report, detail);
                }
            }
        } catch (Exception e) {
            log.error("保存失败", e);
        }
        after();
    }

    private void setEmail(InspectReport report, InspectReportDetail detail) throws Exception {
        List<JSONObject> msgs = JSONObject.parseArray(report.getSendMsg(), JSONObject.class);
        if (ListTool.isEmpty(msgs)) {
            return;
        }
        List<RequestContext> contexts = new ArrayList<>();
        msgs.forEach(msg -> {
            Long userId = msg.getLong("id");
            if (null != userId) {
                contexts.add(new RequestContext(Catalog.Manager, userId, null, null, null));
            }
        });
        if (ListTool.isEmpty(contexts)) {
            return;
        }
        List<File> files = new ArrayList<>();
        files.add(this.handleMail(detail));
        JSONObject message = new JSONObject();
        message.put("subject", "巡检报表【" + report.getName() + "】");
        message.put("content", "巡检报表【" + report.getName() + "】");
        emailSender.send(message, files, contexts);
        for (File file : files) {
            if (!file.exists()) {
                continue;
            }
            if (!file.delete()) {
                log.error("文件删除失败", file.getName());
            }
        }
    }

    /**
     * 处理邮件发送内容发送
     */
    private File handleMail(InspectReportDetail detail) throws IOException {
        HSSFWorkbook workbook = new HSSFWorkbook();
        InspectReportExcelUtil.excel(detail, workbook);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        workbook.write(baos);
        String title = new String(detail.getTitle().getBytes(), StandardCharsets.ISO_8859_1);
        checkInspectionDic();
        File file = new File(storageConfig.getInspectionPath() + "/" + title + ".xls");
        workbook.write(file);
        return file;
    }

    /**
     * 验证巡检文件夹是否存在，不存在创建文件夹
     */
    private void checkInspectionDic() {
        String patch = storageConfig.getInspectionPath();
        File file = new File(patch);
        if (file.exists() && file.isDirectory()) {
            return;
        }
        String path = InspectReport.class.getSimpleName() + "_storepath";
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
            if (!file.exists()) {
                // 文件夹不存在的话,创建
                if (!file.mkdirs()) {
                    log.error("文件夹创建失败:{}", "inspection");
                }
            } else if (!file.isDirectory()) {
                log.error("文件夹不存在:{}", "inspection");
            }
        } catch (Exception e) {
            log.error("Remove task fail:", e);
        }
    }

    private void after() {
        log.info("巡检报表任务结束执行");
    }

}
