package com.bocloud.cop.service;

import com.megatron.common.model.GeneralResult;
import com.bocloud.cop.repository.CommandRepository;
import com.bocloud.cop.entity.Command;
import com.bocloud.cop.intf.CommandCheckService;
import com.bocloud.cop.model.Item;
import com.bocloud.cop.model.Role;
import com.bocloud.cop.service.internal.SmsInternalService;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.MatchResult;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
@Component
public class CommandCheckServiceImpl implements CommandCheckService {
    private static Pattern pattern = Pattern.compile("([\\w]+)");
    @Autowired
    private CommandRepository commandRepository;
    @Autowired
    private SmsInternalService smsInternalService;

    /**
     * 检测限制的命令
     *
     * @param userId
     * @param items
     * @return Map<Item, List < Command>> 脚本-限制的命令
     */
    @Override
    public Map<Item, List<Command>> check(Long userId, List<Item> items) {
        // 脚本里面含有高危命令的ID
        Map<Item, List<Command>> matchedCommandMap = this.getCommandId(items);
        if (matchedCommandMap.size() == 0) {
            return new HashMap<>(16);
        }
        GeneralResult<List<Role>> result = smsInternalService.listByUid(userId);
        // 用户无权执行命令的ID
        //Command Ids
        if (result.isFailed()) {
            throw new RuntimeException("检测高危命令失败，请检查soc服务状态");
        }
        List<Role> roleList = result.getData();
        List<Role> roles = Lists.newArrayList();
        for (Role role : roleList) {
            if (role.isChecked()) {
                roles.add(role);
            }
        }
        if (roles.isEmpty()) {
            throw new RuntimeException("检测高危命令失败，执行用户未绑定任何角色");
        }
        // 用户可以执行的高危命令
        List<Command> allowCommands = Lists.newArrayList();
        try {
            for (Role role : roles) {
                List<Command> comm = commandRepository.listByRid(role.getId());
                allowCommands.addAll(comm);
            }
        } catch (Exception e) {
            throw new RuntimeException("检测高危命令失败，查询放行命令失败");
        }

        if (allowCommands.isEmpty()) {
            return matchedCommandMap;
        }
        Map<Item, List<Command>> limit = new HashMap<>();
        //检测出的command
        for (Map.Entry<Item, List<Command>> entry : matchedCommandMap.entrySet()) {
            Item item = entry.getKey();
            //该命令中存在的高危命令
            List<Command> commands = entry.getValue();
            List<Command> limitCommand = new ArrayList<>();
            for (Command command : commands) {
                boolean allow = false;
                for (Command allowCommand : allowCommands) {
                    //如果检测出的command被允许，跳出
                    if (allowCommand.getId().equals(command.getId())) {
                        allow = true;
                        break;
                    }
                }
                //未检测到command被允许   记录
                if (!allow) {
                    limitCommand.add(command);
                }
            }
            if (!limitCommand.isEmpty()) {
                limit.put(item, limitCommand);
            }
        }
        return limit;
    }


    /**
     * 获取脚本中高危命令的ID
     *
     * @return
     */
    private Map<Item, List<Command>> getCommandId(List<Item> items) {
        //<CommandId,List<ItemId>>
        Map<Item, List<Command>> map =  new HashMap<>(16);
        // 提取内容
        try {
            List<Command> commands = commandRepository.list();
            if (commands.isEmpty()) {
                return map;
            }
            for (Item item : items) {
                String content = item.getScriptContent();
                // 按换行 分割
                String[] contents = content.split("\\n");
                // 从数据库里面查询出高危命令

                for (Command command : commands) {
                    List<Command> commandList = new ArrayList<>();
                    for (String str : contents) {
                        // 处理单行数据
                        String comContent = command.getContent().trim();
                        MatchResult matchResult = pattern.matcher(str).toMatchResult();
                        List<String> commandAndOptionList = getCommandPerLine(matchResult);
                        int index = str.indexOf("#");
                        if (index == -1) {
                            // 没有注释的时候
                            if (commandAndOptionList.contains(comContent)) {
                                commandList.add(command);
                            }
                        } else {
                            // 有注释的时候
                            if (commandAndOptionList.contains(comContent) && str.indexOf("#") > str.indexOf(comContent)) {
                                commandList.add(command);
                            }
                        }
                    }
                    if (!commandList.isEmpty()) {
                        map.put(item, commandList);
                    }
                }
            }
        } catch (Exception e) {
            log.error("get deadly commandID fail:", e);
        }
        return map;
    }

    private List<String> getCommandPerLine(MatchResult matchResult) {
        List<String> list = Lists.newArrayList();
        while (((Matcher) matchResult).find()) {
            list.add(matchResult.group());
        }
        return list;
    }
}
