package com.bocloud.cop.service;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.regex.MatchResult;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.megatron.common.utils.Common;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Command;
import com.bocloud.cop.entity.RoleCommand;
import com.bocloud.cop.intf.CommandService;
import com.bocloud.cop.model.Role;
import com.bocloud.cop.repository.CommandRepository;
import com.bocloud.cop.repository.RoleCommandRepository;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 命令service接口实现
 * 
 * @author weiwei
 *
 */
@Slf4j
@Service("commandService")
public class CommandServiceImpl implements CommandService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private CommandRepository commandRepository;
    @Autowired
    private RoleCommandRepository roleCommandRepository;
    private static Pattern pattern = Pattern.compile("([\\w]+)");

    @Override
    public GeneralResult<GridBean<Command>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = commandRepository.count(pager.getParams());
            List<Command> list = null;
            if (pager.getSimple()) {
                list = commandRepository.list();
            } else {
                list = commandRepository.list(pager);
            }
            GridBean<Command> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("List command fail:", e);
            return new GeneralResult<>(false, "查询失败", "", "");
        }
    }

    @Override
    public GeneralResult<Void> create(Command command, Long userId) {
        try {
            Param param = new Param();
            param.setSign(Sign.EQ);
            Map<String, Object> eqMap = Maps.newHashMap();
            eqMap.put("name", command.getName());
            eqMap.put("content", command.getContent());
            param.setParam(eqMap);
            Command queryCommand = commandRepository.query(param);
            if (queryCommand != null) {
                return new GeneralResult<>(false, "添加失败,已存在相同数据");
            }
            command.setGmtCreate(new Date());
            command.setGmtModify(command.getGmtCreate());
            command.setCreatorId(userId);
            command.setMenderId(userId);
            commandRepository.save(command);
            return new GeneralResult<>(true, "添加成功!");
        } catch (Exception e) {
            log.error("create command failure:", e);
            return new GeneralResult<>(false, "添加失败!");
        }
    }

    @Override
    public GeneralResult<Void> modify(Command command, Long userId) {
        String path = Command.class.getSimpleName() + "_" + command.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Command com = commandRepository.query(command.getId());
            Assert.notNull(com, "数据不存在");
            command.setGmtModify(new Date());
            command.setMenderId(userId);
            boolean result = commandRepository.update(command);
            return result ? new GeneralResult<>(true, "修改成功") : new GeneralResult<>(false, "修改失败", "", "");
        } catch (Exception e) {
            log.error("modify command fail:", e);
            return new GeneralResult<>(false, "修改失败", "", "");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = Command.class.getSimpleName() + "_" + id;
        boolean result = false;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Command com = commandRepository.query(id);
            Assert.notNull(com, "数据不存在");
            // 删除命令和角色的关联关系
            roleCommandRepository.deleteByCid(id);
            result = commandRepository.delete(id, userId);
        } catch (Exception e) {
            log.error("remove command fail:", e);
            return new GeneralResult<>(false, "删除失败", "", "");
        }
        return result ? new GeneralResult<>(true, "删除成功") : new GeneralResult<>(false, "删除失败", "", "");
    }

    @Override
    public GeneralResult<Command> detail(Long id) {
        try {
            Command command = commandRepository.query(id);
            Assert.notNull(command, "数据不存在");
            return new GeneralResult<>(true, command, "获取详情成功!");
        } catch (Exception e) {
            log.error("get command fail", e);
            return new GeneralResult<>(false, "获取详情失败!");
        }
    }

    @Override
    public GeneralResult<List<Command>> listCommands(Long roleId) {
        try {
            List<Command> list = commandRepository.listByRid(roleId);
            List<Command> commands = commandRepository.list();
            if (!ListTool.isEmpty(list)) {
                for (Command command : commands) {
                    for (Command c : list) {
                        if (command.getId().equals(c.getId())) {
                            command.setChecked(true);
                        }
                    }
                }
            }
            return new GeneralResult<>(true, commands, "查询成功");
        } catch (Exception e) {
            log.error("list commands fail:", e);
            return new GeneralResult<>(false, "查询失败!");
        }
    }

    @Override
    public GeneralResult<Void> accCommand(Long roleId, String commands) {
        String path = Role.class.getSimpleName() + "_" + roleId;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 删除角色的命令
            this.roleCommandRepository.deleteByRid(roleId);
            if (null != commands) {
                String[] command = commands.split(",");
                int length = command.length;
                for (int i = 0; i < length; i++) {
                    if (StringUtils.hasText(command[i])) {
                        RoleCommand roleCommand = new RoleCommand(roleId, Long.valueOf(command[i]));
                        roleCommandRepository.save(roleCommand);
                    }
                }
            }
            return new GeneralResult<>(true, "授权角色命令成功");
        } catch (Exception e) {
            log.error("release user fail:", e);
            return new GeneralResult<>(false, "授权角色命令失败", "", "");
        }
    }

    /**
     * 获取脚本中高危命令的ID
     *
     * @param content
     * @return
     */
    @Override
    public List<Long> getCommands(String content) {
        List<Long> list = Lists.newArrayList();
        JSONObject jsonObj = JSONTools.isJSONObj(content);
        JSONObject fronts = jsonObj.getJSONObject("fronts");
        // 提取内容
        if (fronts != null) {
            String contentStr = fronts.getString("content");
            content = StringUtils.hasText(contentStr) ? contentStr : content;
        }
        try {
            // 按换行 分割
            String[] contents = content.split("\\n");
            // 从数据库里面查询出高危命令
            List<Command> commands = commandRepository.list();
            if (commands.isEmpty()) {
                return list;
            }

            for (Command command : commands) {
                for (String str : contents) {
                    // 处理单行数据
                    String comContent = command.getContent().trim();
                    MatchResult matchResult = pattern.matcher(str).toMatchResult();
                    List<String> commandAndOptionList = getCommandPerLine(matchResult);
                    int index = str.indexOf("#");
                    if (index == -1) {
                        // 没有注释的时候
                        if (commandAndOptionList.contains(comContent)) {
                            list.add(command.getId());
                        }
                    } else {
                        // 有注释的时候
                        if (commandAndOptionList.contains(comContent) && str.indexOf("#") > str.indexOf(comContent)) {
                            list.add(command.getId());
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("get deadly commandID fail:", e);
        }
        return list;
    }

    private List<String> getCommandPerLine(MatchResult matchResult) {
        List<String> list = Lists.newArrayList();
        while (((Matcher)matchResult).find()) {
            list.add(matchResult.group());
        }
        return list;
    }

}
