package com.bocloud.cop.service;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.web.multipart.MultipartFile;

import com.bocloud.cop.entity.Identification;
import com.bocloud.cop.entity.ResourceIdentificationRelation;
import com.bocloud.cop.intf.IdentificationService;
import com.bocloud.cop.repository.IdentificationRepository;
import com.bocloud.cop.repository.ResourceIdentificationRelationRepository;
import com.bocloud.cop.service.utils.BoStringUtils;
import com.bocloud.cop.service.utils.ExcelFactory;
import com.bocloud.cop.service.utils.IdentificationExcelHelper;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 认证service接口实现
 */
@Slf4j
@Service("identificationService")
public class IdentificationServiceImpl implements IdentificationService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private IdentificationRepository identificationRepository;
    @Autowired
    private ResourceIdentificationRelationRepository resourceIdentificationRelationRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public GeneralResult<GridBean<Identification>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            if (Boolean.valueOf(pager.getSimple())) {
                return simple(pager);
            }
            int total = identificationRepository.count(pager.getParams());
            List<Identification> list = identificationRepository.list(pager);
            for (Identification identi : list) {
                identi.setCreator(getUserNameByUserId(identi.getCreatorId()));
                identi.setMender(getUserNameByUserId(identi.getMenderId()));
                identi.setRelationCount(resourceIdentificationRelationRepository.count(identi.getId()));
            }
            GridBean<Identification> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询认证账号成功");
        } catch (Exception e) {
            log.error("List identification fail:", e);
            return new GeneralResult<>(false, "查询认证账号失败");
        }
    }

    @Override
    public GeneralResult<GridBean<Identification>> simple(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            if(pager.getSorter().keySet().isEmpty()){
                pager.getSorter().put(Common.ID, Common.ONE);
            }
            if (null == pager.getRows()) {
                pager.setRows(Integer.MAX_VALUE);
            }
            int total = identificationRepository.count(pager.getParams());
            List<Identification> beans = identificationRepository.list(pager.getParams(), pager.getSorter());
            for (Identification bean : beans) {
                bean.setAccountName(bean.getAccountName() + "(" + bean.getUsername() + ")");
            }
            GridBean<Identification> gridBean = new GridBean<>(1, 1, total, beans);
            beans.forEach(
                ident -> ident.setRelationCount(resourceIdentificationRelationRepository.count(ident.getId())));
            return new GeneralResult<>(true, gridBean, "查询认证账号成功");
        } catch (Exception e) {
            log.error("List identification fail:", e);
            return new GeneralResult<>(false, "查询认证账号失败");
        }
    }

    @Override
    public GeneralResult<Void> create(Identification identification, Long userId) {
        try {
            // 判断名称是否存在
            if (checkAccountExist(identification.getAccountName(), identification.getUsername(), null)) {
                log.error("Create identification fail: accountName + userName is exist");
                return new GeneralResult<>(false, "创建认证账号失败:该名称下用户名已经存在");
            }
            // 对密码进行处理
            identification.setPassword(identification.getPassword());
            identification.setGmtCreate(new Date());
            identification.setGmtModify(identification.getGmtCreate());
            identification.setCreatorId(userId);
            identification.setMenderId(userId);
            identification.setDeleted(false);
            identificationRepository.save(identification);
            return new GeneralResult<>(true, "创建认证账号成功");
        } catch (Exception e) {
            log.error("Create identification fail:", e);
            return new GeneralResult<>(false, "创建认证账号失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(Identification identification, Long userId) {
        String path = Identification.class.getSimpleName() + "_" + identification.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Identification ident = identificationRepository.query(identification.getId());
            Assert.notNull(ident, "数据不存在!");
            // 判断名称是否存在
            if (checkAccountExist(identification.getAccountName(), identification.getUsername(),
                identification.getId())) {
                log.error("Modify identification fail: accountName+userName is exist");
                return new GeneralResult<>(false, "编辑认证账号失败:该名称下用户名已经存在");
            }
            // 判断原始密码是否正确
            if (StringUtils.isNotBlank(identification.getPassword())) {
                String oldPassword = identification.getOldPassword();
                if (StringUtils.isBlank(oldPassword)) {
                    log.error("Modify identification fail:lack old password param");
                    return new GeneralResult<>(false, "编辑认证账号失败:缺少原密码参数");
                }
                if (!oldPassword.equals(ident.getPassword())) {
                    log.error("Modify identification fail:old password is wrong");
                    return new GeneralResult<>(false, "编辑认证账号失败:原密码错误");
                }
                ident.setPassword(identification.getPassword());
            }
            ident.setAccountName(identification.getAccountName());
            ident.setUsername(identification.getUsername());
            ident.setRemark(identification.getRemark());
            ident.setGmtModify(new Date());
            ident.setMenderId(userId);
            Boolean result = identificationRepository.update(ident);
            return result ? new GeneralResult<>(true, "编辑认证账号成功") : new GeneralResult<>(false, "编辑认证账号失败");
        } catch (Exception e) {
            log.error("modify identification fail:", e);
            return new GeneralResult<>(false, "编辑认证账号失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, RequestContext context) {
        String path = Identification.class.getSimpleName() + "_" + id;
        boolean result = false;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Identification ident = identificationRepository.query(id);
            Assert.notNull(ident, "数据不存在!");
            result = identificationRepository.delete(id, context.getTarget());
            // 认证账号删除后，删除关系
            resourceIdentificationRelationRepository.removeByIdentificationId(id);
        } catch (Exception e) {
            log.error("remove identification fail:", e);
            return new GeneralResult<>(false, "删除认证账号失败");
        }
        return result ? new GeneralResult<>(true, "删除认证账号成功") : new GeneralResult<>(false, "删除认证账号失败");
    }

    @Override
    public GeneralResult<Identification> detail(Long id) {
        try {
            Identification ident = identificationRepository.query(id);
            Assert.notNull(ident, "获取详情失败!");
            // 查询关联的设备
            ident.setTaskHostIds(resourceIdentificationRelationRepository.listTaskHostIdsByIdentificationId(
                ident.getId(), ResourceIdentificationRelation.ResourceType.HOST.name()));
            ident.setRelationCount(Integer.valueOf(ident.getTaskHostIds().size()).longValue());
            // 不返回密码
            ident.setPassword(null);
            return new GeneralResult<>(true, ident, "获取详情成功!");
        } catch (Exception e) {
            log.error("get identification fail", e);
            return new GeneralResult<>(false, "获取详情失败!");
        }
    }

    private String getUserNameByUserId(Long userId) {
        return redisTemplate.opsForValue().get("User_Name_" + userId);
    }

    /**
     * 校验名称+用户名是否存在
     * 
     * @param name
     *            名称
     * @param userName
     *            用户名
     * @param id
     *            排除id
     * @return 校验结果
     */
    private boolean checkAccountExist(String name, String userName, Long id) {
        return identificationRepository.count(name, userName, id) > 0;
    }

    @Override
    public GeneralResult<Void> importExcel(MultipartFile excel, RequestContext context) throws IOException {
        if (null == excel) {
            return new GeneralResult<>(false, "批量导入认证账号失败");
        }
        String name = excel.getOriginalFilename();
        if (StringUtils.isEmpty(name)) {
            return new GeneralResult<>(false, "批量导入认证账号失败");
        }
        String fileName = URLDecoder.decode(name, "UTF-8");
        ExcelFactory factory = new ExcelFactory(excel.getInputStream(), fileName);
        String path = Identification.class.getSimpleName() + "_update";
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "有其他导入任务正在进行");
            IdentificationExcelHelper excelHelper = new IdentificationExcelHelper(identificationRepository);
            return excelHelper.processExcel(factory.newWorkbook(), context);
        } catch (Exception e) {
            log.error("批量导入认证账号失败:", e);
            return new GeneralResult<>(false, "批量导入认证账号失败:" + e);
        }
    }

    @Override
    public GeneralResult<Map<String, Object>> download(Map<String, Object> paramsMap) {
        try {
            // 校验前端传入的excel的版本（2000或2007），生成后缀。当前为前端写死为2007版本
            String fileName;
            if ("2000".equals(BoStringUtils.toString(paramsMap.get("excelType")))) {
                fileName = "认证账号.xls";
            } else {
                fileName = "认证账号.xlsx";
            }
            Workbook workbook = new ExcelFactory(fileName).newWorkbook();
            IdentificationExcelHelper exportHelper = new IdentificationExcelHelper();
            // 生成模板
            exportHelper.generateSheet(workbook);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            workbook.write(baos);
            byte[] ba = baos.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", fileName);
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new GeneralResult<>(true, resultMap, "下载成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败:" + e);
        }
    }
}
