package com.bocloud.cop.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import com.alibaba.fastjson.JSONArray;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.constant.CommonConstant;
import com.bocloud.cop.entity.NetworkSegment;
import com.bocloud.cop.entity.TaskWorker;
import com.bocloud.cop.intf.ServerServiceMgtService;
import com.bocloud.cop.model.WorkerHost;
import com.bocloud.cop.model.WorkerLoadInfo;
import com.bocloud.cop.model.params.ServerServiceManageParams;
import com.bocloud.cop.model.params.ServerServiceShellParams;
import com.bocloud.cop.repository.NetworkSegmentRepository;
import com.bocloud.cop.repository.TaskWorkerRepository;
import com.bocloud.cop.service.utils.OsCategoryUtils;
import com.bocloud.cop.service.utils.SshPortHelper;
import com.bocloud.cop.service.utils.TaskHostDataUtil;
import com.bocloud.cop.service.utils.worker.TaskWorkerHelper;
import com.megatron.common.enums.OSCategory;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 主机服务管理实现类
 */
@Service("serverServiceMgtService")
@Slf4j
public class ServerServiceMgtServiceImpl implements ServerServiceMgtService {

    @Autowired
    private NetworkSegmentRepository networkSegmentRepository;
    @Autowired
    TaskWorkerRepository taskWorkerRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private StringRedisTemplate redisTemplate;

    /**
     * 服务管理
     */
    @Override
    public GeneralResult<?> manage(ServerServiceManageParams params, Long userId) {
        List<WorkerHost> targets = params.getTargets();
        String serviceName = params.getServiceName();
        String operate = params.getOperate();
        String arguments = params.getArgs();
        Boolean enabled = params.getEnabled();
        String state = operate;
        String startMode = params.getStartMode();
        if (ListTool.isEmpty(targets)) {
            return new GeneralResult<>(false, "目标主机信息targets不能为空");
        }
        if (StringUtils.isEmpty(serviceName)) {
            return new GeneralResult<>(false, "服务名称不能为空");
        }
        // worker服务需要的参数targets
        Long groupId = null;
        for (WorkerHost target : targets) {
            OSCategory category = OsCategoryUtils.getOSCategory(target.getCategory());
            if (category == OSCategory.WINDOWS) {
                // windows有5个状态：started, stopped, restarted, absent, paused
                List<String> winOperations = new ArrayList<>();
                winOperations.add("started");
                winOperations.add("stopped");
                winOperations.add("restarted");
                winOperations.add("absent");
                winOperations.add("paused");
                if (!winOperations.contains(operate)) {
                    return new GeneralResult<>(false,
                        "操作[" + operate + "]无效，取值范围是started, stopped, restarted, absent, paused (Windows)");
                }
            } else {
                // linux有4个状态： started, stopped, restarted, reloaded
                List<String> linuxOperations = new ArrayList<>();
                linuxOperations.add("started");
                linuxOperations.add("stopped");
                linuxOperations.add("restarted");
                linuxOperations.add("reloaded");
                if (!linuxOperations.contains(operate)) {
                    return new GeneralResult<>(false,
                        "操作[" + operate + "]无效，取值范围是started, stopped, restarted, reloaded (Linux/Uinux)");
                }
            }
            target.setCategory(category.name());
            if (target.getPort() == null || target.getPort().intValue() == 0) {
                target.setPort(SshPortHelper.getDefaultPort(target.getCategory()));
            }
            // 判断主机是否存在对应的网段，分组和普通worker
            GeneralResult<Long> result = this.getGroupId(target);
            if (result.isFailed()) {
                return new GeneralResult<>(false, result.getMessage());
            }
            groupId = result.getData();
        }
        // worker服务需要的参数module
        Map<String, Object> module = new HashMap<>();
        module.put(Common.NAME, CommonConstant.SERVICE);
        Map<String, Object> args = new HashMap<>();
        args.put(Common.NAME, serviceName);
        if (!StringUtils.isEmpty(state)) {
            args.put(CommonConstant.STATE, state);
        }
        if (enabled != null) {
            args.put(CommonConstant.ENABLED, enabled);
        }
        args.put(CommonConstant.ARGS, arguments);
        if (!StringUtils.isEmpty(startMode)) {
            args.put(CommonConstant.START_MODE, startMode);
        }
        module.put(CommonConstant.ARGS, args);
        Map<String, Object> options = null == params.getOptions() ? new HashMap<>() : params.getOptions();
        if (!options.isEmpty()) {
            module.put(CommonConstant.OPTIONS, options);
        }
        Map<String, Object> paramMap = new HashMap<>(4);
        paramMap.put(CommonConstant.TARGETS, targets);
        paramMap.put(CommonConstant.MODULE, module);
        // 调用Worker服务
        try {
            String instance = getInstance(groupId);
            if (StringUtils.isEmpty(instance)) {
                return new GeneralResult<>(false, "网段对应分组未配置执行worker");
            }
            GeneralResult<?> result = TaskWorkerHelper.invoke(instance, paramMap, true);
            if (null != result.getData()) {
                JSONArray array = JSONArray.parseArray(JSONObject.toJSONString(result.getData()));
                if (array.size() > 0 && StringUtils.isNotBlank(array.getJSONObject(0).getString("message"))) {
                    result.setMessage(array.getJSONObject(0).getString("message"));
                }
            }
            return result;
        } catch (Exception e) {
            return new GeneralResult<>(false, "服务操作失败：" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<?> shell(ServerServiceShellParams params, Long userId) {
        List<WorkerHost> targets = JSONObject.parseArray(JSONObject.toJSONString(params.getHosts()), WorkerHost.class);
        if (ListTool.isEmpty(targets)) {
            return new GeneralResult<>(false, "目标主机信息targets不能为空");
        }
        // worker服务需要的参数module
        Map<String, Object> module = new HashMap<>(4);
        module.put(Common.NAME, "raw");
        module.put(CommonConstant.ARGS, params.getShell());
        Map<String, Object> paramMap = new HashMap<>(4);
        paramMap.put(CommonConstant.TARGETS, targets);
        paramMap.put(CommonConstant.MODULE, module);
        // 判断主机是否存在对应的网段，分组和普通worker
        Long groupId = null;
        for (WorkerHost target : targets) {
            GeneralResult<Long> result = this.getGroupId(target);
            if (result.isFailed()) {
                return new GeneralResult<>(false, result.getMessage());
            }
            groupId = result.getData();
        }
        // 调用Worker服务
        try {
            String instance = getInstance(groupId);
            if (StringUtils.isEmpty(instance)) {
                return new GeneralResult<>(false, "网段对应分组未配置执行worker");
            }
            return TaskWorkerHelper.invoke(instance, paramMap, true);
        } catch (Throwable t) {
            return new GeneralResult<>(false, "服务操作失败:" + t.getMessage());
        }
    }

    /**
     * 判断主机是否存在对应的网段，分组和普通worker
     * 
     * @param host
     * @return
     */
    private GeneralResult<Long> getGroupId(WorkerHost host) {
        try {
            NetworkSegment segment = null;
            if (null == host.getSegmentId()) {
                List<NetworkSegment> segments = networkSegmentRepository.list();
                if (ListTool.isEmpty(segments)) {
                    return new GeneralResult<>(false, "平台未配置网段信息");
                }
                segment = TaskHostDataUtil.getSegment(host.getHost(), segments);
            } else {
                segment = networkSegmentRepository.query(host.getSegmentId());
            }
            if (null == segment) {
                return new GeneralResult<>(false, "目标机器未匹配到对应网段信息");
            }
            if (null == segment.getGroupId()) {
                return new GeneralResult<>(false, "目标机器匹配的网段未配置分组");
            }
            List<TaskWorker> workers =
                taskWorkerRepository.listGenerals(segment.getGroupId(), TaskWorker.Category.Ansible.name());
            if (ListTool.isEmpty(workers)) {
                return new GeneralResult<>(false, "网段对应分组未配置执行worker");
            }
            return new GeneralResult<>(true, segment.getGroupId(), "success");
        } catch (Exception e) {
            log.error("execute task fail:", e);
            return new GeneralResult<>(false, "网段对应分组未配置执行worker");
        }
    }

    private String getInstance(Long groupId) {
        String key = "workergroup_" + groupId;
        try (AutoCloseLock lock = lockFactory.getACLock(key)) {
            Assert.isTrue(lock.acquire(300, TimeUnit.SECONDS), "请求超时");
            String value = redisTemplate.opsForValue().get(key);
            WorkerLoadInfo loadInfo = JSONObject.parseObject(value, WorkerLoadInfo.class);
            if (null == loadInfo || ListTool.isEmpty(loadInfo.getWorkers())) {
                return null;
            }
            List<String> workers = loadInfo.getWorkers();
            int position = loadInfo.getPosition();
            if (position >= workers.size()) {
                position = 0;
            }
            String instance = workers.get(position);
            position++;
            loadInfo.setPosition(position);
            redisTemplate.opsForValue().set(key, JSONObject.toJSONString(loadInfo));
            return instance;
        } catch (Exception e) {
            log.error("配置worker负载缓存失败:", e);
            return null;
        }
    }

}
