package com.bocloud.cop.service;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cop.entity.SoftwareGroup;
import com.bocloud.cop.intf.SoftwareGroupService;
import com.bocloud.cop.model.SoftwareGroupBean;
import com.bocloud.cop.repository.SoftwareGroupRepository;
import com.bocloud.cop.repository.SoftwareRepository;
import com.google.common.collect.Lists;
import com.megatron.common.enums.BaseStatus;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 文件分组抽象Service接口实现类
 * 
 * @author wei wei
 * @version 1.0
 * @since 2016.08.16
 */
@Slf4j
@Service("softwareGroupService")
public class SoftwareGroupServiceImpl implements SoftwareGroupService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private SoftwareGroupRepository softwareGroupRepository;
    @Autowired
    private SoftwareRepository softwareRepository;

    @Override
    public GeneralResult<List<SoftwareGroup>> list(Long parentId) {
        if (parentId == null) {
            parentId = 0L; // 如果不传默认为0，查根
        }
        try {
            List<SoftwareGroup> softwareGroups = getSoftwareGroupByPid(parentId);
            return new GeneralResult<>(true, softwareGroups, "查询文件分组成功");
        } catch (Exception e) {
            log.error("list softwareGroup fail:", e);
            return new GeneralResult<>(false, "查询文件分组失败");
        }
    }

    /**
     * 此方法递归使用,根据parentId查询子分组
     * 
     * @param parentId
     * @return
     * @throws Exception
     */
    private List<SoftwareGroup> getSoftwareGroupByPid(Long parentId) throws Exception {
        List<SoftwareGroup> softwareGroups = softwareGroupRepository.list(parentId);
        for (SoftwareGroup softwareGroup : softwareGroups) {
            List<SoftwareGroup> children = getSoftwareGroupByPid(softwareGroup.getId());
            if (null != children && !children.isEmpty()) {
                softwareGroup.setChildren("[]");
                softwareGroup.setChildrenList(children);
            }
        }
        return softwareGroups;
    }

    @Override
    public GeneralResult<Void> create(SoftwareGroup group) {
        try {
            boolean flag = softwareGroupRepository.exists(group.getName());
            if (flag) {
                return new GeneralResult<>(false, "此名称已存在!");
            }
            Long parentId = group.getParentId();
            // 设置path
            if (parentId.equals(0L)) {
                group.setPath(File.separator + group.getCode());
                group.setUri(group.getCode());
            } else {
                SoftwareGroup sg = softwareGroupRepository.query(parentId);
                if (null == sg) {
                    return new GeneralResult<>(false, "获取父分组路径失败!");
                }
                String path = sg.getPath() + File.separator + group.getCode();
                group.setPath(path);
                // 设置uri
                SoftwareGroup softwareGroup = softwareGroupRepository.query(parentId);
                if (null != softwareGroup) {
                    group.setUri(softwareGroup.getUri() + File.separator + group.getCode());
                }
            }
            group.setStatus(BaseStatus.NORMAL.name());
            softwareGroupRepository.save(group);
            return new GeneralResult<>(true, "添加文件分组成功");
        } catch (Exception e) {
            log.error("Create software group fail:", e);
            return new GeneralResult<>(false, "添加文件分组失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(SoftwareGroupBean bean, Long userId) {
        String path = SoftwareGroup.class.getSimpleName() + "_" + bean.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            SoftwareGroup group = softwareGroupRepository.query(bean.getId());
            if (null == group) {
                return new GeneralResult<>(false, "文件分组不存在");
            }
            if (!group.getName().equals(bean.getName())) {
                boolean flag = softwareGroupRepository.exists(bean.getName());
                if (flag) {
                    return new GeneralResult<>(false, "此名称已存在!");
                }
            }
            String oldCode = group.getCode();
            BeanUtils.copyProperties(bean, group);
            // 获取父菜单uri更新自己的uri
            Long parentId = group.getParentId();
            String newUri = "";
            if (!parentId.equals(0L)) {
                SoftwareGroup softwareGroup = softwareGroupRepository.query(parentId);
                if (null != softwareGroup) {
                    newUri = softwareGroup.getUri() + File.separator + bean.getCode();
                }
                group.setUri(newUri);
            } else {
                newUri = bean.getCode();
                group.setUri(newUri);
            }
            group.setMenderId(userId);
            Boolean flag = softwareGroupRepository.update(group);
            if (!flag) {
                return new GeneralResult<>(false, "修改文件分组失败");
            }
            // 更新文件分组子节点code
            if (!bean.getCode().equals(oldCode)) {
                List<SoftwareGroup> parents = Lists.newArrayList();
                parents.add(group);
                this.updateChildren(parents);
            }
            return new GeneralResult<>(true, "修改文件分组成功");
        } catch (Exception e) {
            log.error("Modify software group fail:", e.getMessage());
            return new GeneralResult<>(false, "修改文件分组失败");
        }
    }

    /**
     * 递归查询子节点并更新URI
     *
     * @param parents
     * @return
     */
    private void updateChildren(List<SoftwareGroup> parents) {
        try {
            List<SoftwareGroup> groups = new ArrayList<>();
            for (SoftwareGroup parent : parents) {
                List<SoftwareGroup> child = softwareGroupRepository.list(parent.getId());
                for (SoftwareGroup group : child) {
                    group.setUri(parent.getUri() + File.separator + group.getCode());
                    softwareGroupRepository.update(group);
                }
                groups.addAll(child);
            }
            if (!groups.isEmpty()) {
                this.updateChildren(groups);
            }
        } catch (Exception e) {
            log.error("get softwaregroup children fail:", e);
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = SoftwareGroup.class.getSimpleName() + "_" + id;
        boolean result = false;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            SoftwareGroup group = softwareGroupRepository.query(id);
            if (null == group) {
                return new GeneralResult<>(false, "文件分组不存在");
            }
            List<SoftwareGroup> list = softwareGroupRepository.list(id);
            if (null != list && !list.isEmpty()) {
                return new GeneralResult<>(false, "文件分组存在子分组，无法删除!");
            }
            List<Param> params = Lists.newArrayList();
            if (softwareRepository.count(params, id) > 0) {
                return new GeneralResult<>(false, "文件分组存在文件，无法删除!");
            }
            result = softwareGroupRepository.delete(id, userId);
            return result ? new GeneralResult<>(true, "删除文件分组成功") : new GeneralResult<>(false, "删除文件分组失败");
        } catch (Exception e) {
            log.error("Remove software group fail:", e);
            return new GeneralResult<>(false, "删除文件分组失败");
        }
    }

    @Override
    public GeneralResult<SoftwareGroup> detail(Long id) {
        try {
            SoftwareGroup group = softwareGroupRepository.query(id);
            return new GeneralResult<>(true, group, "查询文件分组详情成功");
        } catch (Exception e) {
            log.error("Query software group detail fail:", e);
            return new GeneralResult<>(false, "查询文件分组详情失败");
        }
    }

}
