package com.bocloud.cop.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.cop.constant.CommonConstant;
import com.bocloud.cop.entity.*;
import com.bocloud.cop.entity.ResourcePermissionRelation.PermissionType;
import com.bocloud.cop.entity.ResourcePermissionRelation.ResourceType;
import com.bocloud.cop.enums.TaskHostType;
import com.bocloud.cop.intf.TaskHostService;
import com.bocloud.cop.intf.network.TaskWorkerService;
import com.bocloud.cop.model.*;
import com.bocloud.cop.repository.*;
import com.bocloud.cop.service.internal.CmpInternetService;
import com.bocloud.cop.service.internal.SmsInternalService;
import com.bocloud.cop.service.utils.*;
import com.google.common.collect.Maps;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import io.jsonwebtoken.lang.Collections;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 任务主机service接口实现
 *
 * @author weiwei
 * @version 1.0
 * @since 2017.01.17
 */
@Slf4j
@Service("taskHostService")
public class TaskHostServiceImpl implements TaskHostService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private TaskHostRepository taskHostRepository;
    @Autowired
    private NetworkSegmentRepository networkSegmentRepository;
    @Autowired
    private TaskWorkerRepository taskWorkerRepository;
    @Autowired
    private NetworkGroupRepository networkGroupRepository;
    @Autowired
    private CmpInternetService cmpInternetService;
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private ResourceIdentificationRelationRepository identificationRelationRepository;
    @Autowired
    private ResourcePermissionRelationRepository permissionRelationRepository;
    @Autowired
    private SmsInternalService smsInternalService;
    @Autowired
    private IdentificationRepository identificationRepository;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private TaskWorkerService taskWorkerService;

    private static Map<Long, TaskHostVendorBean> vendorBeanMap = new HashMap<>();

    private static final String subject = "主机资源";

    @Override
    public GeneralResult<Void> create(TaskHost host, RequestContext context) {
        String path = TaskHost.class.getSimpleName() + "_update";
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "有其他任务正在更新");
            // ip验证
            List<TaskHost> taskHosts = taskHostRepository.listAllNoVendor();
            List<NetworkSegment> segmentList = networkSegmentRepository.list();
            Set<String> allIps = TaskHostDataUtil.getAllIp(taskHosts);
            String exsitIp = TaskHostDataUtil.checkIp(host.getIps(), allIps);
            if (null != exsitIp) {
                return new GeneralResult<>(false, "创建失败：IP{" + exsitIp + "}已存在");
            }
            host.setDeleted(false);
            host.setSourceType(TaskHostType.SourceType.CREATE.name());
            host.setOsCategory(
                null != host.getOsCategory() ? host.getOsCategory().toUpperCase() : host.getOsCategory());
            host.setCatalog("Computer");
            Long segmentId = host.getSegmentId();
            TaskHostDataUtil.setIp(host, segmentList);// 自动配置网段
            if (null == host.getSegmentId() && null != segmentId) {
                host.setSegmentId(segmentId);
                host.setUpdateMode(TaskHostType.UpdateModel.MANUAL.name());
            }
            if (!Collections.isEmpty(host.getConnProtocols())) {
                host.setConnProtocol(JSONArray.toJSONString(host.getConnProtocols()));
            }
            host.setCreatorId(context.getTarget());
            host.setMenderId(context.getTarget());
            host.setOwnerId(context.getTarget());
            host.setGmtCreate(new Date());
            host.setGmtModify(new Date());
            host.setTenantId(null == context.getTenant() ? 0L : context.getTenant());
            // 租户端创建的如果没有连接协议，默认为remote
            if (null != context.getTenant() && !context.getTenant().equals(0L) && null == host.getConnProtocol()) {
                List<String> connProtocols = new ArrayList<>();
                connProtocols.add("Remote");
                host.setConnProtocol(JSONArray.toJSONString(connProtocols));
            }
            host.setDeleted(false);
            host.setUserCatalog(null != context.getCatalog() ? context.getCatalog().name() : null);
            taskHostRepository.save(host);
            if (!Collections.isEmpty(host.getIdentificationIds())) {
                // 添加关联账号
                addIdentificationRelation(host.getIdentificationIds(), host.getId());
            }
            return new GeneralResult<>(true, "添加成功!");
        } catch (Exception e) {
            log.error("create host failure:", e);
            return new GeneralResult<>(false, "添加失败!");
        }
    }

    /**
     * 新增主机资源与认证账号关系
     *
     * @param identIds
     *            认证账号id列表
     * @param hostId
     *            主机资源
     */
    private void addIdentificationRelation(List<Long> identIds, Long hostId) {
        List<ResourceIdentificationRelation> relations = new ArrayList<>();
        identIds.forEach(identId -> relations.add(new ResourceIdentificationRelation(hostId, identId, ResourceType.HOST.name())));
        if (relations.size() > 0) {
            identificationRelationRepository.batchSave(relations, 1000);
        }
    }

    @Override
    public GeneralResult<Void> modify(TaskHostBean bean, RequestContext context) {
        String path = TaskHost.class.getSimpleName() + "_" + bean.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "有其他任务正在更新");
            TaskHost host = taskHostRepository.query(bean.getId());
            if (null == host) {
                log.warn("host data not exist");
                return new GeneralResult<>(false, "数据不存在");
            }
            if (!host.getIps().equals(bean.getIps())) {
                List<TaskHost> taskHosts = taskHostRepository.listAllNoVendor();
                Set<String> allIps = TaskHostDataUtil.getAllIp(taskHosts, bean.getId());
                String exsitIp = TaskHostDataUtil.checkIp(bean.getIpsStr(), allIps);
                if (null != exsitIp) {
                    return new GeneralResult<>(false, "修改失败：IP{" + exsitIp + "}已存在");
                }
            }
            bean.setOsCategory(
                null != bean.getOsCategory() ? bean.getOsCategory().toUpperCase() : bean.getOsCategory());
            List<NetworkSegment> segmentList = networkSegmentRepository.list();
            BeanUtils.copyProperties(bean, host);
            Long segmentId = host.getSegmentId();
            String updateMode = host.getUpdateMode();
            TaskHostDataUtil.setIp(host, segmentList);// 自动配置网段
            if (null == host.getSegmentId() && TaskHostType.UpdateModel.MANUAL.name().equals(updateMode)) {
                host.setSegmentId(segmentId);
                host.setUpdateMode(updateMode);
            }
            if (!Collections.isEmpty(host.getConnProtocols())) {
                host.setConnProtocol(JSONArray.toJSONString(host.getConnProtocols()));
            } else {
                host.setConnProtocol(null);
            }
            host.setMenderId(context.getTarget());
            host.setGmtModify(new Date());
            boolean result = taskHostRepository.update(host);
            List<Long> identIds = host.getIdentificationIds();
            if (result && !Collections.isEmpty(identIds)) {
                // 需要对关联账号进行变更
                List<Long> oldIdentIds = identificationRelationRepository
                    .listIdentificationIdsByTaskHostId(host.getId(), ResourceType.HOST.name());
                identIds.forEach(identId -> {
                    if (oldIdentIds.contains(identId)) {
                        oldIdentIds.remove(identId);
                    } else {
                        identificationRelationRepository
                            .save(new ResourceIdentificationRelation(host.getId(), identId, ResourceType.HOST.name()));
                    }
                });
                if (!Collections.isEmpty(oldIdentIds)) {
                    identificationRelationRepository.removeByTaskHostId(host.getId(), oldIdentIds,
                        ResourceType.HOST.name());
                }
            }
            return result ? new GeneralResult<>(true, "修改成功") : new GeneralResult<>(false, "修改失败");
        } catch (Exception e) {
            log.error("modify host fail:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id) {
        String path = TaskHost.class.getSimpleName() + "_" + id;
        boolean result;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskHost host = taskHostRepository.query(id);
            Assert.notNull(host, "主机不存在！");
            result = taskHostRepository.remove(id);
            if (result) {
                // 需要删除关联的认证账号
                identificationRelationRepository.removeByTaskHostId(id, ResourceType.HOST.name());
            }
        } catch (Exception e) {
            log.error("remove host fail:", e);
            return new GeneralResult<>(false, "删除失败");
        }
        return result ? new GeneralResult<>(true, "删除成功") : new GeneralResult<>(false, "删除失败");
    }

    @Override
    public GeneralResult<TaskHost> detail(Long id) {
        try {
            TaskHost host = taskHostRepository.query(id);
            if (null != host) {
                // 添加权限信息
                host.setUserGroupIds(permissionRelationRepository.listPermissionIdsByTaskHostId(host.getId(),
                    PermissionType.USERGROUP.name(), ResourceType.HOST.name()));
                // 添加认证信息
                host.setIdentificationIds(identificationRelationRepository
                    .listIdentificationIdsByTaskHostId(host.getId(), ResourceType.HOST.name()));
                if (StringUtils.isNotBlank(host.getConnProtocol())) {
                    host.setConnProtocol(host.getConnProtocol().replaceAll("\\\\",""));
                    host.setConnProtocols(JSONArray.parseArray(host.getConnProtocol(), String.class));
                }
            }
            Assert.notNull(host, "主机不存在！");
            return new GeneralResult<>(true, host, "获取详情成功!");
        } catch (Exception e) {
            log.error("get host fail", e);
            return new GeneralResult<>(false, "获取详情失败!");
        }
    }

    @Override
    public GeneralResult<GridBean<TaskHost>> list(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<Param> params = pager.getParams();
            boolean isUnknow = false;
            for (Param param : params) {
                // 判断是否查询未知
                if (null == param.getParam() || !param.getParam().containsKey("status")) {
                    continue;
                }
                Object status = param.getParam().get("status");
                if (null != status && "UNKNOWSTATUS".equalsIgnoreCase(status.toString())) {
                    param.getParam().remove("status");
                    isUnknow = true;
                    break;
                }
            }
            int total;
            List<TaskHost> hosts;
            if (RequestContext.Catalog.Tenant == context.getCatalog()) {
                pager.getParams().add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
            } else {
                pager.getParams().add(new Param(MapTools.simpleMap("userCatalog", context.getCatalog()), Sign.EQ));
            }
            // 所有用户能查看的主机资源都是通过用户组确定的
            // 获取用户的用户组
            GeneralResult<List<UserGroupRelationBean>> result =
                smsInternalService.listUserGroupRelation(context.getTarget());
            if (result.isFailed()) {
                log.error("主机资源列表查询失败:无法获取用户组信息{} ", result.getMessage());
                return new GeneralResult<>(false, "查询主机资源失败:用户组获取失败");
            }
            List<Long> groupIds =
                result.getData().stream().map(UserGroupRelationBean::getUserGroupId).collect(Collectors.toList());
            if (isUnknow) {
                total = taskHostRepository.countForUnknowStatus(pager.getParams(), TaskHostDataUtil.getKnowStatus(),
                    groupIds, context);
                hosts =
                    taskHostRepository.listForUnkownStatus(pager, TaskHostDataUtil.getKnowStatus(), groupIds, context);
            } else {
                total = taskHostRepository.count(pager.getParams(), groupIds, context);
                hosts = taskHostRepository.list(pager, groupIds, context);
            }
            Map<String, String> dcMap = getDCMap();
            hosts.forEach(host -> {
                host.setDataCenterName(dcMap.get(host.getDataCenter()));
                host.setRoomName(dcMap.get(host.getDataCenter() + "_" + host.getRoom()));
                host.setRackName(dcMap.get(host.getDataCenter() + "_" + host.getRoom() + "_" + host.getRack()));
                host.setSlotName(dcMap
                    .get(host.getDataCenter() + "_" + host.getRoom() + "_" + host.getRack() + "_" + host.getSlot()));
                if (StringUtils.isBlank(host.getConnProtocol())) {
                    return;
                }
                try {
                    host.setConnProtocol(host.getConnProtocol().replaceAll("\\\\",""));
                    host.setConnProtocols(JSONArray.parseArray(host.getConnProtocol(), String.class));
                } catch (Exception e) {
                    log.error("连接协议转换失败:{} {}", host.getId(), host.getConnProtocol());
                }
            });
            GridBean<TaskHost> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, hosts);
            return new GeneralResult<>(true, bean, "查询主机资源成功");
        } catch (Exception e) {
            log.error("list hosts fail:", e);
            return new GeneralResult<>(false, "查询主机资源失败");
        }
    }

    @Override
    public GeneralResult<GridBean<TaskHost>> listTaskHostAndIdentification(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<Param> params = pager.getParams();
            boolean isUnknow = false;
            for (Param param : params) {
                // 判断是否查询未知
                if (null == param.getParam() || !param.getParam().containsKey("status")) {
                    continue;
                }
                Object status = param.getParam().get("status");
                if (null != status && "UNKNOWSTATUS".equalsIgnoreCase(status.toString())) {
                    param.getParam().remove("status");
                    isUnknow = true;
                    break;
                }
            }

            int total;
            List<TaskHost> hosts;
            if (RequestContext.Catalog.Tenant == context.getCatalog()) {
                pager.getParams().add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
            } else {
                pager.getParams().add(new Param(MapTools.simpleMap("userCatalog", context.getCatalog()), Sign.EQ));
            }
            // 所有用户能查看的主机资源都是通过用户组确定的
            // 获取用户的用户组
            GeneralResult<List<UserGroupRelationBean>> result =
                smsInternalService.listUserGroupRelation(context.getTarget());
            if (result.isFailed()) {
                log.error("主机资源列表查询失败:无法获取用户组信息{} ", result.getMessage());
                return new GeneralResult<>(false, "查询主机资源失败:用户组获取失败");
            }
            List<UserGroupRelationBean> relationBeans = result.getData();
            List<Long> groupIds =
                relationBeans.stream().map(UserGroupRelationBean::getUserGroupId).collect(Collectors.toList());
            if (isUnknow) {
                total = taskHostRepository.countForUnknowStatus(pager.getParams(), TaskHostDataUtil.getKnowStatus(),
                    groupIds, context);
                hosts =
                    taskHostRepository.listForUnkownStatus(pager, TaskHostDataUtil.getKnowStatus(), groupIds, context);
            } else {
                total = taskHostRepository.count(pager.getParams(), groupIds, context);
                hosts = taskHostRepository.list(pager, groupIds, context);
            }
            Map<String, String> dcMap = getDCMap();
            hosts.forEach(host -> {
                try {
                    List<Map<String, Object>> accounts = new ArrayList<>();
                    // 添加认证用户
                    List<Identification> idents =
                        identificationRepository.listByResource(host.getId(), ResourceType.HOST.name());
                    idents.forEach(ident -> {
                        Map<String, Object> account = new HashMap<>();
                        account.put("id", ident.getId());
                        account.put("accountName", ident.getAccountName() + "(" + ident.getUsername() + ")");
                        accounts.add(account);
                    });
                    // 对密码进行处理
                    host.setIdentifications(accounts);
                } catch (Exception e) {
                    log.error("获取认证用户失败:{}", host.getId());
                }
                host.setDataCenterName(dcMap.get(host.getDataCenter()));
                host.setRoomName(dcMap.get(host.getDataCenter() + "_" + host.getRoom()));
                host.setRackName(dcMap.get(host.getDataCenter() + "_" + host.getRoom() + "_" + host.getRack()));
                host.setSlotName(dcMap
                    .get(host.getDataCenter() + "_" + host.getRoom() + "_" + host.getRack() + "_" + host.getSlot()));
                if (StringUtils.isBlank(host.getConnProtocol())) {
                    return;
                }
                try {
                    host.setConnProtocol(host.getConnProtocol().replaceAll("\\\\",""));
                    host.setConnProtocols(JSONArray.parseArray(host.getConnProtocol(), String.class));
                } catch (Exception e) {
                    log.error("连接协议转换失败:{} {}", host.getId(), host.getConnProtocol());
                }
            });
            GridBean<TaskHost> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, hosts);
            return new GeneralResult<>(true, bean, "查询主机资源成功");
        } catch (Exception e) {
            log.error("list hosts fail:", e);
            return new GeneralResult<>(false, "查询主机资源失败");
        }
    }

    @Override
    public GeneralResult<GridBean<TaskHost>> listForManage(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<Param> params = pager.getParams();
            boolean isUnknow = false;
            Long userGroupId = null;
            for (Param param : params) {
                // 判断是否查询未知
                if (null == param.getParam() || !(param.getParam().containsKey("status")
                    || param.getParam().containsKey("userGroupId"))) {
                    continue;
                }
                if (param.getParam().containsKey("status")) {
                    Object status = param.getParam().get("status");
                    if (null != status && "UNKNOWSTATUS".equalsIgnoreCase(status.toString())) {
                        param.getParam().remove("status");
                        isUnknow = true;
                        break;
                    }
                } else {
                    Object userGroupIdObj = param.getParam().get("userGroupId");
                    if (null != userGroupIdObj) {
                        userGroupId = Long.valueOf(userGroupIdObj.toString());
                        param.getParam().remove("userGroupId");
                    }
                }
            }
            // 管理功能可以查看所有的主机资源
            int total;
            List<TaskHost> hosts;
            if (isUnknow) {
                total = taskHostRepository.countForUnknowStatus(pager.getParams(), TaskHostDataUtil.getKnowStatus(),
                    userGroupId);
                hosts = taskHostRepository.listForUnkownStatus(pager, TaskHostDataUtil.getKnowStatus(), userGroupId);
            } else {
                total = taskHostRepository.count(pager.getParams(), userGroupId);
                hosts = taskHostRepository.list(pager, userGroupId);
            }
            Map<Long, String> groupNameMap = new HashMap<>();
            hosts.forEach(taskHost -> {
                try {
                    List<ResourcePermissionRelation> relations = permissionRelationRepository.listRelationByTaskHostId(
                        taskHost.getId(), PermissionType.USERGROUP.name(), ResourceType.HOST.name());
                    // 通过redis获取用户组名
                    List<String> groupNames = new ArrayList<>();
                    relations.forEach(relation -> {
                        String groupName = groupNameMap.get(relation.getPermissionId());
                        if (StringUtils.isBlank(groupName)) {
                            groupName = redisTemplate.opsForValue().get("Group_Name_" + relation.getPermissionId());
                            groupNameMap.put(relation.getPermissionId(), groupName);
                        }
                        if (StringUtils.isNotBlank(groupName)) {
                            groupNames.add(groupName);
                        }
                    });
                    taskHost.setUserGroupNames(groupNames);
                } catch (Exception e) {
                    log.error("获取用户组失败:{} {} ", taskHost.getId(), e);
                }
                if (StringUtils.isNotBlank(taskHost.getConnProtocol())) {
                    try {
                        taskHost.setConnProtocol(taskHost.getConnProtocol().replaceAll("\\\\",""));
                        taskHost.setConnProtocols(JSONArray.parseArray(taskHost.getConnProtocol(), String.class));
                    } catch (Exception e) {
                        log.error("连接协议转换失败:{} {}", taskHost.getId(), taskHost.getConnProtocol());
                    }
                }
                if (null != taskHost.getTenantId() && taskHost.getTenantId() != 0) {
                    taskHost.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + taskHost.getTenantId()));
                }
            });
            GridBean<TaskHost> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, hosts);
            return new GeneralResult<>(true, bean, "查询主机资源成功");
        } catch (Exception e) {
            log.error("list hosts fail:", e);
            return new GeneralResult<>(false, "查询主机资源失败");
        }
    }

    @Override
    public GeneralResult<Void> configure(Long hostId, Long segmentId) {
        try {
            TaskHost host = taskHostRepository.query(hostId);
            Assert.notNull(host, "主机数据不存在");
            NetworkSegment segment = networkSegmentRepository.query(segmentId);
            Assert.notNull(segment, "网段数据不存在");
            host.setSegmentId(segmentId);
            host.setUpdateMode(TaskHostType.UpdateModel.MANUAL.name());
            boolean update = taskHostRepository.update(host);
            return update ? new GeneralResult<>(true, "配置网段成功") : new GeneralResult<>(false, "配置网段失败");
        } catch (DataBaseException e) {
            log.error("Configure task host fail:", e);
            return new GeneralResult<>(false, "配置网段失败");
        }
    }

    @Override
    public GeneralResult<Task> configHostAllow(Long hostId) {
        try (AutoCloseLock lock = lockFactory.getACLock("Host_Allow_" + hostId)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "有其他任务正在配置");
            TaskHost host = taskHostRepository.query(hostId);
            Assert.notNull(host, "主机数据不存在");
            if (null == host.getSegmentId()) {
                log.warn("主机网段没有配置");
                return new GeneralResult<>(false, "主机网段没有配置");
            }
            if (StringUtils.isEmpty(host.getIp())) {
                log.warn("主机没有可用IP地址");
                return new GeneralResult<>(false, "主机没有可用IP地址");
            }
            NetworkSegment segment = networkSegmentRepository.query(host.getSegmentId());
            if (null == segment) {
                log.warn("主机网段数据不存在");
                return new GeneralResult<>(false, "主机网段数据不存在");
            }
            NetworkGroup group = networkGroupRepository.query(segment.getGroupId());
            if (null == group) {
                log.warn("主机网段分组数据不存在");
                return new GeneralResult<>(false, "主机网段分组数据不存在");
            }
            List<TaskWorker> workers =
                taskWorkerRepository.listGenerals(group.getId(), TaskWorker.Category.Ansible.name());
            if (ListTool.isEmpty(workers)) {
                log.warn("主机网段分组不存在worker");
                return new GeneralResult<>(false, "主机网段分组不存在worker");
            }
            Task task = HostsAllowTaskBuilder.getConfigTask(host, workers);
            return new GeneralResult<>(true, task, "success");
        } catch (Exception e) {
            log.error("Configure host allow task fail:", e);
            return new GeneralResult<>(false, "构建主机HostsAllow配置任务失败");
        }
    }

    @Override
    public void syncHostFromIMS(RequestContext context, String category) {
        taskExecutor.execute(() -> {
            getOsNameMap();
            String path = TaskHost.class.getSimpleName() + "_ims_sync";
            String message;
            NotifyMessage notifyMessage;
            try (AutoCloseLock lock = lockFactory.getACLock(path)) {
                Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "有其他任务正在同步");
                log.info("主机资源IMS数据同步开始");
                // 获取所有纳管的云供应商
                List<TaskHostVendorBean> vendors = getAllVendors();
                if (null == vendors) {// 接口调用异常不做任何处理
                    return;
                }
                // 查询所有网段
                List<NetworkSegment> segments = networkSegmentRepository.list();
                // 原有平台
                List<TaskHost> oldVendorList;
                if (StringUtils.isBlank(category) || TaskHostType.Category.ALL.name().equalsIgnoreCase(category)) {
                    oldVendorList = taskHostRepository.listVendorId(TaskHostType.SourceType.IMS.name());
                } else {
                    oldVendorList = taskHostRepository.listVendorId(category, TaskHostType.SourceType.IMS.name());
                }
                log.info("自动化运维获取ims云平台：" + JSONObject.toJSONString(oldVendorList));
                // 同步获取的平台
                List<Long> oldVendorIds = TaskHostDataUtil.getVendorIds(oldVendorList);
                // 对同步的平台数据进行处理
                for (TaskHostVendorBean vendor : vendors) {
                    boolean pmFlag;
                    boolean vmFlag = true;
                    // 根据平台id同步主机信息
                    if (StringUtils.isBlank(category) || TaskHostType.Category.ALL.name().equals(category)) {
                        pmFlag = syncHostByVendorId(vendor.getId(), segments, TaskHostType.Category.PHYSICAL.name(), context);
                        vmFlag = syncHostByVendorId(vendor.getId(), segments, TaskHostType.Category.LOGICAL.name(), context);
                    } else {
                        pmFlag = syncHostByVendorId(vendor.getId(), segments, category, context);
                    }
                    oldVendorIds.remove(vendor.getId());
                    if (!pmFlag || !vmFlag) {
                        // 发送站内信
                        notifyMessage =
                            new NotifyMessage(context, subject, "平台" + vendor.getName() + "同步失败", true, false);
                        amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(),
                            CommonConstant.MESSAGE_QUEUE_ROUTINGKEY, notifyMessage);
                    }
                }
                // 未处理的原有平台，说明已经被删除
                for (Long vendorId : oldVendorIds) {
                    // 根据平台id删除所有同步信息
                    if (StringUtils.isBlank(category) || TaskHostType.Category.ALL.name().equalsIgnoreCase(category)) {
                        // 删除关系(平台下所有主机资源)
                        List<TaskHost> taskHosts =
                            taskHostRepository.listByVendor(vendorId, TaskHostType.SourceType.IMS.name());
                        removeRelationByTaskHost(taskHosts);
                        // 删除平台下所有主机资源
                        taskHostRepository.removeByVendorId(vendorId, TaskHostType.SourceType.IMS.name());
                    } else {
                        // 删除关系(平台下指定类型的主机资源)
                        List<TaskHost> taskHosts =
                            taskHostRepository.listByVendor(category, vendorId, TaskHostType.SourceType.IMS.name());
                        removeRelationByTaskHost(taskHosts);
                        // 删除平台下指定类型的主机资源
                        taskHostRepository.removeByVendorId(vendorId, TaskHostType.SourceType.IMS.name(), category);
                    }
                }
                log.info("主机资源IMS{}数据同步完成", null == category ? TaskHostType.Category.ALL.name() : category);
                notifyMessage = new NotifyMessage(context, subject, "主机资源同步完成", true, false);
                amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(),
                    CommonConstant.MESSAGE_QUEUE_ROUTINGKEY, notifyMessage);
            } catch (Exception e) {
                message = "主机资源同步失败:" + e;
                log.error("IMS数据同步失败:", e);
                // 发送站内信
                notifyMessage = new NotifyMessage(context, subject, message, true, false);
                amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(),
                    CommonConstant.MESSAGE_QUEUE_ROUTINGKEY, notifyMessage);
            }
        });
    }

    /**
     * 获取IMS全量的计算平台id
     *
     * @return 所有云平台
     */
    private List<TaskHostVendorBean> getAllVendors() {
        List<TaskHostVendorBean> vendors = new ArrayList<>();
        try {
            GeneralResult<?> result = cmpInternetService.listVendorIdByVendorCategory("COMPUTE");
            if (result.isFailed()) {
                log.error("调用IMS接口/v1/vendors失败{}", result.getMessage());
                return null;
            }
            if (null == result.getData()) {
                log.error("调用IMS接口/v1/vendors失败返回值为null");
                return null;
            }
            // 对返回对象进行解析获取vendorIds
            JSONObject data = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
            JSONArray rows = data.getJSONArray("rows");
            Map<Long, TaskHostVendorBean> beanMap = new HashMap<>();
            for (int i = 0; i < rows.size(); i++) {
                Long vendorId = rows.getJSONObject(i).getLong("id");
                String vendorName = rows.getJSONObject(i).getString("name");
                String vendorType = rows.getJSONObject(i).getString("type");
                if (null == vendorId) {
                    continue;
                }
                TaskHostVendorBean bean = new TaskHostVendorBean(vendorId, vendorName, vendorType);
                vendors.add(bean);
                beanMap.put(vendorId, bean);
            }
            // 更新静态map
            vendorBeanMap = beanMap;
            return vendors;
        } catch (Exception e) {
            log.error("调用IMS接口/v1/vendors失败:", e);
            return null;
        }
    }


    /**
     * 根据平台id同步主机信息
     * @param vendorId 云平台id
     * @param segments 网段列表
     * @param category 服务器类型
     * @param context 操作人信息
     * @return 操作结果
     */
    private boolean syncHostByVendorId(Long vendorId, List<NetworkSegment> segments, String category,
        RequestContext context) {
        // 获取全量宿主机
        try {
            List<TaskHost> newTaskHosts;
            if (TaskHostType.Category.LOGICAL.name().equals(category)) {
                GeneralResult<?> result = cmpInternetService.listVmByVendor(vendorId);
                if (result.isFailed()) {
                    log.error("调用IMS接口/v1/vms失败{}:{}", vendorId, result.getMessage());
                    return false;
                }
                if (null == result.getData()) {
                    log.error("调用IMS接口/v1/vms失败返回值为null");
                    return false;
                }
                // 对返回对象进行解析获取vendorIds
                JSONObject data = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
                newTaskHosts = TaskHostDataUtil.convertVmToTaskHost(data, context, getOsNameMap());
            } else if (TaskHostType.Category.PHYSICAL.name().equals(category)) {
                GeneralResult<?> result = cmpInternetService.listHostByVender(vendorId);
                if (result.isFailed()) {
                    log.error("调用IMS接口/v1/hosts失败{}", result.getMessage());
                    return false;
                }
                if (null == result.getData()) {
                    log.error("调用IMS接口/v1/hosts失败返回值为null");
                    return false;
                }
                // 对返回对象进行解析获取vendorIds
                JSONObject data = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
                newTaskHosts = TaskHostDataUtil.convertPmToTaskHost(data, context);
            } else {
                log.error("不支持的同步类型:{}", category);
                return false;
            }

            // 对平台同步的主机数据进行处理
            List<TaskHost> addTaskHosts = new ArrayList<>();
            List<TaskHost> updateHosts = new ArrayList<>();
            List<TaskHost> singleAddHosts = new ArrayList<>();
            List<TaskHost> singleUpdateHosts = new ArrayList<>();
            List<TaskHost> oldTaskHosts =
                taskHostRepository.listByVendor(category, vendorId, TaskHostType.SourceType.IMS.name());
            Map<String, TaskHost> oldTaskHostMap =
                oldTaskHosts.stream().collect(Collectors.toMap(TaskHost::getInstanceId, t -> t, (v1, v2) -> v2));
            Set<String> oldTaskHostMapKey = oldTaskHostMap.keySet();
            List<Long> removeTaskHostIds = oldTaskHosts.stream().map(TaskHost::getId).collect(Collectors.toList());
            // 数据变更
            newTaskHosts.forEach(taskHost -> {
                if (oldTaskHostMapKey.contains(taskHost.getInstanceId())) {
                    // 编辑
                    TaskHost oldTaskHost = oldTaskHostMap.get(taskHost.getInstanceId());
                    if (null != taskHost.getIps()) {
                        //根据ips和网段集合 设置ip以及网段数据
                        TaskHostDataUtil.setIp(taskHost, segments);
                    }
                    if (null == taskHost.getSegmentId()
                        && TaskHostType.UpdateModel.MANUAL.name().equalsIgnoreCase(oldTaskHost.getUpdateMode())) {
                        taskHost.setSegmentId(oldTaskHost.getSegmentId());
                        taskHost.setUpdateMode(oldTaskHost.getUpdateMode());
                    }
                    TaskHostDataUtil.updateTaskHost(taskHost, oldTaskHost);
                    if ((StringUtils.isNotBlank(oldTaskHost.getName()) && oldTaskHost.getName().contains("'"))) {
                        singleUpdateHosts.add(oldTaskHost);
                    } else {
                        updateHosts.add(oldTaskHost);
                    }
                    oldTaskHostMapKey.remove(taskHost.getInstanceId());
                    removeTaskHostIds.remove(oldTaskHost.getId());
                } else {
                    // 新增
                    if (null != taskHost.getIps()) {
                        TaskHostDataUtil.setIp(taskHost, segments);
                    }
                    if ((StringUtils.isNotBlank(taskHost.getName()) && taskHost.getName().contains("'"))) {
                        singleAddHosts.add(taskHost);
                    } else {
                        addTaskHosts.add(taskHost);
                    }
                }
            });
            // 新增
            if (addTaskHosts.size() > 0) {
                taskHostRepository.batchSave(addTaskHosts, 1000);
            }
            // 修改
            if (updateHosts.size() > 0) {
                taskHostRepository.batchUpdate(updateHosts, 1000);
            }
            //单条新增
            singleAddHosts.forEach(host -> taskHostRepository.save(host));
            //单条修改
            singleUpdateHosts.forEach(host -> taskHostRepository.update(host));
            // 删除
            removeTaskHostIds.forEach(taskHostId -> {
                taskHostRepository.remove(taskHostId);
                identificationRelationRepository.removeByTaskHostId(taskHostId, ResourceType.HOST.name());
                permissionRelationRepository.removeByTaskHostId(taskHostId, ResourceType.HOST.name());
            });
        } catch (Exception e) {
            log.error("平台:{}主机资源同步失败:{}", vendorId, e);
            return false;
        }
        return true;
    }

    @Override
    public GeneralResult<Void> upload(MultipartFile excel, RequestContext context) throws IOException {
        try {
            if (null == excel) {
                return new GeneralResult<>(false, "批量导入主机资源失败");
            }
            String name = excel.getOriginalFilename();
            if (StringUtils.isEmpty(name)) {
                return new GeneralResult<>(false, "批量导入主机资源失败");
            }
            String fileName = URLDecoder.decode(name, "UTF-8");
            ExcelFactory factory = new ExcelFactory(excel.getInputStream(), fileName);
            taskExecutor.execute(() -> {
                String path = TaskHost.class.getSimpleName() + "_update";
                try (AutoCloseLock lock = lockFactory.getACLock(path)) {
                    Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "有其他导入任务正在进行");
                    TaskHostExcelHelper excelHelper = new TaskHostExcelHelper(taskHostRepository,
                        networkSegmentRepository, amqpTemplate, resourceEventPublisher);
                    excelHelper.readExcel(factory.newWorkbook(), context);
                } catch (Exception e) {
                    log.error("批量导入主机资源失败", e);
                }
            });
        } catch (Exception e) {
            log.error("批量导入主机资源失败", e);
            return new GeneralResult<>(false, "批量导入主机资源失败");
        }
        return new GeneralResult<>(true, "批量导入主机资源任务已下发");
    }

    @Override
    public GeneralResult<?> download(Map<String, Object> paramsMap) {
        Workbook workbook;
        try {
            // 校验前端传入的excel的版本（2000或2007），生成后缀。当前为前端写死为2007版本
            String fileName;
            if ("2000".equals(BoStringUtils.toString(paramsMap.get("excelType")))) {
                fileName = "主机资源.xls";
            } else {
                fileName = "主机资源.xlsx";
            }
            workbook = new ExcelFactory(fileName).newWorkbook();
            TaskHostExcelHelper exportHelper = new TaskHostExcelHelper();
            // 生成模板
            exportHelper.generateSheet(workbook);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            workbook.write(baos);
            byte[] ba = baos.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", fileName);
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new GeneralResult<>(true, resultMap, "下载成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public void modifyByMessage(String action, JSONObject data, String category) {
        if (null == data) {
            return;
        }
        String path = TaskHost.class.getSimpleName() + "_ims_sync";
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "有其他任务正在同步");
            List<NetworkSegment> segments = networkSegmentRepository.list();
            if (TaskHostType.Action.DEL.name().equalsIgnoreCase(action)) {
                String instacneId = "";
                if (TaskHostType.Category.PHYSICAL.name().equals(category)) {
                    instacneId = data.getString("uuid");
                } else if (TaskHostType.Category.LOGICAL.name().equals(category)) {
                    instacneId = data.getString("instanceId");
                }
                Long vendorId = data.getLong("vendorId");
                if (StringUtils.isBlank(instacneId) || null == vendorId) {
                    return;
                }
                Long hostId = taskHostRepository.queryId(instacneId, vendorId, category);
                if (null != hostId) {
                    taskHostRepository.delete(hostId);
                    // 需要删除关联的认证账号
                    identificationRelationRepository.removeByTaskHostId(hostId, ResourceType.HOST.name());
                    permissionRelationRepository.removeByTaskHostId(hostId, ResourceType.HOST.name());
                }
                return;
            }
            if (TaskHostType.Action.DELYVID.name().equalsIgnoreCase(action)) {
                Long vendorId = data.getLong("vendorId");
                if (null == vendorId) {
                    return;
                }
                List<TaskHost> hosts = taskHostRepository.listByVendor(vendorId, TaskHostType.SourceType.IMS.name());
                removeRelationByTaskHost(hosts);
                taskHostRepository.removeByVendorId(vendorId, TaskHostType.SourceType.IMS.name());
                return;
            }

            // JSON转换成tasHost
            TaskHost taskHost = TaskHostDataUtil.convertMessageToTaskHost(data, null, getOsNameMap());
            if (null == taskHost) {// 过滤的数据
                return;
            }
            TaskHost oldTaskHost = taskHostRepository.queryForIms(taskHost.getInstanceId(), taskHost.getVendorId(),
                taskHost.getCategory());
            if (null != oldTaskHost) { // 编辑
                if (null != taskHost.getIps()) {
                    TaskHostDataUtil.setIp(taskHost, segments);
                }
                if (null == taskHost.getSegmentId()
                    && TaskHostType.UpdateModel.MANUAL.name().equalsIgnoreCase(oldTaskHost.getUpdateMode())) {
                    taskHost.setSegmentId(oldTaskHost.getSegmentId());
                    taskHost.setUpdateMode(oldTaskHost.getUpdateMode());
                }
                TaskHostDataUtil.updateTaskHost(taskHost, oldTaskHost);
                if (null != oldTaskHost.getDeleted() && oldTaskHost.getDeleted()) {
                    taskHostRepository.remove(oldTaskHost.getId());// 默认的update方法无法修改isDelete字段为true
                } else {
                    oldTaskHost.setIps(oldTaskHost.getIps());
                    taskHostRepository.update(oldTaskHost);
                }
            } else if (TaskHostType.Action.ADD.name().equals(action)
                || TaskHostType.Action.UPDATE.name().equals(action)) { // 新增
                // 如果是模板，不新增
                if (null != taskHost.getIsTemplate() && taskHost.getIsTemplate()) {
                    return;
                }
                if (null != taskHost.getIps()) {
                    TaskHostDataUtil.setIp(taskHost, segments);
                }
                // 如果是新增，并且没有vendorName与vendorType,从静态平台map中获取
                // （每次调用平台会重置静态平台Map，平台新增需要手动同步，平台删除会调用删除操作，理论上平台map应该是准的）
                if (StringUtils.isBlank(taskHost.getVendorName()) || StringUtils.isBlank(taskHost.getVendorType())) {
                    TaskHostVendorBean vendorBean = vendorBeanMap.get(taskHost.getVendorId());
                    if (null != vendorBean && StringUtils.isBlank(taskHost.getVendorName())) {
                        taskHost.setVendorName(vendorBean.getName());
                    }
                    if (null != vendorBean && StringUtils.isBlank(taskHost.getVendorType())) {
                        taskHost.setVendorType(vendorBean.getType());
                    }
                }
                taskHost.setIps(taskHost.getIps());
                taskHostRepository.save(taskHost);
            } else {
                log.error("主机资源增量同步失败：不支持的操作类型{}", data.toJSONString());
            }
        } catch (Exception e) {
            log.error("主机资源增量同步失败:", e);
        }
    }

    @Override
    public void syncSegment() {
        taskExecutor.execute(() -> {
            String path = TaskHost.class.getSimpleName() + "_sync_segment";
            try (AutoCloseLock lock = lockFactory.getACLock(path)) {
                Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "有其他任务正在更新");
                List<NetworkSegment> segments = networkSegmentRepository.list();
                List<TaskHost> hosts = taskHostRepository.listAllNoVendor();
                List<TaskHost> updateHosts = new ArrayList<>();
                List<TaskHost> singleUpdateList = new ArrayList<>();
                for (TaskHost host : hosts) {
                    String oldIp = host.getIp();
                    Long oldSegmentId = host.getSegmentId();
                    String oldUpdateMode = host.getUpdateMode();
                    if (null != host.getIps()) {
                        TaskHostDataUtil.setIp(host, segments);
                    }
                    if (null == host.getSegmentId()
                        && TaskHostType.UpdateModel.MANUAL.name().equalsIgnoreCase(oldUpdateMode)) {
                        continue;
                    }
                    if (TaskHostDataUtil.compare(oldIp, host.getIp())
                        && TaskHostDataUtil.compare(oldSegmentId, host.getSegmentId())) {
                        continue;
                    }
                    if (StringUtils.isNotBlank(host.getName()) && host.getName().contains("'")) {
                        singleUpdateList.add(host);
                    } else {
                        updateHosts.add(host);
                    }
                }
                if (updateHosts.size() > 0) {
                    taskHostRepository.batchUpdate(updateHosts, 100);
                }
                singleUpdateList.forEach(host -> taskHostRepository.update(host));
            } catch (Exception e) {
                log.error("主机资源ip校验失败", e);
            }
        });
    }

    @Override
    public GeneralResult<NetworkSegment> hasWorker(String host) {
        try {
            if ("localhost".equalsIgnoreCase(host)) {
                // localhost在超级worker上执行，判断超级worker数量
                List<TaskWorker> supers = taskWorkerRepository.listSupers(TaskWorker.Category.Ansible.name());
                if (supers.size() > 0) {
                    return new GeneralResult<>(true, new NetworkSegment(), "success");
                }
                return new GeneralResult<>(false, "当前平台未配置超级worker");
            }
            List<NetworkSegment> segments = networkSegmentRepository.list();
            if (ListTool.isEmpty(segments)) {
                return new GeneralResult<>(false, "平台未配置网段信息");
            }
            NetworkSegment segment = TaskHostDataUtil.getSegment(host, segments);
            if (null == segment) {
                return new GeneralResult<>(false, "目标机器未匹配到对应网段信息");
            }
            if (null == segment.getGroupId()) {
                return new GeneralResult<>(false, "目标机器匹配的网段未配置分组");
            }
            List<TaskWorker> workers =
                taskWorkerRepository.listGenerals(segment.getGroupId(), TaskWorker.Category.Ansible.name());
            if (ListTool.isEmpty(workers)) {
                return new GeneralResult<>(false, "网段对应分组未配置执行worker");
            }
            return new GeneralResult<>(true, segment, "success");
        } catch (Exception e) {
            log.error("check host has worker fail:", e);
            return new GeneralResult<>(false, "未找到可执行worker");
        }
    }

    @Override
    public GeneralResult<Void> configPermissionRelation(List<Long> ids, List<Long> groupIds) {
        try {
            List<ResourcePermissionRelation> relations = new ArrayList<>();
            if (Collections.isEmpty(ids)) {
                return new GeneralResult<>(false, "权限配置失败:缺少必要参数");
            }
            ids.forEach(id -> {
                permissionRelationRepository.removeByTaskHostId(id,
                    ResourcePermissionRelation.ResourceType.HOST.name());
                // 每次配置权限都是覆盖，删除重新建
                if (null != groupIds) {
                    groupIds.forEach(groupId -> relations.add(new ResourcePermissionRelation(id, groupId, PermissionType.USERGROUP.name(),
                        ResourceType.HOST.name())));
                }
            });
            if (relations.size() > 0) {
                permissionRelationRepository.batchSave(relations, 100);
            }
        } catch (Exception e) {
            log.error("Permission Config fail:", e);
            return new GeneralResult<>(false, "权限配置失败");
        }
        return new GeneralResult<>(true, "权限配置成功");
    }

    @Override
    public GeneralResult<Void> configIdentificationRelation(List<Long> ids, List<Long> identIds) {
        try {
            List<ResourceIdentificationRelation> relations = new ArrayList<>();
            if (Collections.isEmpty(ids)) {
                return new GeneralResult<>(false, "关联账号失败:缺少必要参数");
            }
            ids.forEach(id -> {
                identificationRelationRepository.removeByTaskHostId(id, ResourceType.HOST.name());
                // 每次配置权限都是覆盖，删除重新建
                if (null != identIds) {
                    for (Long identId : identIds) {
                        relations.add(new ResourceIdentificationRelation(id, identId, ResourceType.HOST.name()));
                    }
                }
            });
            if (relations.size() > 0) {
                identificationRelationRepository.batchSave(relations, 100);
            }
        } catch (Exception e) {
            log.error("Permission Config fail:", e);
            return new GeneralResult<>(false, "关联账号失败");
        }
        return new GeneralResult<>(true, "关联账号成功");
    }

    private void removeRelationByTaskHost(List<TaskHost> hosts) {
        if (Collections.isEmpty(hosts)) {
            return;
        }
        hosts.forEach(host -> {
            identificationRelationRepository.removeByTaskHostId(host.getId(), ResourceType.HOST.name());
            permissionRelationRepository.removeByTaskHostId(host.getId(), ResourceType.HOST.name());
        });
    }

    private Map<String, String> getOsNameMap() {
        Map<String, String> map = new HashMap<>();
        String value = redisTemplate.opsForValue().get("Dictionary_OS_CATEGORY_TASK");
        if (StringUtils.isEmpty(value)) {
            return map;
        }
        try {
            List<String> list = JSONObject.parseArray(value, String.class);
            list.forEach(osName -> map.put(osName.toLowerCase(), osName));
        } catch (Exception e) {
            log.error("parse array error:", e);
        }
        return map;
    }

    /**
     * 获取数据中心相关map
     *
     * @return map
     */
    private Map<String, String> getDCMap() {
        Map<String, String> map = new HashMap<>();
        String value = redisTemplate.opsForValue().get("Dictionary_DATA_CENTER");
        if (StringUtils.isEmpty(value)) {
            return map;
        }
        try {
            map = JSONObject.parseObject(value, new TypeReference<Map<String, String>>() {});
        } catch (Exception e) {
            log.error("parse array error:", e);
        }
        return map;
    }

    @Override
    public GeneralResult<Void> syncSaltMinions(RequestContext context, Long saltId) {
        try {
            TaskWorker saltWorker = null;
            if (null != saltId) {
                saltWorker = taskWorkerRepository.query(saltId);
                if (null == saltWorker) {
                    return new GeneralResult<>(false, "salt master不存在");
                } else if (!TaskWorker.Category.SaltMaster.equals(saltWorker.getCategory())) {
                    return new GeneralResult<>(true, "salt minions 同步只支持saltMaster");
                }
            }
            TaskWorker finalWorker = saltWorker;
            //异步执行同步
            taskExecutor.execute(() -> {
                String path = TaskHost.class.getSimpleName() + "_salt_minion_sync_"+saltId;
                String message = null;
                NotifyMessage notifyMessage;
                try (AutoCloseLock lock = lockFactory.getACLock(path)) {
                    Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "有其他任务正在同步");
                    log.info("salt minions数据同步开始");
                    //查询出所有的saltMaster
                    List<TaskWorker> workers = new ArrayList<>();
                    if (null == saltId) {
                        workers = taskWorkerRepository.listWorkers(TaskWorker.Category.SaltMaster.name());
                    } else {
                        workers.add(finalWorker);
                    }
                    //编辑时修改的字段
                    String[] changeFiles = new String[]{"saltId", "connProtocol"};
                    if (CollectionUtils.isEmpty(workers)) {
                        //SaltMaster不存在时，删除数据源是Salt的主机资源
                        taskHostRepository.removeBySourceType(TaskHostType.SourceType.SALT.name(), saltId);
                        //其他数据的主机资源，删除saltId并在连接协议中删除salt协议
                        List<TaskHost> hosts = taskHostRepository.listBySaltId(saltId);
                        hosts.forEach(host -> {
                            host.setSaltId(null);
                            host.setSaltMinionId(null);
                            if (StringUtils.isNotBlank(host.getConnProtocol())) {
                                List<String> protocols = JSONArray.parseArray(host.getConnProtocol().replaceAll("\\\\", ""), String.class);
                                protocols.remove(TaskHostType.ConnProtocol.SaltStack.name());
                                host.setConnProtocol(protocols.size() > 0 ? JSONObject.toJSONString(protocols) : null);
                            }
                            taskHostRepository.update(host, changeFiles);
                        });
                    } else {
                        //查询出所有的主机资源
                        List<TaskHost> allHost = taskHostRepository.listAll();
                        //多数据源的情况下，ip可能重复
                        Map<String, List<TaskHost>> ipHostMap = new HashMap<>();
                        allHost.forEach(host -> {
                            List<String> ipList = JSONArray.parseArray(host.getIps(), String.class);
                            ipList.forEach(ip -> {
                                List<TaskHost> list = ipHostMap.get(ip);
                                if (null == list) {
                                    list = new ArrayList<>();
                                }
                                list.add(host);
                                ipHostMap.put(ip, list);
                            });
                        });
                        //查询出所有连接方式包含SaltStack的数据
                        List<TaskHost> saltHosts = taskHostRepository.listBySaltId(saltId);
                        Map<Long, TaskHost> saltHostMap = saltHosts.stream().collect(Collectors.toMap(TaskHost::getId, Function.identity(), (t1, t2) -> t2));
                        //报存查询minion失败的saltIds(不对这些SaltId对应的主机资源进行处理)
                        List<Long> failSaltIds = new ArrayList<>();
                        //获取网段信息
                        List<Long> groupIds = workers.stream().map(TaskWorker::getGroupId).collect(Collectors.toList());
                        List<NetworkSegment> segments =networkSegmentRepository.listByGroupId(groupIds);
                        Map<Long,List<NetworkSegment>> segmentMap = segments.stream().collect(Collectors.groupingBy(NetworkSegment::getGroupId));
                        //查询出每个saltMaster下的数据，进行比对处理
                        GeneralResult<Void> syncResult = null;
                        for (TaskWorker worker : workers) {
                            syncResult = syncMinionByWorker(worker,failSaltIds,ipHostMap,saltHostMap,context,segmentMap);
                        }
                        if(null != saltId && null != syncResult && syncResult.isFailed()){
                            //指定saltId同步，minion编辑失败，后续逻辑不需要执行，直接返回站内信
                            message = syncResult.getMessage();
                            log.error("minion数据同步失败:{}",JSONObject.toJSONString(syncResult));
                        }else if (saltHostMap.values().size() > 0) {
                            unMatchMinionProcess(failSaltIds, saltHostMap, changeFiles);
                        }
                    }
                    if(StringUtils.isBlank(message)) {
                        message = "minion同步完成";
                        log.info("minion数据同步完成");
                    }
                } catch (Exception e) {
                    message = "minion同步失败:" + e;
                    log.error("minion数据同步失败:", e);
                }
                if (null != context) {
                    // 发送站内信
                    notifyMessage = new NotifyMessage(context, subject, message, true, false);
                    amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(),
                            CommonConstant.MESSAGE_QUEUE_ROUTINGKEY, notifyMessage);
                }
            });
            return new GeneralResult<>(true, "minion数据同步任务下发成功");
        }catch (Exception e){
            log.error("minion数据同步任务下发失败:",e);
            return new GeneralResult<>(true, "minion数据同步任务下发失败:"+e.getMessage());
        }
    }

    /**
     * 存量剩余有SaltId但未匹配saltMinion的主机资源处理。
     * @param failSaltIds 查询minion失败的saltId
     * @param saltHostMap 剩余SaltId不为空但未匹配salt minion的主机资源
     * @param changeFiles 修改字段
     */
    private void unMatchMinionProcess(List<Long> failSaltIds,Map<Long, TaskHost> saltHostMap,String[] changeFiles){
        for (TaskHost host : saltHostMap.values()) {
            if (failSaltIds.contains(host.getSaltId())) {
                //同步minion失败的salt数据暂时不进行变更
                continue;
            }
            //来源为salt的主机资源删除
            if(TaskHostType.SourceType.SALT.name().equals(host.getSourceType())){
                taskHostRepository.remove(host.getId());
                continue;
            }
            //删除没有匹配上的主机资源的Salt关联
            host.setSaltId(null);
            host.setSaltMinionId(null);
            if (StringUtils.isNotBlank(host.getConnProtocol())) {
                List<String> protocols = JSONArray.parseArray(host.getConnProtocol().replaceAll("\\\\", ""), String.class);
                protocols.remove(TaskHostType.ConnProtocol.SaltStack.name());
                host.setConnProtocol(protocols.size() > 0 ? JSONObject.toJSONString(protocols) : null);
            }
            taskHostRepository.update(host, changeFiles);
        }
    }

    /**
     * 通过woker同步salt minion信息
     * @param worker  worker（只支持SaltMaster）
     * @param failSaltIds 查询minion失败的SaltId
     * @param ipHostMap ip与主机资源列表的Map
     * @param saltHostMap 存量salt minion对应的主机资源
     * @param context 操作人
     */
    private GeneralResult<Void> syncMinionByWorker(TaskWorker worker,List<Long> failSaltIds,Map<String, List<TaskHost>> ipHostMap,
                                       Map<Long, TaskHost> saltHostMap,RequestContext context,Map<Long,List<NetworkSegment>> segmentMap){
        //日期传到TSE服务序列化的时候会失败
        worker.setGmtCreate(null);
        worker.setGmtModify(null);
        GeneralResult<List<SaltMinionModel>> result = taskWorkerService.minions(worker);
        if (result.isFailed()) {
            log.error(" salt master[{}] 下minions同步失败:{}", worker.getUri(),result.getMessage());
            failSaltIds.add(worker.getId());
            return new GeneralResult<>(false,"salt master[{}] 下minions同步失败:"+result.getMessage());
        }
        List<SaltMinionModel> models = result.getData();
        if(null == models){
            log.error(" salt master[{}] 下minions获取为null:{}", worker.getUri(),result.getMessage());
            failSaltIds.add(worker.getId());
            return new GeneralResult<>(false,"salt master[{}] 下minions同步失败:"+result.getMessage());
        }
        for (SaltMinionModel model : models) {
            List<String> ips = Arrays.asList(model.getIpv4s().split(","));
            String hostIp = null;
            for (String ip : ips) {
                if ((ipHostMap.containsKey(ip))) {
                    hostIp = ip;
                    break;
                }
            }
            //IP匹配的上编辑
            if (StringUtils.isNotBlank(hostIp)) {
                List<TaskHost> hosts = ipHostMap.get(hostIp);
                for (TaskHost host : hosts) {
                    host.setSaltId(worker.getId());
                    List<String> protocols;
                    if (StringUtils.isNotBlank(host.getConnProtocol())) {
                        protocols = JSONArray.parseArray(host.getConnProtocol().replaceAll("\\\\", ""), String.class);
                        if (!protocols.contains(TaskHostType.ConnProtocol.SaltStack.name())) {
                            protocols.add(TaskHostType.ConnProtocol.SaltStack.name());
                        }
                        host.setConnProtocol(JSONObject.toJSONString(protocols));
                    } else {
                        protocols = new ArrayList<>();
                        protocols.add(TaskHostType.ConnProtocol.SaltStack.name());
                    }
                    host.setConnProtocol(JSONObject.toJSONString(protocols));
                    //如果数据源是Salt，修改信息
                    if(TaskHostType.SourceType.SALT.name().equals(host.getSourceType())){
                        host.setName(model.getHost());
                        host.setIps(ips);
                        TaskHostDataUtil.setIp(host, ips, segmentMap.get(worker.getGroupId()));
                        host.setOsCategory(StringUtils.isNotBlank(model.getKernel()) ? model.getKernel().toUpperCase() : model.getKernel());
                        host.setOsName(StringUtils.isNotBlank(model.getOs()) ? model.getOs().toUpperCase() : model.getOs());
                    }
                    host.setSaltMinionId(model.getSaltMinionId());
                    host.setMenderId(null == context ? 1L : context.getTarget());
                    host.setGmtModify(new Date());
                    taskHostRepository.update(host);
                    saltHostMap.remove(host.getId());
                }
                continue;
            }
            //ip匹配不上新增
            TaskHost host = new TaskHost();
            host.setName(model.getHost());
            host.setSaltMinionId(model.getSaltMinionId());
            host.setIps(ips);
            List<String> protocols = new ArrayList<>();
            protocols.add(TaskHostType.ConnProtocol.SaltStack.name());
            host.setConnProtocol(JSONObject.toJSONString(protocols));
            host.setOsCategory(StringUtils.isNotBlank(model.getKernel()) ? model.getKernel().toUpperCase() : model.getKernel());
            host.setOsName(StringUtils.isNotBlank(model.getOs()) ? model.getOs().toUpperCase() : model.getOs());
            host.setSourceType(TaskHostType.SourceType.SALT.name());
            host.setCreatorId(1L);
            host.setCatalog("Computer");
            host.setMenderId(null == context ? 1L : context.getTarget());
            host.setGmtCreate(new Date());
            host.setGmtModify(new Date());
            host.setTenantId(null == context ? 0L : context.getTenant());
            host.setStatus("RUNNING");
            host.setUserCatalog(null == context ?  RequestContext.Catalog.Manager.name() : context.getCatalog().name());
            host.setSaltId(worker.getId());
            //获取符合salt网段的ip
            TaskHostDataUtil.setIp(host, ips, segmentMap.get(worker.getGroupId()));
            taskHostRepository.save(host);
        }
        return new GeneralResult<>(true,"minion信息变更成功");
    }
}
