package com.bocloud.cop.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskPlatformAlert;
import com.bocloud.cop.enums.Status;
import com.bocloud.cop.intf.TaskPlatformAlertService;
import com.bocloud.cop.model.TaskPlatformModel;
import com.bocloud.cop.repository.TaskInstanceRepository;
import com.bocloud.cop.repository.TaskPlatformAlertRepository;
import com.bocloud.cop.service.utils.PlatformAlertConstant;
import com.google.common.collect.Lists;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 任务实例平台告警接口实现
 *
 * @author majiali
 * @version 1.0
 * @since 2022.08.24
 */
@Slf4j
@Service("taskPlatformAlertService")
public class TaskPlatformAlertServiceImpl implements TaskPlatformAlertService {

    @Autowired
    private TaskPlatformAlertRepository taskPlatformAlertRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private TaskInstanceRepository taskInstanceRepository;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public GeneralResult<GridBean<TaskPlatformAlert>> list(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<TaskPlatformAlert> list = taskPlatformAlertRepository.list(pager);
            for (TaskPlatformAlert alert : list) {
                if (null != alert.getDealId()) {
                    alert.setDealName(redisTemplate.opsForValue().get("User_Name_" + alert.getDealId()));
                }
            }
            int total = taskPlatformAlertRepository.count(pager.getParams());
            GridBean<TaskPlatformAlert> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询任务平台告警成功");
        } catch (Exception e) {
            log.error("List task platform record fail:", e);
            return new GeneralResult<>(false, "查询任务平台告警失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(TaskPlatformAlert instance, Long userId) {
        try {
            TaskPlatformAlert platform = taskPlatformAlertRepository.query(instance.getId());
            if (null == platform) {
                return new GeneralResult<>(false, "任务平台告警不存在");
            }
            if (!PlatformAlertConstant.Status.CREATED.name().equals(platform.getStatus())) {
                return new GeneralResult<>(false, "当前告警不是待处理状态");

            }
            if (StringUtils.isEmpty(instance.getDealMethod())) {
                return new GeneralResult<>(false, "参数异常");
            }
            //平台告警修改只能修改限流梳理方法
            platform.setDealMethod(instance.getDealMethod());
            platform.setStatus(PlatformAlertConstant.Status.PROCESSED.name());
            platform.setMenderId(userId);
            platform.setDealTime(new Date());
            platform.setDealId(userId);
            taskPlatformAlertRepository.update(platform);
            modifyUpdateRedis(platform);
            return new GeneralResult<>(true, "修改任务平台告警成功");
        } catch (Exception e) {
            log.error("Modify task platform record fail:", e);
            return new GeneralResult<>(false, "修改任务平台告警失败");
        }
    }

    private void modifyUpdateRedis(TaskPlatformAlert platform) {
        try (AutoCloseLock lock = lockFactory.getACLock("task_instance_platform_" + platform.getPlatform())) {
            Assert.isTrue(lock.acquire(30, TimeUnit.SECONDS), "平台：" + platform.getPlatform() + "正在校验限流信息");
            String platformTemp = redisTemplate.opsForValue().get("platform_" + platform.getPlatform());
            if (StringUtils.isNotEmpty(platformTemp)) {
                TaskPlatformModel platformModel = JSONObject.parseObject(platformTemp, TaskPlatformModel.class);
                if (null != platformModel && platform.getPlatformConfigId().equals(platformModel.getPlatformConfigId())) {
                    platformModel.setPlatformAlertId(platform.getId());
                    platformModel.setDealMethod(platform.getDealMethod());
                    redisTemplate.opsForValue().set("platform_" + platform.getPlatform(), JSONObject.toJSONString(platformModel));
                }
            }

        } catch (Exception e) {
            log.error("修改告警更新缓存出错：", e);
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        try {
            TaskPlatformAlert instance = this.taskPlatformAlertRepository.query(id);
            Assert.notNull(instance, "数据不存在！");
            taskPlatformAlertRepository.remove(id, userId);
            removeUpdateRedis(instance);
        } catch (Exception e) {
            log.error("Remove record fail:", e);
            return new GeneralResult<>(false, "删除记录失败");
        }
        return new GeneralResult<>(true, "删除记录成功");
    }

    private void removeUpdateRedis(TaskPlatformAlert instance) {
        try (AutoCloseLock lock = lockFactory.getACLock("task_instance_platform_" + instance.getPlatform())) {
            Assert.isTrue(lock.acquire(30, TimeUnit.SECONDS), "平台：" + instance.getPlatform() + "正在校验限流信息");
            String platformTemp = redisTemplate.opsForValue().get("platform_" + instance.getPlatform());
            if (StringUtils.isNotEmpty(platformTemp)) {
                TaskPlatformModel platformModel = JSONObject.parseObject(platformTemp, TaskPlatformModel.class);
                if (null != platformModel && instance.getPlatformConfigId().equals(platformModel.getPlatformConfigId())) {
                    platformModel.setPlatformAlertId(null);
                    platformModel.setDealMethod(null);
                    platformModel.setCurrPlatformAlertId(null);
                    platformModel.setExistCreateAlert(false);
                    redisTemplate.opsForValue().set("platform_" + instance.getPlatform(), JSONObject.toJSONString(platformModel));
                }
            }

        } catch (Exception e) {
            log.error("修改告警更新缓存出错：", e);
        }
    }

    @Override
    public GeneralResult<?> instanceList(Long id, String type, Pager pager) {
        try {
            TaskPlatformAlert instance = this.taskPlatformAlertRepository.query(id);
            Assert.notNull(instance, "数据不存在！");
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            //查询正在执行的
            List<Param> params = Lists.newArrayList();
            params.add(new Param(MapTools.simpleMap("platform", instance.getPlatform()), Sign.EQ));
            //根据状态查询正在执行、暂缓执行及丢弃执行的任务
            if ("running".equalsIgnoreCase(type)) {
                params.add(new Param(MapTools.simpleMap("status", Status.Inst.RUNNING), Sign.EQ));
            } else if ("queue".equalsIgnoreCase(type)) {
                params.add(new Param(MapTools.simpleMap("platformAlertId", id), Sign.EQ));
                params.add(new Param(MapTools.simpleMap("status", Status.Inst.CREATED), Sign.EQ));
                params.add(new Param(MapTools.simpleMap("isLimited", 1), Sign.EQ));
            } else if ("discard".equalsIgnoreCase(type)) {
                params.add(new Param(MapTools.simpleMap("platformAlertId", id), Sign.EQ));
                params.add(new Param(MapTools.simpleMap("status", Status.Inst.CANCELED), Sign.EQ));
                params.add(new Param(MapTools.simpleMap("isLimited", 1), Sign.EQ));
            } else {
                return new GeneralResult<>(false, "类型错误");
            }
            pager.setParams(params);
            List<TaskInstance> list = taskInstanceRepository.list(pager, "");
            int total = taskInstanceRepository.count(pager.getParams(), "");
            GridBean<TaskInstance> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("Remove record fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

}
