package com.bocloud.cop.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.cop.entity.TaskConfig;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskPlatformAlert;
import com.bocloud.cop.entity.TaskPlatformConfig;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.PlatformAlertConstant;
import com.bocloud.cop.enums.TaskParam;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.intf.TaskPlatformConfigService;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.model.TaskPlatformModel;
import com.bocloud.cop.repository.TaskConfigRepository;
import com.bocloud.cop.repository.TaskInstanceRepository;
import com.bocloud.cop.repository.TaskPlatformAlertRepository;
import com.bocloud.cop.repository.TaskPlatformConfigRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 任务实例平台接口实现
 *
 * @author majiali
 * @version 1.0
 * @since 2022.08.24
 */
@Slf4j
@Service("taskPlatformConfigService")
public class TaskPlatformConfigServiceImpl implements TaskPlatformConfigService {

    @Autowired
    private TaskPlatformConfigRepository taskPlatformConfigRepository;

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private TaskPlatformAlertRepository taskPlatformAlertRepository;
    @Autowired
    private TaskConfigRepository taskConfigRepository;
    @Autowired
    private TaskInstanceRepository taskInstanceRepository;
    @Autowired
    private TaskInstanceService taskInstanceService;

    @Override
    public GeneralResult<GridBean<TaskPlatformConfig>> list(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<TaskPlatformConfig> list = taskPlatformConfigRepository.list(pager);
            if (CollectionUtils.isNotEmpty(list)) {
                dealUserName(list);
            }
            int total = taskPlatformConfigRepository.count(pager.getParams());
            GridBean<TaskPlatformConfig> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询任务平台成功");
        } catch (Exception e) {
            log.error("List task platform config record fail:", e);
            return new GeneralResult<>(false, "查询任务平台失败");
        }
    }

    private void dealUserName(List<TaskPlatformConfig> list) {
        for (TaskPlatformConfig platform : list) {
            platform.setCreaterName(redisTemplate.opsForValue().get("User_Name_" + platform.getCreatorId()));
            String noticers = platform.getNoticers();
            if (StringUtils.isNotEmpty(noticers)) {
                String[] noticeArr = noticers.split(",");
                List<String> userNames = new ArrayList<>();
                for (String userId : noticeArr) {
                    userNames.add(redisTemplate.opsForValue().get("User_Name_" + userId));
                }
                platform.setNoticersName(String.join(",", userNames));
            }

        }
    }
    @Override
    public GeneralResult<TaskPlatformConfig> get(Long id) {
        try {
            TaskPlatformConfig instance = this.taskPlatformConfigRepository.query(id);
            Assert.notNull(instance, "数据不存在！");
            return new GeneralResult<>(true, instance, "查询任务平台详细成功");
        } catch (Exception e) {
            log.error("Query task platform config record detail fail:", e);
            return new GeneralResult<>(false, "查询任务平台详细失败");
        }
    }

    @Override
    public GeneralResult<Void> create(TaskPlatformConfig instance, Long userId) {
        try {
            if (!paramCheck(instance)) {
                return new GeneralResult<>(false, "参数异常");
            }
            if (instance.getStartTime().compareTo(instance.getEndTime()) > 0) {
                return new GeneralResult<>(false, "开始时间不能大于结束时间");
            }
            List<TaskPlatformConfig> platformConfigs = taskPlatformConfigRepository.getByPlatform(instance.getPlatform());
            boolean flag = false;
            if (CollectionUtils.isNotEmpty(platformConfigs)) {
                List<String> times = new ArrayList<>();
                times.add(instance.getStartTime() + "-" + instance.getEndTime());
                for (TaskPlatformConfig config : platformConfigs) {
                    times.add(config.getStartTime() + "-" + config.getEndTime());
                }
                //平台告警配置中 【执行时间】 重叠校验； 校验目标是：每个平台所有的告警配置中的执行时间不可以重叠。
                flag = checkOverlap(times);
            }
            if (flag) {
                return new GeneralResult<>(false, "该平台告警执行时间与已有执行时间有重叠");
            }
            GeneralResult<TaskPlatformConfig> result = exists(instance.getName());
            if (result.isSuccess()) {
                return new GeneralResult<>(false, "该名称已存在");
            }
            instance.setDeleted(false);
            instance.setCreatorId(userId);
            instance.setMenderId(userId);
            taskPlatformConfigRepository.save(instance);
            return new GeneralResult<>(true, "添加任务平台成功");
        } catch (Exception e) {
            log.error("Create task platform config record fail:", e);
            return new GeneralResult<>(false, "添加任务平台失败");
        }
    }

    private boolean paramCheck(TaskPlatformConfig instance) {
        if (StringUtils.isEmpty(instance.getName()) || StringUtils.isEmpty(instance.getPlatform()) || StringUtils.isEmpty(instance.getStartTime()) || StringUtils.isEmpty(instance.getEndTime())) {
            return false;
        }
        if (null == instance.getPeriod() || 0 == instance.getPeriod() || null == instance.getInstanceNum() || 0 == instance.getInstanceNum() || null == instance.getTargetHostNum() || 0 == instance.getTargetHostNum()) {
            return false;
        }
        if (StringUtils.isEmpty(instance.getNoticers())) {
            return false;
        }

        return true;
    }

    @Override
    public GeneralResult<Void> modify(TaskPlatformConfig instance, Long userId) {
        try {
            if (!paramCheck(instance)) {
                return new GeneralResult<>(false, "参数异常");
            }
            if (instance.getStartTime().compareTo(instance.getEndTime()) > 0) {
                return new GeneralResult<>(false, "开始时间不能大于结束时间");
            }
            TaskPlatformConfig platform = taskPlatformConfigRepository.query(instance.getId());
            if (null == platform) {
                return new GeneralResult<>(false, "平台配置信息不存在");
            }
            List<TaskPlatformConfig> platformConfigs = taskPlatformConfigRepository.getByPlatform(instance.getPlatform());
            boolean flag = false;
            if (CollectionUtils.isNotEmpty(platformConfigs)) {
                List<String> times = new ArrayList<>();
                times.add(instance.getStartTime() + "-" + instance.getEndTime());
                for (TaskPlatformConfig config : platformConfigs) {
                    if (config.getId().equals(instance.getId())) {
                        continue;
                    }
                    times.add(config.getStartTime() + "-" + config.getEndTime());
                }
                //平台告警配置中 【执行时间】 重叠校验； 校验目标是：每个平台所有的告警配置中的执行时间不可以重叠。
                flag = checkOverlap(times);
            }
            if (flag) {
                return new GeneralResult<>(false, "该平台告警执行时间与已有执行时间有重叠");
            }
            GeneralResult<TaskPlatformConfig> result = exists(instance.getName());
            if (result.isSuccess() && !result.getData().getId().equals(instance.getId())) {
                return new GeneralResult<>(false, "该名称已存在");
            }
            instance.setDeleted(false);
            instance.setPlatform(platform.getPlatform());
            instance.setMenderId(userId);
            taskPlatformConfigRepository.update(instance);
            //修改告警配置，如果修改关键信息则原本告警数据失效并删除缓存，否则只需要更新缓存即可.
            modfiyUpdateRedis(instance, platform, userId);
            return new GeneralResult<>(true, "修改任务平台成功");
        } catch (Exception e) {
            log.error("Modify task platform config record fail:", e);
            return new GeneralResult<>(false, "修改任务平台失败");
        }
    }

    private void modfiyUpdateRedis(TaskPlatformConfig instance, TaskPlatformConfig platform, Long userId) {
        if (!instance.getOpened() && platform.getOpened()){
            //如果禁用该配置则删除对应缓存以及放行所有限流的任务
            deleteRedisData(instance, userId);
        }
        try (AutoCloseLock lock = lockFactory.getACLock("task_instance_platform_" + platform.getPlatform())) {
            Assert.isTrue(lock.acquire(30, TimeUnit.SECONDS), "平台：" + platform.getPlatform() + "正在校验限流信息");
            //关键数据：执行时间、告警策略变动的时候，已有告警将失效
            if (isChange(instance, platform)) {
                //配置变更后，已有告警失效，重新加载配置信息。
                taskPlatformAlertRepository.updateInvalidAndDel(instance.getId(), userId);
                String platformTemp = redisTemplate.opsForValue().get("platform_" + platform.getPlatform());
                if (StringUtils.isNotEmpty(platformTemp)) {
                    TaskPlatformModel platformModel = JSONObject.parseObject(platformTemp, TaskPlatformModel.class);
                    if (null != platformModel && instance.getId().equals(platformModel.getPlatformConfigId())) {
                        //删除缓存中的信息，等一分钟后定时任务那里会重新加载（TaskPlatformLimitScheduler）
                        redisTemplate.delete("platform_" + platform.getPlatform());
                    }

                }
            } else {
                String platformTemp = redisTemplate.opsForValue().get("platform_" + platform.getPlatform());
                if (StringUtils.isNotEmpty(platformTemp)) {
                    TaskPlatformModel platformModel = JSONObject.parseObject(platformTemp, TaskPlatformModel.class);
                    if (null != platformModel && instance.getId().equals(platformModel.getPlatformConfigId())) {
                        platformModel.setName(instance.getName());
                        platformModel.setNoticers(instance.getNoticers());
                        redisTemplate.opsForValue().set("platform_" + platform.getPlatform(), JSONObject.toJSONString(platformModel));
                    }
                }
            }
        } catch (Exception e) {
            log.error("update limited redis fail:", e);
        }
    }

    /**
     * 关键数据是否有改动
     * @param target
     * @param source
     * @return
     */
    public boolean isChange(TaskPlatformConfig target, TaskPlatformConfig source) {
        //是否关键字段有改动
        if (!target.getStartTime().equals(source.getStartTime())) {
            return true;
        }
        if (!target.getEndTime().equals(source.getEndTime())) {
            return true;
        }
        if (!target.getPeriod().equals(source.getPeriod())) {
            return true;
        }
        if (!target.getInstanceNum().equals(source.getInstanceNum())) {
            return true;
        }
        if (!target.getTargetHostNum().equals(source.getTargetHostNum())) {
            return true;
        }

        return false;
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        try {
            TaskPlatformConfig instance = this.taskPlatformConfigRepository.query(id);
            Assert.notNull(instance, "数据不存在！");
            List<TaskPlatformAlert> alerts = taskPlatformAlertRepository.getByPlatformConfigId(id);
            if (CollectionUtils.isNotEmpty(alerts)) {
                return new GeneralResult<>(false, "该配置下存在告警信息，不可删除");
            }
            taskPlatformConfigRepository.remove(id, userId);
            deleteRedisData(instance, userId);

        } catch (Exception e) {
            log.error("Remove record fail:", e);
            return new GeneralResult<>(false, "删除记录失败");
        }
        return new GeneralResult<>(true, "删除记录成功");
    }

    @Override
    public GeneralResult<Void> updateOpened(Long id, Boolean opened, Long userId) {
        try {
            TaskPlatformConfig instance = this.taskPlatformConfigRepository.query(id);
            Assert.notNull(instance, "数据不存在！");
            taskPlatformConfigRepository.updateOpened(id, opened, userId);
            if (!opened) {
                deleteRedisData(instance, userId);
            }

        } catch (Exception e) {
            log.error("update limited fail:", e);
            return new GeneralResult<>(false, "更新是否限流失败");
        }
        return new GeneralResult<>(true, "更新是否限流成功");
    }

    private void deleteRedisData(TaskPlatformConfig instance, Long userId) {
        try (AutoCloseLock lock = lockFactory.getACLock("task_instance_platform_" + instance.getPlatform())) {
            Assert.isTrue(lock.acquire(30, TimeUnit.SECONDS), "平台：" + instance.getPlatform() + "正在校验限流信息");
            //关闭限流，已有限制执行的任务需要下发
            String platformTemp = redisTemplate.opsForValue().get("platform_" + instance.getPlatform());
            if (StringUtils.isEmpty(platformTemp)) {
                return;
            }
            TaskPlatformModel platformModel = JSONObject.parseObject(platformTemp, TaskPlatformModel.class);
            if (null == platformModel || !instance.getId().equals(platformModel.getPlatformConfigId())) {
                return;
            }
            redisTemplate.delete("platform_" + instance.getPlatform());
            //如果有暂缓执行的任务，需要放开执行
            List<TaskInstance> instanceList = taskInstanceRepository.listLimited(instance.getPlatform(), null);
            for (TaskInstance taskInstance : instanceList) {
                //还要判断任务数及目标主机数是否超出限制
                taskInstance.setIsLimited(false);
                taskInstance.setLimitedReleaseTime(new Date());
                instance.setMenderId(userId);
                taskInstanceRepository.updateLimitedRelease(instance.getId(), userId);
                taskInstanceService.putJob(buildJob(taskInstance, taskInstance.getPriority()));
            }

        } catch (Exception e) {
            log.error("关闭平台告警配置更新缓存出错：", e);
        }
    }

    private Job buildJob(TaskInstance instance, Integer priority) {
        Job job = new Job();
        job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
        job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
        job.setTaskId(instance.getUuid());
        job.setParam(Common.NAME, instance.getName());
        job.setParam(TaskParam.TASKCODE.name(), instance.getCode());
        if (null != priority) {
            job.setPriority(priority);
        }
        return job;
    }
    @Override
    public GeneralResult<TaskPlatformModel> chargeLimit(String platform, int targetHostNum) {

        try (AutoCloseLock lock = lockFactory.getACLock("task_instance_platform_" + platform)) {
            Assert.isTrue(lock.acquire(30, TimeUnit.SECONDS), "平台：" + platform + "正在校验限流信息");
            TaskPlatformModel temp = new TaskPlatformModel();
            //平台限流任务总开关未开启
            if (!limitSwitch()) {
                temp.setLimit(false);
                return new GeneralResult<>(true, temp, "判断平台是否限流成功");
            }
            //获取缓存中平台对应的限流配置信息
            String platformTemp = redisTemplate.opsForValue().get("platform_" + platform);
            if (StringUtils.isEmpty(platformTemp)) {
                temp.setLimit(false);
                return new GeneralResult<>(true, temp, "判断平台是否限流成功");
            }

            boolean flag = false;
            // 判断当天时间段中是否有告警处理方法，有的话按照告警处理方法处理数据
            //没有的话查看是否有未处理的告警，有的执行放行，没有的话触发告警直接执行
            temp = JSONObject.parseObject(platformTemp, TaskPlatformModel.class);
            temp.setCurrInstanceNum(temp.getCurrInstanceNum() + 1);
            temp.setCurrTargetHostNum(temp.getCurrTargetHostNum() + targetHostNum);
            if (temp.getInstanceNum() < temp.getCurrInstanceNum() || temp.getTargetHostNum() < temp.getCurrTargetHostNum()) {
                //已处理告警
                Long platformAlertId = temp.getPlatformAlertId();
                //当前统计周期未处理告警
                Long currPlatformAlertId = temp.getCurrPlatformAlertId();
                //存在未处理告警
                boolean existCreateAlert = temp.isExistCreateAlert();
                if (null != platformAlertId) {
                    //有处理逻辑，做限流控制
                    flag = true;
                } else if (null == currPlatformAlertId) {
                    //当前时间段内是否有限流告警配置信息，没有的话发送消息告警
                    String sendTo = temp.getNoticers();
                    List<String> userIds = Arrays.asList(sendTo.split(","));
                    StringBuffer buf = new StringBuffer();
                    buf.append("在任务执行时间" + temp.getStartTime() + "至" + temp.getEndTime() + "，");
                    buf.append("任务周期" + temp.getPeriodStartTime() + "至" + temp.getPeriodEndTime() + ",");
                    buf.append("告警策略：过去" + temp.getPeriod() + "分钟，任务实例数大于" + temp.getInstanceNum() + "个，目标主机数大于" + temp.getTargetHostNum() + "台中，");
                    buf.append("当前任务实例数为" + temp.getCurrInstanceNum() + "个，目标主机数为" + temp.getCurrTargetHostNum() + "台，达到告警阈值，产生告警");
                    if (!existCreateAlert) {
                        TaskPlatformAlert currAlert = new TaskPlatformAlert();
                        currAlert.setPeriod(temp.getPeriod());
                        currAlert.setPlatform(temp.getPlatform());
                        currAlert.setInstanceNum(temp.getInstanceNum());
                        currAlert.setRemark(buf.toString());
                        currAlert.setTargetHostNum(temp.getTargetHostNum());
                        currAlert.setPlatformConfigId(temp.getPlatformConfigId());
                        currAlert.setName(temp.getName());
                        currAlert.setStatus(PlatformAlertConstant.Status.CREATED.name());
                        currAlert.setCreatorId(1L);
                        currAlert.setStartTime(temp.getStartTime());
                        currAlert.setEndTime(temp.getEndTime());
                        taskPlatformAlertRepository.save(currAlert);
                        temp.setCurrPlatformAlertId(currAlert.getId());
                        temp.setExistCreateAlert(true);
                    }
                    String subject = "平台【" + platform + "】限流告警";
                    String content = "您好！平台【" + platform + "】，" + buf.toString() + "，请您到资源运维->作业限流->当前告警中处理";
                    for (String userId : userIds) {
                        RequestContext context = new RequestContext(RequestContext.Catalog.Manager, Long.parseLong(userId), null, null, null);
                        notice(subject, content, context);
                    }
                }
            }
            temp.setLimit(flag);
            redisTemplate.opsForValue().set("platform_" + platform, JSONObject.toJSONString(temp));
            return new GeneralResult<>(true, temp, "判断平台是否限流成功");
        } catch (Exception e) {
            log.error("update limited fail:", e);
            return new GeneralResult<>(false, "判断平台是否限流失败");
        }
    }

    private void notice(String subject, String content, RequestContext context) {
        JSONObject object = new JSONObject();
        object.put("context", context);
        object.put("subject", subject);
        object.put("content", content);
        object.put("sendMsg", true);
        object.put("sendEmail", false);
        String exchange = BoCloudExchange.Default.DIRECT.getExchange();
        String queue = "event.notice.queue";
        amqpTemplate.convertAndSend(exchange, queue, object);
    }

    @Override
    public boolean limitSwitch() {
        boolean flag = false;
        // 是否开启校验
        String auditSwitch = getValue("TaskPlatformLimitSwitch");
        if ("true".equalsIgnoreCase(auditSwitch)) {
            flag = true;
        }
        return flag;
    }

    private String getValue(String code) {
        try {
            String value = redisTemplate.opsForValue().get(code);
            if (StringUtils.isEmpty(value)) {
                TaskConfig config = taskConfigRepository.queryByCode(code);
                if (null == config) {
                    return null;
                }
                value = config.getValue();
            }
            return value;
        } catch (Exception e) {
            return null;
        }
    }

    @Override
    public GeneralResult<Void> updatePlatformRedis(Long id) {
        try {
            TaskPlatformConfig platformInfo = taskPlatformConfigRepository.query(id);
            if (null == platformInfo) {
                return new GeneralResult<>(false, "平台限流信息不存在");
            }
            String platform = platformInfo.getPlatform();
            try (AutoCloseLock lock = lockFactory.getACLock("task_instance_platform_" + id)) {
                Assert.isTrue(lock.acquire(30, TimeUnit.SECONDS), "平台：" + id + "正在校验限流信息");
                Map<String, String> timeMap = getTime(platformInfo.getPeriod());
                String startTime = timeMap.get("startTime");
                String endTime = timeMap.get("endTime");
                log.info("平台{}限流任务周期{}至{}", platform, startTime, endTime);
                String platformTemp = redisTemplate.opsForValue().get("platform_" + platform);
                if (StringUtils.isNotEmpty(platformTemp)) {
                    TaskPlatformConfig temp = JSONObject.parseObject(platformTemp, TaskPlatformConfig.class);
                    platformInfo.setCurrTargetHostNum(temp.getCurrTargetHostNum());
                    platformInfo.setCurrInstanceNum(temp.getCurrInstanceNum());
                } else {
                    Integer instanceNum = taskInstanceRepository.countInstanceNum(platform, startTime, endTime) + taskInstanceRepository.countInstanceNumLast(platform, startTime, endTime);
                    platformInfo.setCurrInstanceNum(instanceNum);
                    Integer targetHostNum = taskInstanceRepository.countTargetHostNum(platform, startTime, endTime) + taskInstanceRepository.countTargetHostNumLast(platform, startTime, endTime);
                    platformInfo.setCurrTargetHostNum(targetHostNum);
                }
                List<TaskPlatformAlert> alertList = taskPlatformAlertRepository.getByPlatformConfigId(platformInfo.getId());
                platformInfo.setAlertList(alertList);
                redisTemplate.opsForValue().set("platform_" + platform, JSONObject.toJSONString(platformInfo));
                return new GeneralResult<>(true, "更新平台限流缓存信息成功");
            } catch (Exception e) {
                log.error("update limited fail:", e);
                return new GeneralResult<>(false, "更新平台限流缓存信息失败");
            }
        } catch (Exception e) {
            log.error("update limited fail:", e);
            return new GeneralResult<>(false, "更新平台限流缓存信息失败");
        }
    }
    private Map<String, String> getTime(Integer period) {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.SECOND, 0);
        int mins = cal.get(Calendar.MINUTE);
        int remainder = mins % period;
        cal.add(Calendar.MINUTE, period - remainder);
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String endTime = df.format(cal.getTime());
        cal.add(Calendar.MINUTE, -period);
        String startTime = df.format(cal.getTime());
        Map<String, String> map = new HashMap<>();
        map.put("startTime", startTime);
        map.put("endTime", endTime);
        return map;
    }

    /**
     * 平台告警配置中 【执行时间】 重叠校验； 校验目标是：每个平台所有的告警配置中的执行时间不可以重叠。
     * @param list
     * @return
     */
    private boolean checkOverlap(List<String> list) {
        Collections.sort(list);//排序ASC
        boolean flag = false;//是否重叠标识
        for (int i = 0; i < list.size(); i++) {
            if (i > 0) {
                //跳过第一个时间段不做判断
                String[] itime = list.get(i).split("-");
                for (int j = 0; j < list.size(); j++) {
                    //如果当前遍历的i开始时间小于j中某个时间段的结束时间那么则有重叠，反之没有重叠
                    //这里比较时需要排除i本身以及i之后的时间段，因为已经排序了所以只比较自己之前(不包括自己)的时间段
                    if (j == i || j > i) {
                        continue;
                    }

                    Date day = new Date();
                    SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");

                    String[] jtime = list.get(j).split("-");
                    //此处DateUtils.compare为日期比较(返回负数date1小、返回0两数相等、返回正整数date1大)
                    int compare = compareDate(
                            (df.format(day) + " " + itime[0] + ":00"),
                            (df.format(day) + " " + jtime[1] + ":00"),
                            "yyyy-MM-dd HH:mm:ss");
                    if (compare < 0) {
                        flag = true;
                        break;//只要存在一个重叠则可退出内循环
                    }
                }
            }

            //当标识已经认为重叠了则可退出外循环
            if (flag) {
                break;
            }
        }

        return flag;
    }

    private Integer compareDate(String DATE1, String DATE2, String dateFormat) {
        DateFormat df = new SimpleDateFormat(dateFormat);
        try {
            Date dt1 = df.parse(DATE1);
            Date dt2 = df.parse(DATE2);
            if (dt1.getTime() > dt2.getTime()) {
                return 1;
            } else if (dt1.getTime() < dt2.getTime()) {
                return -1;
            } else {
                return 0;
            }
        } catch (Exception e) {
            log.error("时间比较出错", e);
        }

        return null;
    }

    @Override
    public GeneralResult<TaskPlatformConfig> exists(String name) {
        try {
            TaskPlatformConfig config = taskPlatformConfigRepository.exists(name);
            if (null == config) {
                log.error("TaskPlatformConfig data not exist");
                return new GeneralResult<>(false, "平台告警配置不存在");
            }
            return new GeneralResult<>(true, config, "平台告警配置存在");
        } catch (Exception e) {
            log.error("get TaskPlatformConfig error:", e);
            return new GeneralResult<>(false, "平台告警配置不存在");
        }
    }

}
