package com.bocloud.cop.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskScene;
import com.bocloud.cop.entity.TaskTemplate;
import com.bocloud.cop.intf.TaskSceneService;
import com.bocloud.cop.repository.TaskRepository;
import com.bocloud.cop.repository.TaskSceneRepository;
import com.bocloud.cop.repository.TaskTemplateRepository;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 作业场景抽象Service接口实现类
 *
 * @author wei wei
 * @version 1.0
 * @since 2019.01.10
 */
@Slf4j
@Service("taskSceneService")
public class TaskSceneServiceImpl implements TaskSceneService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private TaskSceneRepository taskSceneRepository;
    @Autowired
    private TaskRepository taskRepository;
    @Autowired
    private TaskTemplateRepository taskTemplateRepository;

    @Override
    public GeneralResult<GridBean<TaskScene>> list(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ZERO);
            int total = taskSceneRepository.count(pager.getParams());
            List<TaskScene> list = taskSceneRepository.list(pager);
            List<TaskScene> scenes = new ArrayList<>();
            for (TaskScene scene : list) {
                pager.getParams().clear();
                pager.getParams().add(new Param(MapTools.simpleMap("sceneId", scene.getId()), Sign.EQ));
                if (com.megatron.common.model.RequestContext.Catalog.Tenant == context.getCatalog()
                    || com.megatron.common.model.RequestContext.Catalog.User == context.getCatalog()) {
                    int count = taskTemplateRepository.countByTenant(pager.getParams(), context.getTenant());
                    if (count == 0) {
                        continue;
                    }
                    scene.setTemplateCount(count);
                } else {
                    scene.setTemplateCount(taskTemplateRepository.count(pager.getParams()));
                    pager.getParams().add(new Param(MapTools.simpleMap("code", "standard"), Sign.EQ));
                    pager.getParams().add(new Param(MapTools.simpleMap("status", "APPROVED"), Sign.EQ));
                    if (RequestContext.Catalog.Manager == context.getCatalog()) {
                        scene.setTaskCount(taskRepository.count(pager.getParams()));
                    }
                }
                scenes.add(scene);
            }
            GridBean<TaskScene> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, scenes);
            return new GeneralResult<>(true, bean, "查询作业场景成功");
        } catch (Exception e) {
            log.error("List task scene fail:", e);
            return new GeneralResult<>(false, "查询作业场景失败");
        }
    }

    @Override
    public GeneralResult<GridBean<SimpleBean>> simple(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ZERO);
            int total = taskSceneRepository.count(pager.getParams());
            List<SimpleBean> list = taskSceneRepository.list(pager.getParams(), pager.getSorter());
            GridBean<SimpleBean> bean = new GridBean<>(1, 1, total, list);
            return new GeneralResult<>(true, bean, "查询作业场景成功");
        } catch (Exception e) {
            log.error("List task scene fail:", e);
            return new GeneralResult<>(false, "查询作业场景失败");
        }
    }

    @Override
    public GeneralResult<Void> create(TaskScene scene) {
        try {
            // 判断作业场景名称是否存在
            boolean flag = taskSceneRepository.exists(scene.getName());
            if (flag) {
                return new GeneralResult<>(false, "该名称已存在，请重新输入!");
            }
            taskSceneRepository.save(scene);
            return new GeneralResult<>(true, "添加作业场景成功");
        } catch (Exception e) {
            log.error("query task scene error:", e);
            return new GeneralResult<>(false, "添加作业场景失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(TaskScene scene) {
        String path = Task.class.getSimpleName() + "_" + scene.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskScene taskScene = this.taskSceneRepository.query(scene.getId());
            Assert.notNull(taskScene, "数据不存在！");
            if (!taskScene.getName().equals(scene.getName())) {
                // 判断某种类的作业场景名称是否存在
                boolean flag = taskSceneRepository.exists(scene.getName());
                if (flag) {
                    return new GeneralResult<>(false, "该名称已存在，请重新输入!");
                }
            }
            taskScene.setName(scene.getName());
            taskScene.setRemark(scene.getRemark());
            taskScene.setCode(scene.getCode());
            taskScene.setIcon(scene.getIcon());
            taskScene.setGmtModify(new Date());
            taskSceneRepository.update(taskScene);
            return new GeneralResult<>(true, "修改作业场景成功");
        } catch (Exception e) {
            log.error("Update task scene fail:", e);
            return new GeneralResult<>(false, "修改作业场景失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = Task.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            List<TaskTemplate> templates = taskTemplateRepository.listByScene(id);
            if (!ListTool.isEmpty(templates)) {
                return new GeneralResult<>(false, "该场景下存在作业模板，无法删除");
            }
            List<Task> tasks = taskRepository.listByScene(id);
            if (!ListTool.isEmpty(tasks)) {
                return new GeneralResult<>(false, "该场景下存在作业，无法删除");
            }
            boolean result = this.taskSceneRepository.remove(id, userId);
            return result ? new GeneralResult<>(true, "删除作业场景成功") : new GeneralResult<>(false, "删除作业场景失败");
        } catch (Exception e) {
            log.error("Remove task scene fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<TaskScene> detail(Long id) {
        try {
            TaskScene scene = this.taskSceneRepository.query(id);
            Assert.notNull(scene, "数据不存在！");
            List<Param> params = Lists.newArrayList();
            params.add(new Param(MapTools.simpleMap("sceneId", scene.getId()), Sign.EQ));
            scene.setTemplateCount(taskTemplateRepository.count(params));
            params.add(new Param(MapTools.simpleMap("code", "standard"), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("status", "APPROVED"), Sign.EQ));
            scene.setTaskCount(taskRepository.count(params));
            return new GeneralResult<>(true, scene, "查询作业场景详细成功");
        } catch (Exception e) {
            log.error("Query task scene detail fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

}
