package com.bocloud.cop.service.backup;

import com.bocloud.cop.entity.backup.TaskBackup;
import com.bocloud.cop.entity.backup.TaskBackupScene;
import com.bocloud.cop.intf.backup.TaskBackupSceneService;
import com.bocloud.cop.repository.backup.TaskBackupRepository;
import com.bocloud.cop.repository.backup.TaskBackupSceneRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author huliwei
 * @date 2022/10/19
 */
@Service("taskBackupSceneService")
@Slf4j
public class TaskBackupSceneServiceImpl implements TaskBackupSceneService {

    @Autowired
    private TaskBackupSceneRepository taskBackupSceneRepository;
    @Autowired
    private TaskBackupRepository taskBackupRepository;
    @Autowired
    private LockFactory lockFactory;

    /**
     * 配置场景查询
     * 
     * @param pager
     * @return
     */
    @Override
    public GeneralResult<GridBean<List<TaskBackupScene>>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = taskBackupSceneRepository.count(pager.getParams());
            List<TaskBackupScene> list = this.taskBackupSceneRepository.list(pager);
            GridBean gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("TaskBackupScene list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    /**
     * 配置场景新增
     * 
     * @param scene
     *            新增实例信息
     * @param context
     *            系统参数信息
     * @return
     */
    @Override
    public GeneralResult create(TaskBackupScene scene, RequestContext context) {
        try {
            if (StringUtils.isEmpty(scene.getName())) {
                return new GeneralResult<>(false, "名称不能为空");
            }
            int sameName = taskBackupSceneRepository.countSameNameOrCode(scene.getName(), null);
            if (sameName > 0) {
                return new GeneralResult<>(false, "添加失败，该名称已存在，请尝试其他名称");
            }
            if (StringUtils.isEmpty(scene.getPath())) {
                return new GeneralResult<>(false, "路径不能为空");
            }
            scene.setGmtCreate(new Date());
            scene.setGmtModify(new Date());
            scene.setCreatorId(context.getTarget());
            scene.setMenderId(context.getTarget());
            scene.setDeleted(false);
            taskBackupSceneRepository.save(scene);
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            log.error("create taskBackupScene fail:", e);
            return new GeneralResult<>(false, "创建失败");
        }
    }

    /**
     * 配置场景修改
     * 
     * @param scene
     *            实例信息
     * @param context
     *            系统参数信息
     * @return
     */
    @Override
    public GeneralResult modify(TaskBackupScene scene, RequestContext context) {
        String path = TaskBackupScene.class.getSimpleName() + "_" + scene.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskBackupScene oldScene = taskBackupSceneRepository.query(scene.getId());
            if (null == oldScene) {
                return new GeneralResult<>(false, "当前修改的数据不存在");
            }
            if (StringUtils.isEmpty(scene.getName())) {
                return new GeneralResult<>(false, "名称不能为空");
            }
            if (!oldScene.getName().equals(scene.getName())) {
                int sameName = taskBackupSceneRepository.countSameNameOrCode(scene.getName(), null);
                if (sameName > 0) {
                    return new GeneralResult<>(false, "该名称已存在，请尝试其他名称");
                }
            }
            if (!scene.getPath().equals(oldScene.getPath())) {
                return new GeneralResult<>(false, "路径不能修改");
            }
            oldScene.setName(scene.getName());
            oldScene.setRemark(scene.getRemark());
            oldScene.setGmtModify(new Date());
            oldScene.setMenderId(context.getTarget());
            taskBackupSceneRepository.update(oldScene);
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("modify taskBackupScene fail:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    /**
     * 配置场景删除
     * 
     * @param id
     *            数据主键
     * @param context
     *            系统参数信息
     * @return
     */
    @Override
    public GeneralResult remove(Long id, RequestContext context) {
        String path = TaskBackupScene.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskBackupScene scene = taskBackupSceneRepository.query(id);
            if (null == scene) {
                return new GeneralResult<>(false, "当前删除的数据不存在");
            }
            List<Long> sceneIds = new ArrayList<>();
            sceneIds.add(id);
            List<TaskBackup> backups = taskBackupRepository.listBySenceIds(sceneIds);
            if (CollectionUtils.isNotEmpty(backups)) {
                return new GeneralResult<>(false, "请先删除对应任务数据");
            }
            taskBackupSceneRepository.delete(id, context.getTarget());
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("remove taskBackupScene fail:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }
}
