package com.bocloud.cop.service.backup;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import com.bocloud.cop.model.Target;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.curator.framework.recipes.leader.LeaderLatchListener;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.Trigger;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.backup.TaskBackup;
import com.bocloud.cop.entity.backup.TaskBackupInstance;
import com.bocloud.cop.entity.backup.TaskBackupScene;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.ExecuteMode;
import com.bocloud.cop.enums.Status;
import com.bocloud.cop.enums.TaskParam;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.intf.backup.TaskBackupService;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.repository.TaskHostRepository;
import com.bocloud.cop.repository.backup.TaskBackupInstanceRepository;
import com.bocloud.cop.repository.backup.TaskBackupRepository;
import com.bocloud.cop.repository.backup.TaskBackupSceneRepository;
import com.bocloud.cop.service.builder.BackUpTaskBuilder;
import com.bocloud.cop.service.internal.CopInternalService;
import com.bocloud.cop.service.utils.JobScheduler;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.IDFactory;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @author huliwei
 * @date 2022/10/19
 */
@Service("taskBackupService")
@Slf4j
public class TaskBackupServiceImpl implements TaskBackupService {

    private static final String FIXED = "fixed";
    private static final String SKIPED = "skiped";
    private static final Map<Long, ScheduledFuture<?>> SCHEDULED_FUTURES = new ConcurrentHashMap<>();
    private ThreadPoolTaskScheduler threadPoolTaskScheduler = new ThreadPoolTaskScheduler();

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private TaskBackupRepository taskBackupRepository;
    @Autowired
    private TaskBackupSceneRepository taskBackupSceneRepository;
    @Autowired
    private TaskHostRepository taskHostRepository;
    @Autowired
    private TaskInstanceService taskInstanceService;
    @Autowired
    private JobScheduler taskScheduler;
    @Autowired
    private CurrentService currentService;
    @Autowired
    private CopInternalService copInternalService;
    @Autowired
    private TaskBackupInstanceRepository taskBackupInstanceRepository;

    @Override
    public GeneralResult<GridBean<TaskBackup>> list(Pager pager, RequestContext context) {
        try {
            GridBean<TaskBackup> gridBean = null;
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            if (!RequestContext.Catalog.Manager.equals(context.getCatalog())) {
                pager.getParams().add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
            }
            int total = taskBackupRepository.count(pager.getParams());
            List<TaskBackup> backups = taskBackupRepository.list(pager);
            if (CollectionUtils.isEmpty(backups)) {
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, backups);
                return new GeneralResult<>(true, gridBean, "查询任务成功");
            }
            for (TaskBackup backup : backups) {
                List<Long> ids = JSONArray.parseArray(backup.getHostIds()).toJavaList(Long.class);
                List<TaskHost> targets = taskHostRepository.queryByIds(ids);
                String hostIps = targets.stream().map(TaskHost::getIp).collect(Collectors.joining(","));
                backup.setHostIps(hostIps);
                backup.setCreator(redisTemplate.opsForValue().get("User_Name_" + backup.getCreatorId()));
                if (null != backup.getMenderId()) {
                    backup.setMender(redisTemplate.opsForValue().get("User_Name_" + backup.getMenderId()));
                }
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, backups);
            return new GeneralResult<>(true, gridBean, "查询任务成功");
        } catch (Exception e) {
            log.error("List task fail:", e);
            return new GeneralResult<>(false, "查询任务失败");
        }
    }

    @Override
    public GeneralResult<Void> create(TaskBackup backup, RequestContext context) {
        try {
            if (ListTool.isEmpty(backup.getTaskBackupScenes())) {
                return new GeneralResult<>(false, "场景不能为空");
            }
            if (StringUtils.isEmpty(backup.getHostIds())) {
                return new GeneralResult<>(false, "主机ip不能为空");
            }
            List<Long> hostIds = JSONArray.parseArray(backup.getHostIds()).toJavaList(Long.class);
            if (hostIds.size() > 50) {
                return new GeneralResult<>(false, "主机数量过大(大于50)，无法进行任务下发");
            }
            for (TaskBackupScene scene : backup.getTaskBackupScenes()) {
                TaskBackup entity = new TaskBackup();
                BeanUtils.copyProperties(backup, entity);
                entity.setSceneId(scene.getId());
                entity.setSceneName(scene.getName());
                entity.setPath(scene.getPath());
                entity.setStatus(Status.Inst.CREATED.name());
                entity.setCreatorId(context.getTarget());
                entity.setCode(IDFactory.instance().uuid());
                entity.setGmtCreate(new Date());
                taskBackupRepository.save(entity);
                log.info("backup--", entity);
                // 分立即执行任务和定时任务
                if (Status.TaskType.IMMEDIATE.name().equals(backup.getTaskType())) {
                    // 任务下发
                    this.start(entity, context);
                    taskBackupRepository.updateStatus(entity.getId(), Status.Task.EXECUTED.name(), context.getTarget());
                }
            }
            return new GeneralResult<>(true, "任务创建成功");
        } catch (Exception e) {
            log.error("create task error:", e);
            return new GeneralResult<>(false, "任务创建失败");
        }
    }

    @Override
    public TaskBackup basic(Long id) {
        try {
            return taskBackupRepository.query(id);
        } catch (Exception e) {
            log.error("Query taskBackup fail:", e);
            return null;
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = Task.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskBackup backup = taskBackupRepository.query(id);
            if (null == backup) {
                return new GeneralResult<>(false, "备份任务不存在");
            }
            boolean result = taskBackupRepository.delete(id, userId);
            return result ? new GeneralResult<>(true, "删除任务成功") : new GeneralResult<>(false, "删除任务失败");
        } catch (Exception e) {
            log.error("Remove task fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<Void> operate(String action, Long id, RequestContext context) {
        String path = TaskBackup.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskBackup backup = taskBackupRepository.query(id);
            if (null == backup) {
                return new GeneralResult<>(false, "任务不存在");
            }
            if ("run".equals(action)) {
                // 先修改一遍保留当前操作人员信息
                taskBackupRepository.updateStatus(id, backup.getStatus(), context.getTarget());
                return this.run(id, context.getTarget());
            } else if ("stop".equals(action)) {
                return this.stop(id, context.getTarget());
            }
            return new GeneralResult<>(false, "操作失败!");
        } catch (Exception e) {
            return GeneralResult.FAILED("操作失败!");
        }
    }

    @Override
    public GeneralResult<String> addScheduleIfLeader(Long id, RequestContext context) {
        if (!currentService.isLeader()) {
            log.info("收到定时任务[id={}]添加定时任务请求，当前服务非主服务，已经忽略", id);
            return new GeneralResult<>(true, SKIPED, "success");
        }
        try {
            TaskBackup timing = taskBackupRepository.query(id);
            Assert.notNull(timing, "定时任务数据不存在");
            this.addSchedule(timing);
            return new GeneralResult<>(true, FIXED, "success");
        } catch (Exception e) {
            log.error("AddScheduleIfLeader fail:", e);
            return new GeneralResult<>(false, "fail");
        }
    }

    @Override
    public GeneralResult<String> removeScheduleIfLeader(Long id) {
        if (!currentService.isLeader()) {
            log.info("收到定时任务[id={}]删除定时任务请求，当前服务非主服务，已经忽略", id);
            return new GeneralResult<>(true, SKIPED, "success");
        }
        try {
            TaskBackup timing = taskBackupRepository.query(id);
            Assert.notNull(timing, "定时任务数据不存在");
            this.removeSchedule(timing);
            return new GeneralResult<>(true, FIXED, "success");
        } catch (Exception e) {
            log.error("RemoveScheduleIfLeader fail:", e);
            return new GeneralResult<>(false, "fail");
        }
    }

    @Override
    public void onApplicationReadyEvent() {
        threadPoolTaskScheduler.setPoolSize(Runtime.getRuntime().availableProcessors());
        threadPoolTaskScheduler.setThreadNamePrefix("TimeInspect-");
        threadPoolTaskScheduler.initialize();
        final TaskBackupServiceImpl service = this;
        LeaderLatchListener listener = new LeaderLatchListener() {
            @Override
            public void isLeader() {
                List<TaskBackup> list = taskBackupRepository.findAllByStatus(Status.Task.ACTIVE.name());
                list.forEach(service::addSchedule);
            }

            @Override
            public void notLeader() {
                List<TaskBackup> list = taskBackupRepository.findAllByStatus(Status.Task.SUSPEND.name());
                list.forEach(service::removeSchedule);
            }
        };
        try {
            currentService.addListener(listener);
        } catch (Exception e) {
            log.error("定时任务初始化失败：{}", e.getMessage());
        }
        if (currentService.isLeader()) {
            listener.isLeader();
        }
    }

    private GeneralResult<Void> run(Long id, Long userId) {
        try {
            boolean success = copInternalService.scheduleBackup(id, HttpMethod.GET);
            if (success) {
                taskBackupRepository.updateStatus(id, Status.Task.ACTIVE.name(), userId);
            }
            return new GeneralResult<>(success, success ? "定时任务激活成功" : "启动定时任务激活失败");
        } catch (Exception e) {
            log.error("run fail:", e);
        }
        return new GeneralResult<>(false, "执行失败");
    }

    private GeneralResult<Void> stop(Long id, Long userId) {
        try {
            boolean success = copInternalService.scheduleBackup(id, HttpMethod.DELETE);
            if (success) {
                taskBackupRepository.updateStatus(id, Status.Task.SUSPEND.name(), userId);
            }
            return new GeneralResult<>(success, success ? "定时任务挂起成功" : "定时任务挂起失败");
        } catch (Exception e) {
            log.error("stop fail:", e);
        }
        return new GeneralResult<>(false, "执行失败");
    }

    private void addSchedule(TaskBackup backup) {
        if (SCHEDULED_FUTURES.containsKey(backup.getId())) {
            this.removeSchedule(backup);
        }
        Trigger trigger = new CronTrigger(backup.getCron());
        ScheduledFuture<?> future = threadPoolTaskScheduler.schedule(() -> {
            RequestContext context = new RequestContext();
            context.setTarget(backup.getMenderId());
            log.info("-----定时备份任务-------");
            this.start(backup, context);
        }, trigger);
        SCHEDULED_FUTURES.put(backup.getId(), future);
        log.info("定时任务[{}]已经添加到定时任务，调度策略：{}", backup.getName(), backup.getCron());
        log.info("定时任务：" + JSONObject.toJSONString(SCHEDULED_FUTURES.keySet()));
    }

    private void removeSchedule(TaskBackup backup) {
        ScheduledFuture<?> future = SCHEDULED_FUTURES.remove(backup.getId());
        if (future != null) {
            future.cancel(true);
        }
        log.info("定时任务[{}]已经从定时任务中移除", backup.getName());
    }

    private GeneralResult<Void> start(TaskBackup backup, RequestContext context) {
        long start = System.currentTimeMillis();
        try {
            List<Long> hostIds = JSONArray.parseArray(backup.getHostIds()).toJavaList(Long.class);
            if (hostIds.size() > 50) {
                return new GeneralResult<>(false, "主机数量过大(大于50)，无法进行任务下发");
            }
            TaskBackupScene scene = taskBackupSceneRepository.query(backup.getSceneId());
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("source", scene.getPath());
            jsonObject.put("name", backup.getName());
            jsonObject.put("sceneId", scene.getId());
            GeneralResult<Task> result = this.assembleTask(backup.getTarget(), jsonObject, 1L);
            if (result.isFailed() || null == result.getData()) {
                return new GeneralResult<>(false, result.getMessage());
            }
            Task task = result.getData();
            task.setType("backup");
            TaskInstance instance = taskInstanceService.initInstance(task, ExecuteMode.WEB.name(), context);
            if (null == instance) {
                return new GeneralResult<>(false, "执行失败：原因可能是作业节点分类参数格式错误");
            }
            TaskBackupInstance taskBackupInstance = new TaskBackupInstance();
            taskBackupInstance.setBackupId(backup.getId());
            taskBackupInstance.setTaskInstanceId(instance.getId());
            taskBackupInstanceRepository.save(taskBackupInstance);
            log.info("任务实例化用时：" + (System.currentTimeMillis() - start));
            start = System.currentTimeMillis();
            boolean flag = taskScheduler.insertTask(buildJob(instance));
            log.info("任务实例放入任务队列用时：" + (System.currentTimeMillis() - start));
            if (!flag) {
                return new GeneralResult<>(false, "任务下发失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new GeneralResult<>(true, "任务下发成功");
    }

    private GeneralResult<Task> assembleTask(String targetInfo, JSONObject object, Long userId) {
        try {
            Target target = JSONObject.parseObject(targetInfo,Target.class);
            Task task = BackUpTaskBuilder.getTelnetTaskTask(object, target, 1800);
            return new GeneralResult<>(true, task, "success");
        } catch (Exception e) {
            log.error("telnet命令任务下发失败", e);
            return new GeneralResult<>(false, "主机telnet命令任务下发失败");
        }
    }

    private Job buildJob(TaskInstance instance) {
        Job job = new Job();
        job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
        job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
        job.setTaskId(instance.getUuid());
        job.setParam(Common.NAME, instance.getName());
        job.setParam(TaskParam.TASKCODE.name(), instance.getCode());
        return job;
    }
}
