package com.bocloud.cop.service.compare;

import java.io.File;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.apache.commons.beanutils.locale.LocaleBeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.web.multipart.MultipartFile;

import com.bocloud.cop.entity.compare.StandardDocument;
import com.bocloud.cop.entity.compare.StandardDocumentVersion;
import com.bocloud.cop.intf.compare.StandardDocumentService;
import com.bocloud.cop.repository.compare.StandardDocumentRepository;
import com.bocloud.cop.repository.compare.StandardDocumentVersionRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-292 13:46:31
 * @Description:
 */
@Slf4j
@Service("standardDocumentService")
public class StandardDocumentServiceImpl implements StandardDocumentService {
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private StandardDocumentRepository standardDocumentRepository;
    @Autowired
    private StandardDocumentVersionRepository standardDocumentVersionRepository;

    @Value("${comparison.upload.path:/home/cmp/standard-file}")
    private String standardFilepath;

    @Override
    public GeneralResult<GridBean<StandardDocument>> list(Pager pager) {
        GridBean gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.standardDocumentRepository.count(pager.getParams());
            if (pager.getSimple()) {
                List<SimpleBean> beans = this.standardDocumentRepository.list(pager.getParams(), pager.getSorter());
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                List<StandardDocument> list = this.standardDocumentRepository.list(pager.getPage(), pager.getRows(),
                    pager.getParams(), pager.getSorter());
                for (StandardDocument document : list) {
                    if (null != document.getCreatorId()) {
                        document.setCreator(redisTemplate.opsForValue().get("User_Name_" + document.getCreatorId()));
                    }
                    if (null != document.getMenderId()) {
                        document.setMender(redisTemplate.opsForValue().get("User_Name_" + document.getMenderId()));
                    }
                }
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<GridBean<StandardDocument>>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("List Query Is Fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Void> save(Long versionId, String remark, MultipartFile file, RequestContext context) {
        try {
            String fileName = URLDecoder.decode(file.getOriginalFilename(), "UTF-8");
            boolean result = standardDocumentRepository.exists(fileName, versionId);
            if (result) {
                return new GeneralResult<>(false, "该文件已存在");
            }
            StandardDocumentVersion version = standardDocumentVersionRepository.query(versionId);
            if (null == version) {
                return new GeneralResult<>(false, "标准文件版本不存在");
            }
            String filePath = standardFilepath + File.separatorChar + version.getCode() + File.separatorChar;
            // 判断路径是否存在,不存在则创建
            File newfile = new File(filePath);
            if (!newfile.exists()) {
                newfile.mkdirs();
            }
            file.transferTo(new File(filePath + fileName));
            log.info("uploadStandardFile filePath :" + filePath + fileName);
            StandardDocument document = new StandardDocument();
            document.setName(fileName);
            document.setVersionId(versionId);
            document.setRemark(remark);
            document.setCreatorId(context.getTarget());
            document.setMenderId(context.getTarget());
            document.setGmtCreate(new Date());
            standardDocumentRepository.save(document);
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            log.error("Creation failed:", e);
            return new GeneralResult<>(false, "创建失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(StandardDocument param, RequestContext context) {
        String path = StandardDocumentServiceImpl.class.getSimpleName() + "_" + param.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            StandardDocument document = standardDocumentRepository.query(param.getId());
            if (null == document) {
                return new GeneralResult<>(false, "数据不存在");
            }
            if (!document.getName().equals(param.getName())) {
                boolean result = standardDocumentRepository.exists(param.getName(), param.getVersionId());
                if (result) {
                    return new GeneralResult<>(false, "此名称已存在!");
                }
            }
            LocaleBeanUtils.copyProperties(document, param);
            document.setGmtModify(new Date());
            document.setMenderId(context.getTarget());
            standardDocumentRepository.update(document);
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("Update failed:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = StandardDocumentServiceImpl.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            StandardDocument document = standardDocumentRepository.query(id);
            if (null == document) {
                return new GeneralResult<>(false, "数据不存在");
            }
            StandardDocumentVersion version = standardDocumentVersionRepository.query(document.getVersionId());
            if (null == version) {
                return new GeneralResult<>(false, "文件版本数据不存在");
            }
            File file = new File(
                standardFilepath + File.separatorChar + version.getCode() + File.separatorChar + document.getName());
            // 判断文件是否存在
            if (!file.exists()) {
                log.warn("deleteStandardFile file is not exist");
            } else {
                file.delete();
            }
            standardDocumentRepository.delete(id, userId);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("Delete failed:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }

    @Override
    public GeneralResult<StandardDocument> detail(Long id) {
        try {
            StandardDocument document = standardDocumentRepository.query(id);
            if (null == document) {
                return new GeneralResult<>(false, "数据不存在");
            }
            return new GeneralResult<>(true, document, "查询成功");
        } catch (Exception e) {
            log.error("Details query failed:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }
}