package com.bocloud.cop.service.inspection;

import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectItem;
import com.bocloud.cop.entity.inspection.InspectRecord;
import com.bocloud.cop.entity.inspection.InspectRecordItem;
import com.bocloud.cop.entity.inspection.RecordItem;
import com.bocloud.cop.intf.inspection.DataAdaptorService;
import com.bocloud.cop.intf.inspection.InspectItemService;
import com.bocloud.cop.model.inspection.ScriptLog;
import com.bocloud.cop.repository.inspection.InspectRecordItemRepository;
import com.bocloud.cop.repository.inspection.InspectRecordRepository;
import com.bocloud.cop.service.utils.GenericEntityUtil;
import com.megatron.common.model.GeneralResult;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service("compliance")
public class ComplianceDataAdaptorServiceImpl implements DataAdaptorService {

    @Autowired
    private InspectRecordRepository inspectRecordRepository;
    @Autowired
    private InspectRecordItemRepository inspectRecordItemRepository;
    @Autowired
    private InspectItemService inspectItemService;

    @Override
    public void handleContent(RecordItem item, String content) {
        InspectRecordItem recordItem = (InspectRecordItem)item;
        InspectItem inspectItem = new InspectItem();
        ScriptLog scriptLog;// 存储日志解析结果
        // 解析日志
        try {
            if (StringUtils.isEmpty(content)) {
                throw new RuntimeException("解析结果失败：未查询到输出日志");
            }
            GeneralResult<?> result = inspectItemService.get(recordItem.getItemId());
            if (result.isFailed()) {
                throw new RuntimeException("未查询到合规项：" + result.getMessage());
            }
            inspectItem = (InspectItem)result.getData();
            recordItem.setTargetRule(inspectItem.getTargetRule());
            // 实际解析日志
            scriptLog = ScriptLog.parasLog(content, null);
        } catch (RuntimeException e) {
            scriptLog = ScriptLog.errorLog(e.getMessage());
        } catch (Exception e) {
            log.error("日志解析失败", e);
            scriptLog = ScriptLog.errorLog("服务器错误,请联系管理员查询日志");
        }

        // 将合规结果转存数据库
        try {
            if (scriptLog.getScriptStatus() != null
                && scriptLog.getScriptStatus() == ScriptLog.RunStatus.SUCCESS.ordinal()) {
                recordItem.setStatus(InspectRecordItem.Status.SUCCESS.name());
            } else {
                recordItem.setStatus(InspectRecordItem.Status.ERROR.name());
                recordItem.setScriptErrorInfo(scriptLog.getScriptErrorInfo());
            }
            recordItem.setInspectResult(scriptLog.getInspectResult());
            recordItem.setLog(JSONObject.toJSONString(scriptLog.getInspectMessage()));
            if (scriptLog.getInspectResult() != null) {
                JSONArray rules = JSONObject.parseArray(inspectItem.getTargetRule());
                recordItem.setWarning(getOutCome(rules, scriptLog.getInspectResult())); // 是否合规
                recordItem.setResult(scriptLog.getInspectResult());
            }
        } catch (Exception e) {
            log.error("处理【合规检查记录】失败", e);
        }
        // 更新数据
        try {
            GenericEntityUtil.fillModifyData(recordItem);
            inspectRecordItemRepository.update(recordItem);
        } catch (Exception e) {
            log.error("更新【合规检查记录】失败", e);
        }
    }

    @Override
    public void synRecord(Long recordId) {
        InspectRecord record = inspectRecordRepository.findOne(recordId);
        Long unfinishedCountItemByRecordId = inspectRecordItemRepository
            .countByStatusAndRecordId(InspectRecordItem.Status.RUNNING.name(), record.getId());

        if (unfinishedCountItemByRecordId == 0) {
            Double totalTime = (System.currentTimeMillis() - record.getGmtCreate().getTime()) / 1000.0;
            Long errorCountItemByRecordId = inspectRecordItemRepository
                .countByStatusAndRecordId(InspectRecordItem.Status.ERROR.name(), record.getId());
            String status =
                errorCountItemByRecordId > 0 ? InspectRecord.Status.ERROR.name() : InspectRecord.Status.SUCCESS.name();
            inspectRecordRepository.updateStatus(record.getId(), status, false, totalTime);
            updateRecord(record.getId());
        }
    }

    /**
     * 更新执行结果
     *
     * @param recordId
     * @throws Exception
     */
    private void updateRecord(Long recordId) {
        try {
            List<InspectRecordItem> items = inspectRecordItemRepository.findAllByRecordId(recordId);
            if (0 == items.size()) {
                return;
            }
            List<InspectRecordItem> passItems = items.stream().filter(a -> a.getWarning()).collect(Collectors.toList()); // 合规检查项
            List<InspectRecordItem> errorItems =
                items.stream().filter(a -> InspectRecord.Status.ERROR.name().equals(a.getStatus())).collect(Collectors.toList()); // 错误
            List<InspectRecordItem> noPassAndErrorItems = items.stream()
                .filter(a -> !a.getWarning() && !InspectRecord.Status.ERROR.name().equals(a.getStatus())).collect(Collectors.toList()); // 不合规
            Map<String, List<InspectRecordItem>> groupBySex =
                noPassAndErrorItems.stream().collect(Collectors.groupingBy(InspectRecordItem::getIp)); // 跟据ip分组
                                                                                                       // 不合规的主机
            InspectRecord record = inspectRecordRepository.findOne(recordId);
            int itemNum = items.size(); // 总数
            int passItem = passItems.size(); // 合规数
            int noPassHost = groupBySex.size(); // 不合规服务器数
            int errorItem = errorItems.size(); // 错误数
            record.setNoPassItemNum(itemNum - passItem - errorItem); // 不合规项
            record.setNoPassHostNum(noPassHost); // 不合规服务器数
            DecimalFormat df = new DecimalFormat("0.00");// 设置保留位数
            record.setPassRates(df.format((float)passItem / itemNum * 100)); // 合规通过率
            inspectRecordRepository.update(record);
        } catch (Exception e) {
            log.error("更新历史记录数据失败！");
        }
    }

    /**
     * 解析结果判断是否合规
     *
     * @param rules
     * @param scriptResult
     * @return
     */
    private boolean getOutCome(JSONArray rules, String scriptResult) {
        try {
            List<String> messageGroups = new ArrayList<>();
            if (scriptResult.contains("@@@@")) {
                String[] messages = scriptResult.split("@@@@");
                for (String s : messages) {
                    String[] split = s.split("\\+\\+\\+\\+");
                    messageGroups.add(split[1]);
                }
            } else {
                String[] messages = scriptResult.split("[|][|][|][|]");
                messageGroups.addAll(Arrays.asList(messages));
            }
            Map<String, JSONObject> rulesMap = new HashMap<>();
            for (int j = 0; j < rules.size(); j++) { // 该基线的规则
                JSONObject r = rules.getJSONObject(j);
                String key = r.getString("name");
                rulesMap.put(key, r); // 将该基线的规则放进map集合中
            }
            boolean in = false;// 是否有值进行比较了
            for (String result : messageGroups) {
                String key = "";
                String value = "";
                String[] param = result.split("====");
                if (param.length < 2) { // 对应的值为空
                    key = param[0];
                } else {
                    key = param[0];
                    value = param[1];// 脚本执行结果的值
                }
                if (rulesMap.containsKey(key)) { // 如果脚本返回的值有基线需要判断的
                    JSONObject r = rulesMap.get(key);
                    in = true;
                    String rule = r.getString("rule"); // 规则 大于，等于，包含等
                    String itemValue = r.getString("value"); // 基线设置的阈值
                    Boolean compliance = r.getBoolean("pass");
                    if ("".equals(value) || // 脚本结果值不为空
                        !getPass(rule, itemValue, value, compliance)) { // 脚本执行结果的值和基线规则对比，结果是否合规
                        return false;
                    }
                } else {
                    log.error("基线设置的规则的值与脚本执行返回值不匹配！");
                }
            }
            if (in) {
                return true;
            }
            return false;

        } catch (Exception e) {
            return false;
        }
    }

    /**
     * 匹配规则
     *
     * @param rule
     * @param itemValue
     * @param resultValue
     * @param pass
     * @return
     */
    private boolean getPass(String rule, String itemValue, String resultValue, Boolean pass) {
        try {
            switch (rule) {
                case "GT": // 大于
                    // if (resultValue.contains("%")) {
                    //
                    // }
                    if (Double.parseDouble(resultValue) > Double.parseDouble(itemValue)) {
                        return pass;
                    }
                    return !pass;
                case "EQ": // 等于
                    if (resultValue.equals(itemValue)) {
                        return pass;
                    }
                    return !pass;
                case "NEQ": // 不等于
                    if (!resultValue.equals(itemValue)) {
                        return pass;
                    }
                    return !pass;
                case "LT": // 小于
                    if (Double.parseDouble(resultValue) < Double.parseDouble(itemValue)) {
                        return pass;
                    }
                    return !pass;
                case "GE": // 大于等于
                    if (Double.parseDouble(resultValue) >= Double.parseDouble(itemValue)) {
                        return pass;
                    }
                    return !pass;
                case "LE": // 小于等于
                    if (Double.parseDouble(resultValue) <= Double.parseDouble(itemValue)) {
                        return pass;
                    }
                    return !pass;
                case "IN": // 包含
                    if (resultValue.contains("$$$$")) { // 判断是否是精确匹配包含关系，比如判断 abc1$$$$abc2$$$$abc是否包含abc
                        String[] strings = resultValue.split("\\$\\$\\$\\$");
                        for (int i = 0; i < strings.length; i++) {
                            if (strings[i].equals(itemValue)) {
                                return pass;
                            }
                        }
                    } else {
                        if (resultValue.contains(itemValue)) {
                            return pass;
                        }
                    }
                    return !pass;
                case "NIN": // 不包含
                    if (resultValue.contains("$$$$")) { // 判断是否是精确匹配包含关系，比如判断 abc1$$$$abc2$$$$abc是否包含abc
                        String[] strings = resultValue.split("\\$\\$\\$\\$");
                        boolean mark = false;
                        for (int i = 0; i < strings.length; i++) {
                            if (strings[i].equals(itemValue)) {
                                mark = true;
                            }
                        }
                        if (!mark) {
                            return pass;
                        }
                    } else {
                        if (!resultValue.contains(itemValue)) {
                            return pass;
                        }
                    }
                    return !pass;
                default:
                    log.error("规则不匹配！");
            }
        } catch (Exception e) {
            log.error("脚本返回值与基线规则对比失败！脚本返回值为{} ，基线值为{}", resultValue, itemValue);
        }
        return false;
    }
}
