package com.bocloud.cop.service.inspection;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Base64;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Script;
import com.bocloud.cop.entity.inspection.InspectItem;
import com.bocloud.cop.entity.inspection.InspectItemWarn;
import com.bocloud.cop.entity.inspection.InspectWarnLevel;
import com.bocloud.cop.enums.KeyWordEnum;
import com.bocloud.cop.enums.ScriptTypeEnum;
import com.bocloud.cop.intf.inspection.InspectItemService;
import com.bocloud.cop.model.inspection.WarningRuleEntity;
import com.bocloud.cop.repository.ScriptRepository;
import com.bocloud.cop.repository.inspection.InspectGroupItemRepository;
import com.bocloud.cop.repository.inspection.InspectItemRepository;
import com.bocloud.cop.repository.inspection.InspectItemWarnRepository;
import com.bocloud.cop.repository.inspection.InspectTemplateItemRepository;
import com.bocloud.cop.repository.inspection.InspectWarnRepository;
import com.bocloud.cop.service.utils.GenericEntityUtil;
import com.bocloud.cop.service.utils.IgnorePropertiesUtil;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class InspectItemServiceImpl implements InspectItemService {

    @Autowired
    private InspectItemRepository inspectItemRepository;
    @Autowired
    private InspectWarnRepository inspectWarnRepository;
    @Autowired
    private InspectItemWarnRepository inspectItemWarnRepository;
    @Autowired
    private InspectGroupItemRepository inspectGroupItemRepository;
    @Autowired
    private InspectTemplateItemRepository inspectItemTemplateRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private ScriptRepository scriptRepository;

    @Override
    public GeneralResult<GridBean<InspectItem>> list(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = inspectItemRepository.count(pager.getParams());
            List<InspectItem> items = inspectItemRepository.list(pager);
            for (InspectItem item : items) {
                item.setItemParam(null);
                item.setCreator(redisTemplate.opsForValue().get("User_Name_" + item.getCreatorId()));
                if (null != item.getMenderId()) {
                    item.setMender(redisTemplate.opsForValue().get("User_Name_" + item.getMenderId()));
                }
            }
            GridBean<InspectItem> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, items);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("List item fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<String>> loadSameParams(String ids) {
        try {
            if (StringUtils.isEmpty(ids)) {
                throw new RuntimeException("请选择至少一个检查项");
            }
            String[] array = ids.split(",");
            if (array.length == 0) {
                throw new RuntimeException("请选择至少一个检查项");
            }
            Set<Long> set = new HashSet<>();
            for (String id : array) {
                try {
                    set.add(Long.parseLong(id));
                } catch (Exception e) {
                    throw new RuntimeException("检查项id格式错误");
                }
            }
            return new GeneralResult<>(true, loadParams(set), "查询成功");
        } catch (Exception e) {
            log.error("List item fail:", e);
            return new GeneralResult<>(false, "查询失败：" + e.getMessage());
        }
    }

    private Boolean isRelatedTemplate(Long id) {
        return inspectItemTemplateRepository.count(id).size() > 0;
    }

    private List<String> loadParams(Set<Long> ids) throws RuntimeException {
        if (ids.size() == 0) {
            throw new RuntimeException("巡检项id格式错误");
        }
        List<InspectItem> items = inspectItemRepository.findAllByIdIn(ids);
        if (items.size() != ids.size()) {
            for (InspectItem item : items) {
                ids.remove(item.getId());
            }
            String notFoundIds = StringUtils.join(ids, ",");
            throw new RuntimeException("检查项不存在[" + notFoundIds + "]");
        }

        Set<String> set = itemToParam(items.get(0));
        for (int i = 1; i < items.size(); i++) {
            set.retainAll(itemToParam(items.get(i)));
        }
        return new ArrayList<>(set);
    }

    private Set<String> itemToParam(InspectItem item) {
        String scriptParam = item.getScriptParam();
        Set<String> paramsSet = new HashSet<>();
        JSONArray params = JSON.parseArray(scriptParam);
        if (params == null || params.size() == 0) {
            return paramsSet;
        }
        for (Object param : params) {
            String key = ((JSONObject)param).getString("name");
            paramsSet.add(key);
        }
        return paramsSet;
    }

    @Override
    public GeneralResult<InspectItem> get(Long id) {
        try {
            InspectItem item = inspectItemRepository.query(id);
            String content = item.getScriptContent() == null ? null
                : new String(Base64.getDecoder().decode(item.getScriptContent().getBytes()), StandardCharsets.UTF_8);
            item.setScriptContent(content);
            if (item.getSensitiveParam() != null && item.getSensitiveParam()) {
                item.setItemParam(null);
            } else {
                String param = item.getItemParam() == null ? null
                    : new String(Base64.getDecoder().decode(item.getItemParam().getBytes()), StandardCharsets.UTF_8);
                item.setItemParam(param);
            }
            // 更新告警模板 巡检有告警模板，合规没有告警模板
            if (item.getType().equalsIgnoreCase(InspectItem.Type.INSPECTION.name())) {
                JSONArray jsonArray = JSONArray.parseArray(item.getTargetRule());
                if (jsonArray != null) {
                    item.setTargetRule(getWarnByItemId(jsonArray).toJSONString());
                }
            }
            return new GeneralResult<>(true, item, "查询成功");
        } catch (Exception e) {
            log.error("Query item detail fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public GeneralResult<Void> save(InspectItem item, Long userId) {
        try {
            if (StringUtils.isEmpty(item.getName())) {
                return new GeneralResult<>(false, "请输入巡检名称");
            }
            if (StringUtils.isEmpty(item.getItemType())) {
                return new GeneralResult<>(false, "请选择指标分类");
            }
            boolean exists = inspectItemRepository.exists(item.getName(), item.getType());
            if (exists) {
                return new GeneralResult<>(false, "名称已存在，请重新输入");
            }
            if (StringUtils.isEmpty(item.getScriptType())) {
                return new GeneralResult<>(false, "脚本类型不能为空");
            }
            if (!ScriptTypeEnum.SCRIPT_TYPE.getValue().contains(item.getScriptType())) {
                return new GeneralResult<>(false, "不支持的脚本类型：" + item.getScriptType());
            }
            if (StringUtils.isEmpty(item.getScriptContent())) {
                return new GeneralResult<>(false, "请输入脚本内容");
            }
            if (item.getDescription() != null && item.getDescription().length() > 50) {
                return new GeneralResult<>(false, "脚本描述不能大于50字符");
            }
            if (item.getSensitiveParam() == null) {
                item.setSensitiveParam(false);
            }
            encodeBase64(item);
            GenericEntityUtil.fillCreateData(item, userId);
            item.setScriptName(getScriptName(item.getScriptId()));
            item.setStatus(InspectItem.Status.APPROVAL.name());
            // 巡检参数、告警校验
            if (item.getType().equalsIgnoreCase(InspectItem.Type.INSPECTION.name())) {
                verify(item.getMultiCompare(), item.getTargetRule(), item.getScriptParam());
            }
            inspectItemRepository.save(item);
            // 巡检项与告警关系入库
            if (item.getType().equalsIgnoreCase(InspectItem.Type.INSPECTION.name())) {
                JSONArray jsonArray = JSONArray.parseArray(item.getTargetRule());
                saveItemWarn(jsonArray, item.getId());
            }
            return new GeneralResult<>(true, "保存成功");
        } catch (Exception e) {
            log.error("错误为：", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new GeneralResult<>(false, "保存失败:" + e.getMessage());
        }
    }

    private void verify(Boolean multiCompare, String rule, String params) {
        try {
            if (!StringUtils.isEmpty(params)) {
                List<JSONObject> objects = JSONObject.parseArray(params, JSONObject.class);
                Set<String> set = new HashSet<>();
                for (JSONObject object : objects) {
                    String key = object.getString("name");
                    if (StringUtils.isEmpty(key)) {
                        throw new IllegalArgumentException("参数名称不能为空");
                    }
                    for (String value : KeyWordEnum.RESOURCE.getValue()) {
                        if (value.equals(key)) {
                            throw new IllegalArgumentException("内置[参数名称][" + value + "]不可使用，请选用其他参数名称");
                        }
                        if (!value.matches("^[a-zA-Z][a-zA-Z0-9_]*$")) {
                            throw new IllegalArgumentException("内置[参数名称][" + value + "]不合法，请以字母开头，仅包含字母、数字、下划线");
                        }
                    }
                    if (set.contains(key)) {
                        throw new IllegalArgumentException("参数名称重复:[" + key + "]");
                    }
                    set.add(key);
                }
            }
        } catch (Exception e) {
            throw new IllegalArgumentException("参数校验失败：" + e.getMessage());
        }
        try {
            JSONArray array = JSONArray.parseArray(rule);
            if (array != null) {
                rule = getWarnByItemId(array).toJSONString();
            }
            WarningRuleEntity.paras(rule, multiCompare, null);
        } catch (Exception e) {
            throw new IllegalArgumentException("阈值校验失败：" + e.getMessage());
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public GeneralResult<Void> update(InspectItem item, Long id, Long userId) {
        item.setId(id);
        String path = InspectItem.class.getSimpleName() + "_" + item.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Boolean isRelated = inspectGroupItemRepository.count(item.getId()) > 0;
            if (isRelated) {
                return new GeneralResult<>(false, "该指标已被关联使用，请解除关联后尝试");
            }
            if (StringUtils.isEmpty(item.getScriptType())) {
                return new GeneralResult<>(false, "脚本类型不能为空");
            }
            if (!ScriptTypeEnum.SCRIPT_TYPE.getValue().contains(item.getScriptType())) {
                return new GeneralResult<>(false, "不支持的脚本类型：" + item.getScriptType());
            }
            if (StringUtils.isEmpty(item.getScriptContent())) {
                return new GeneralResult<>(false, "请输入脚本内容");
            }
            if (null != item.getDescription() && item.getDescription().length() > 50) {
                return new GeneralResult<>(false, "脚本描述不能大于50字符");
            }
            if (item.getTimeout() == null) {
                return new GeneralResult<>(false, "超时时间不能为空");
            }
            InspectItem bean = inspectItemRepository.query(item.getId());
            if (bean == null) {
                return new GeneralResult<>(false, "未查询到要修改的指标");
            }
            if (!bean.getName().equals(item.getName())) {
                boolean exists = inspectItemRepository.exists(item.getName(), item.getType());
                if (exists) {
                    return new GeneralResult<>(false, "名称已存在，请重新输入");
                }
            }
            String oldParams = bean.getItemParam();
            BeanUtils.copyProperties(item, bean, IgnorePropertiesUtil.getNullPropertyNames(item));
            GenericEntityUtil.fillModifyData(bean, userId);
            bean.setMender(redisTemplate.opsForValue().get("User_Name_" + userId));
            encodeBase64(bean);
            if (item.getParamChanged() == null || !item.getParamChanged()) {
                bean.setItemParam(oldParams);
            }
            item.setScriptName(getScriptName(item.getScriptId()));
            if (item.getType().equalsIgnoreCase(InspectItem.Type.INSPECTION.name())) {
                // 巡检参数、告警校验
                verify(item.getMultiCompare(), item.getTargetRule(), item.getScriptParam());
                // 巡检项与告警关系入库
                JSONArray array = JSONArray.parseArray(item.getTargetRule());
                updateItemWarn(array, item.getId());
            }
            inspectItemRepository.update(bean);
            return new GeneralResult<>(true, "更新成功");
        } catch (Exception e) {
            log.error("错误为：", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new GeneralResult<>(false, "保存失败:" + e.getMessage());
        }
    }

    @Transactional
    @Override
    public GeneralResult<Void> delete(Long id, Long userId) {
        String path = InspectItem.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Boolean isRelated = inspectGroupItemRepository.count(id) > 0;
            if (isRelated) {
                return new GeneralResult<>(false, "该指标已被关联使用，请解除关联后尝试");
            }
            Boolean isRelatedTemplate = isRelatedTemplate(id);
            if (isRelatedTemplate) {
                return new GeneralResult<>(false, "该指标已被模板关联使用，请解除关联后尝试");
            }
            InspectItem item = inspectItemRepository.query(id);
            if (item == null) {
                return new GeneralResult<>(false, "该指标不存在,删除失败!");
            }
            inspectItemRepository.delete(id, userId);
            inspectGroupItemRepository.deleteByItemId(id, userId);
            inspectItemWarnRepository.deleteByItemId(id);
            return new GeneralResult<>(true, "删除成功！");
        } catch (Exception e) {
            log.error("错误为：", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    private void encodeBase64(InspectItem item) {
        String content =
            new String(Base64.getEncoder().encode(item.getScriptContent().getBytes(StandardCharsets.UTF_8)));
        item.setScriptContent(content);
        String param =
            item.getItemParam() == null ? null : new String(Base64.getEncoder().encode(item.getItemParam().getBytes()));
        item.setItemParam(param);
    }

    /**
     * 通过阈值json 获取告警信息
     *
     * @param array
     * @return
     */
    private JSONArray getWarnByItemId(JSONArray array) {
        for (int i = 0; i < array.size(); i++) {
            Long warnId = array.getJSONObject(i).getLong("warnId");
            if (null == warnId) {
                throw new RuntimeException("告警信息不完整！");
            }
            InspectWarnLevel level = inspectWarnRepository.findById(warnId);
            if (null == level) {
                throw new RuntimeException("对应的告警等级不存在！");
            }
            array.getJSONObject(i).put("warnLevel", level.getWarnLevel());
        }
        return array;
    }

    /**
     * @param array
     *            阈值json
     * @param id
     *            指标id
     */
    private void saveItemWarn(JSONArray array, Long id) {
        for (int i = 0; i < array.size(); i++) {
            Long warnId = array.getJSONObject(i).getLong("warnId");
            InspectItemWarn warn = new InspectItemWarn();
            warn.setItemId(id);
            warn.setWarnId(warnId);
            inspectItemWarnRepository.save(warn);
        }
    }

    /**
     * @param array
     *            阈值json
     * @param id
     *            指标id
     */
    private void updateItemWarn(JSONArray array, Long id) {
        for (int i = 0; i < array.size(); i++) {
            inspectItemWarnRepository.deleteByItemId(id); // 先删除关联信息，再重新构建
            Long warnId = array.getJSONObject(i).getLong("warnId");
            InspectItemWarn warn = new InspectItemWarn();
            warn.setItemId(id);
            warn.setWarnId(warnId);
            inspectItemWarnRepository.save(warn);
        }
    }

    /**
     * 取脚本名称
     *
     * @param id
     * @return
     */
    private String getScriptName(Long id) {
        Script query = scriptRepository.query(id);
        return Optional.ofNullable(query).map(Script::getName).orElse(null);
    }
}
