package com.bocloud.cop.service.inspection;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.curator.framework.recipes.leader.LeaderLatchListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.Trigger;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.Assert;

import com.bocloud.cop.entity.inspection.InspectReport;
import com.bocloud.cop.intf.inspection.InspectReportService;
import com.bocloud.cop.model.OperationParam;
import com.bocloud.cop.repository.inspection.InspectReportRepository;
import com.bocloud.cop.schedule.inspection.report.InspectReportSchedulerJob;
import com.bocloud.cop.service.internal.CopInternalService;
import com.bocloud.cop.service.utils.CronUtil;
import com.bocloud.cop.service.utils.DateTransfer;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 巡检报表Service实现
 */
@Slf4j
@Service
public class InspectReportServiceImpl implements InspectReportService {

    private static final Map<Long, ScheduledFuture<?>> SCHEDULED_FUTURES = new ConcurrentHashMap<>();
    private static final String FIXED = "fixed";
    private static final String SKIPED = "skiped";
    private ThreadPoolTaskScheduler taskScheduler = new ThreadPoolTaskScheduler();
    @Autowired
    private CopInternalService copInternalService;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private CurrentService currentService;
    @Autowired
    private InspectReportRepository inspectReportRepository;
    @Autowired
    private InspectReportSchedulerJob inspectReportSchedulerJob;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public GeneralResult<GridBean<InspectReport>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            Map<String, Object> anotherParams = new HashMap<>(16);
            for (Param param : pager.getParams()) {
                Map<String, Object> pm = param.getParam();
                Object type = pm.get("inspectType");
                Object name = pm.get("inspectName");
                if (type != null) {
                    anotherParams.put("category", type.toString().trim());
                    pm.remove("inspectType");
                }
                if (name != null) {
                    anotherParams.put("inspectName", name.toString().trim());
                    pm.remove("inspectName");
                }
            }
            int total = inspectReportRepository.count(pager.getParams(), anotherParams);
            List<InspectReport> list = inspectReportRepository.list(pager, anotherParams);
            for (final InspectReport report : list) {
                report.setCreator(redisTemplate.opsForValue().get("User_Name_" + report.getCreatorId()));
                report.setMender(redisTemplate.opsForValue().get("User_Name_" + report.getMenderId()));
            }
            GridBean<InspectReport> bean;
            if (pager.getSimple()) {
                bean = new GridBean<>(1, 1, total, list);
            } else {
                bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("List inspectReport fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Void> create(InspectReport entity, Long userId) {
        try {
            entity.setCreatorMender(userId);
            // 检查名称是否存在
            if (inspectReportRepository.countName(entity.getName()) > 0) {
                return new GeneralResult<>(false, "名称[" + entity.getName() + "]已经存在");
            }
            String status = entity.getStatus();
            entity.setStatus(InspectReport.Status.Off.name());
            inspectReportRepository.save(entity);
            // 选择启用时，启用报表
            if (InspectReport.Status.On.name().equals(status)) {
                Date currentTime = new Date();
                Date cronDate = DateTransfer.getDate(currentTime, entity.getPeriod(), entity.getChoose(),
                    entity.getGenerateTime(), entity.getWeek(), entity.getDay());
                if (cronDate.compareTo(currentTime) < 0) {
                    return new GeneralResult<>(false, "该日期不能小于当前日期");
                }
                GeneralResult<Void> result = this.start(entity.getId(), userId);
                return new GeneralResult<>(result.isSuccess(), "添加成功，" + result.getMessage());
            }
            return new GeneralResult<>(true, "添加成功");
        } catch (Exception e) {
            log.error("create inspectReport fail:", e);
            return new GeneralResult<>(false, "添加失败");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public GeneralResult<Void> modify(Long id, InspectReport entity, Long userId) {
        String path = InspectReport.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "获取分布式锁失败！");
            InspectReport bean = inspectReportRepository.query(id);
            if (null == bean) {
                log.warn("Inspect report data not exist");
                return new GeneralResult<>(false, "数据不存在!");
            }
            if (InspectReport.Status.On.name().equals(bean.getStatus())) {
                return new GeneralResult<>(false, "修改失败：巡检报表任务正在启动中");
            }
            BeanUtils.copyProperties(bean, entity);
            bean.setMenderId(userId);
            bean.setGmtModify(new Date());
            bean.setStatus(InspectReport.Status.Off.name());
            inspectReportRepository.update(bean);
            // 选择启用时，启用报表
            if (InspectReport.Status.On.name().equals(entity.getStatus())) {
                Date currentTime = new Date();
                Date cronDate = DateTransfer.getDate(currentTime, entity.getPeriod(), entity.getChoose(),
                    entity.getGenerateTime(), entity.getWeek(), entity.getDay());
                if (cronDate.compareTo(currentTime) < 0) {
                    return new GeneralResult<>(false, "该日期不能小于当前日期");
                }
                GeneralResult<Void> result = this.start(id, userId);
                return new GeneralResult<>(result.isSuccess(), "修改成功，" + result.getMessage());
            }
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("修改失败：", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public GeneralResult<InspectReport> remove(Long id, Long userId) {
        String path = InspectReport.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "获取分布式锁失败！");
            InspectReport entity = inspectReportRepository.query(id);
            if (null == entity) {
                return new GeneralResult<>(false, "数据不存在");
            }
            if (InspectReport.Status.On.name().equalsIgnoreCase(entity.getStatus())) {
                return new GeneralResult<>(false, "删除失败：巡检报表任务正在启动中");
            }
            inspectReportRepository.delete(id, userId);
            return new GeneralResult<>(true, entity, "删除成功");
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("移除失败:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }

    @Override
    public GeneralResult<InspectReport> detail(Long id) {
        try {
            InspectReport entity = inspectReportRepository.query(id);
            return entity == null ? new GeneralResult<>(false, "数据不存在") : new GeneralResult<>(true, entity, "获取详情成功");
        } catch (Exception e) {
            log.error("Query inspectReport fail:", e);
            return new GeneralResult<>(false, "获取详情失败");
        }
    }

    private GeneralResult<Void> start(Long id, Long userId) {
        boolean success = copInternalService.scheduleReportBroadcast(id, HttpMethod.POST);
        if (success) {
            inspectReportRepository.updateStatus(id, InspectReport.Status.On.name(), userId);
        }
        return new GeneralResult<>(success, success ? "启动巡检报表任务成功" : "启动巡检报表任务失败");
    }

    private GeneralResult<Void> stop(Long id, Long userId) {
        boolean success = copInternalService.scheduleReportBroadcast(id, HttpMethod.DELETE);
        if (success) {
            inspectReportRepository.updateStatus(id, InspectReport.Status.Off.name(), userId);
        }
        return new GeneralResult<>(success, success ? "禁用巡检报表任务成功" : "禁用巡检报表任务失败");
    }

    @Override
    public GeneralResult<String> addScheduleIfLeader(Long id) {
        if (!currentService.isLeader()) {
            log.info("收到巡检报表[id={}]添加定时任务请求，当前服务非主服务，已经忽略", id);
            return new GeneralResult<>(true, SKIPED, "success");
        }
        try {
            InspectReport report = inspectReportRepository.query(id);
            Assert.notNull(report, "巡检报表数据不存在");
            this.addSchedule(report);
            return new GeneralResult<>(true, FIXED, "success");
        } catch (Exception e) {
            log.error("Add inspect report schedule fail:");
            return new GeneralResult<>(false, "fail");
        }
    }

    @Override
    public GeneralResult<String> removeScheduleIfLeader(Long id) {
        if (!currentService.isLeader()) {
            log.info("收到巡检报表[id={}]删除定时任务请求，当前服务非主服务，已经忽略", id);
            return new GeneralResult<>(true, SKIPED, "success");
        }
        try {
            InspectReport report = inspectReportRepository.query(id);
            Assert.notNull(report, "巡检报表数据不存在");
            this.removeSchedule(report);
            return new GeneralResult<>(true, FIXED, "success");
        } catch (Exception e) {
            log.error("Remove inspect report schedule fail:");
            return new GeneralResult<>(false, "fail");
        }
    }

    private void addSchedule(InspectReport report) {
        if (SCHEDULED_FUTURES.containsKey(report.getId())) {
            this.removeSchedule(report);
        }
        Date cronDate = DateTransfer.getDate(new Date(), report.getPeriod(), report.getChoose(),
            report.getGenerateTime(), report.getWeek(), report.getDay());
        String cron = CronUtil.getLoopCron(cronDate, report.getPeriod(), report.getWeek(), report.getDay());
        Trigger trigger = new CronTrigger(cron);
        ScheduledFuture<?> future = taskScheduler.schedule(() -> inspectReportSchedulerJob.execute(report.getId()), trigger);
        SCHEDULED_FUTURES.put(report.getId(), future);
        log.info("巡检报表[{}]已经添加到定时任务，调度策略：{}", report.getName(), cron);
    }

    private void removeSchedule(InspectReport report) {
        ScheduledFuture<?> future = SCHEDULED_FUTURES.remove(report.getId());
        if (future != null) {
            future.cancel(true);
        }
        log.info("巡检报表[{}]已经从定时任务中移除", report.getName());
    }

    @Override
    public void onApplicationReadyEvent() {
        taskScheduler.setPoolSize(Runtime.getRuntime().availableProcessors() * 10);
        taskScheduler.setThreadNamePrefix("InspectReport-");
        taskScheduler.initialize();
        InspectReportServiceImpl service = this;
        LeaderLatchListener listener = new LeaderLatchListener() {
            @Override
            public void isLeader() {
                List<InspectReport> reports = inspectReportRepository.queryStatus(InspectReport.Status.On.name());
                reports.forEach(service::addSchedule);
            }

            @Override
            public void notLeader() {
                List<InspectReport> reports = inspectReportRepository.queryStatus(InspectReport.Status.On.name());
                reports.forEach(service::removeSchedule);
            }
        };
        try {
            currentService.addListener(listener);
        } catch (Exception e) {
            log.error("巡检报表任务初始化失败：{}", e.getMessage());
        }
        if (currentService.isLeader()) {
            listener.isLeader();
        }
    }

    @Override
    public GeneralResult<Void> start(Long id, String action, OperationParam param, RequestContext user) {
        if (action == null) {
            action = param.getAction();
        }
        if ("start".equals(action)) {
            return this.start(id, user.getTarget());
        } else if ("stop".equals(action)) {
            return this.stop(id, user.getTarget());
        } else {
            return new GeneralResult<>(false, "不支持的操作类型");
        }
    }

}
