package com.bocloud.cop.service.inspection;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.Inspect;
import com.bocloud.cop.entity.inspection.InspectGroup;
import com.bocloud.cop.entity.inspection.InspectGroupItem;
import com.bocloud.cop.entity.inspection.InspectRecord;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.ComplianceCategory;
import com.bocloud.cop.enums.InspectCategory;
import com.bocloud.cop.enums.InspectTypeEnums;
import com.bocloud.cop.intf.inspection.ExecuteTaskService;
import com.bocloud.cop.intf.inspection.InspectRoutineService;
import com.bocloud.cop.model.OperationParam;
import com.bocloud.cop.model.StatResult;
import com.bocloud.cop.model.Target;
import com.bocloud.cop.model.WorkerHost;
import com.bocloud.cop.model.params.InspectCustomParams;
import com.bocloud.cop.model.params.ParamEntry;
import com.bocloud.cop.repository.inspection.InspectGroupItemRepository;
import com.bocloud.cop.repository.inspection.InspectGroupRepository;
import com.bocloud.cop.repository.inspection.InspectRecordRepository;
import com.bocloud.cop.repository.inspection.InspectRepository;
import com.bocloud.cop.repository.inspection.InspectTimingRepository;
import com.bocloud.cop.service.utils.GenericEntityUtil;
import com.bocloud.cop.service.utils.IgnorePropertiesUtil;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.DateTools;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class InspectRoutineServiceImpl implements InspectRoutineService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private InspectRepository inspectRepository;
    @Autowired
    private InspectRecordRepository inspectRecordRepository;
    @Autowired
    private InspectGroupRepository inspectGroupRepository;
    @Autowired
    private InspectGroupItemRepository inspectGroupItemRepository;
    @Autowired
    private InspectTimingRepository inspectTimingRepository;
    @Autowired
    private ExecuteTaskService taskService;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public GeneralResult<Inspect> get(Long id) {
        try {
            Inspect inspect = inspectRepository.findOne(id);
            if (inspect == null) {
                return new GeneralResult<>(false, inspect, "未查询到巡检记录");
            }
            List<InspectGroup> groups = inspectGroupRepository.findAllByRoutineId(id);
            if (ListTool.isEmpty(groups)) {
                return new GeneralResult<>(false, inspect, "查询对应主机组失败");
            }
            for (InspectGroup group : groups) {
                List<InspectGroupItem> items = inspectGroupItemRepository.findAllByRoutineGroupId(group.getId());
                if (ListTool.isEmpty(items)) {
                    continue;
                }
                List<Long> itemIds = new ArrayList<>();
                for (InspectGroupItem item : items) {
                    itemIds.add(item.getItemId());
                }
                group.setItemIds(itemIds);
            }
            inspect.setGroups(groups);
            return new GeneralResult<>(true, inspect, "查询成功");
        } catch (Exception e) {
            log.error("Query inspect routine detail fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Inspect> save(Inspect inspect, Long userId) {
        try {
            verify(inspect);
            boolean flag = inspectRepository.exists(inspect.getName(), inspect.getType());
            if (flag) {
                return new GeneralResult<>(false, "该名称已存在，请重新输入!");
            }
            GenericEntityUtil.fillCreateData(inspect, userId);
            inspect.setStatus(ApplyStatus.DEVELOPING.name());
            return new GeneralResult<>(true, baseSave(inspect, userId), "保存成功");
        } catch (Exception e) {
            log.error("create inspect routine error:", e);
            return new GeneralResult<>(false, "创建巡检失败");
        }
    }

    /**
     * 校验数据
     *
     * @param inspect
     */
    private void verify(Inspect inspect) {
        if (StringUtils.isBlank(inspect.getName())) {
            throw new RuntimeException("名称不能为空");
        }
        if (ListTool.isEmpty(inspect.getGroups())) {
            throw new RuntimeException("主机组不能为空");
        }
        for (InspectGroup group : inspect.getGroups()) {
            if (ListTool.isEmpty(group.getItemIds())) {
                throw new RuntimeException(String.format("主机组【%s】检查项不能为空", group.getName()));
            }
            if (StringUtils.isEmpty(group.getExecuteType())) {
                throw new RuntimeException(String.format("主机组【%s】执行方式不能为空", group.getName()));
            }
            String customParams = group.getCustomParams();
            if (StringUtils.isEmpty(customParams)) {
                continue;
            }
            List<InspectCustomParams> params = JSON.parseArray(customParams, InspectCustomParams.class);
            Map<String, Map<String, String>> dataMap = new HashMap<>();
            for (InspectCustomParams param : params) {
                // 检查空数据
                if (param.getResourceId() == null) {
                    throw new RuntimeException(String.format("主机组【%s】自定义参数【资源不能为空】", group.getName()));
                }
                if (param.getItemId() == null) {
                    throw new RuntimeException(String.format("主机组【%s】自定义参数【检查项】", group.getName()));
                }
                String key = param.getResourceId() + "-" + param.getItemId();
                // 检查重复数据
                Map<String, String> innerMap = dataMap.get(key);
                if (innerMap == null) {
                    List<ParamEntry> entries = param.getCustomParams();
                    if (!ListTool.isEmpty(entries)) {
                        innerMap = new HashMap<>();
                        for (ParamEntry entry : entries) {
                            if (innerMap.putIfAbsent(entry.getKey(), entry.getValue()) != null) {
                                throw new RuntimeException(String.format("主机组【%s】自定义参数【参数项%s-%s】中有重复的参数",
                                    group.getName(), param.getResourceName(), param.getItemName()));
                            }
                        }
                        dataMap.put(key, innerMap);
                    }
                } else {
                    throw new RuntimeException(String.format("主机组【%s】自定义参数【资源】和【检查项】选择重复", group.getName()));
                }
            }
        }
    }

    @Override
    public GeneralResult<Void> delete(Long id, Long userId) {
        String path = Inspect.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            if (isRelated(id)) {
                return new GeneralResult<>(false, "已关联定时检查，请解除关联后重试");
            }
            inspectRepository.delete(id, userId);
            inspectGroupRepository.deleteByRoutineId(id, userId);
            inspectGroupItemRepository.deleteByRoutineId(id, userId);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("delete inspect routine error:", e);
            return new GeneralResult<>(false, "删除巡检失败");
        }
    }

    @Override
    public GeneralResult<Inspect> update(Inspect inspect, Long userId) {
        String path = Inspect.class.getSimpleName() + "_" + inspect.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            verify(inspect);
            Inspect bean = this.get(inspect.getId()).getData();
            if (bean == null) {
                return new GeneralResult<>(false, "未查询到要修改的常规巡检");
            }
            if (!bean.getName().equals(inspect.getName())) {
                boolean flag = inspectRepository.exists(inspect.getName(), inspect.getType());
                if (flag) {
                    return new GeneralResult<>(false, "该名称已存在，请重新输入!");
                }
            }
            inspectGroupRepository.deleteByRoutineId(bean.getId(), userId);
            inspectGroupItemRepository.deleteByRoutineId(bean.getId(), userId);
            BeanUtils.copyProperties(inspect, bean, IgnorePropertiesUtil.getNullPropertyNames(inspect));
            GenericEntityUtil.fillModifyData(bean, userId);
            return new GeneralResult<>(true, baseUpdate(bean, userId), "更新成功");
        } catch (Exception e) {
            log.error("Update inspect routine fail:", e);
            return new GeneralResult<>(false, "修改巡检失败");
        }
    }

    private Inspect baseSave(Inspect inspect, Long userId) {
        List<InspectGroup> groups = inspect.getGroups();
        inspectRepository.save(inspect);
        for (InspectGroup group : groups) {
            GenericEntityUtil.fillCreateData(group, userId);
            group.setInspectId(inspect.getId());
        }
        List<InspectGroupItem> refList = new ArrayList<>();
        for (InspectGroup group : groups) {
            inspectGroupRepository.save(group);
            for (Long itemId : group.getItemIds()) {
                InspectGroupItem ref = new InspectGroupItem();
                ref.setInspectId(inspect.getId());
                ref.setItemId(itemId);
                ref.setGroupId(group.getId());
                refList.add(ref);
            }
        }
        boolean result = inspectGroupItemRepository.batchSave(refList, refList.size());
        if (!result) {
            throw new RuntimeException("保存失败");
        }
        return inspect;
    }

    private Inspect baseUpdate(Inspect inspect, Long userId) {
        List<InspectGroup> groups = inspect.getGroups();
        GenericEntityUtil.fillModifyData(inspect, userId);
        inspectRepository.update(inspect);
        for (InspectGroup group : groups) {
            GenericEntityUtil.fillCreateData(group, userId);
            group.setInspectId(inspect.getId());
        }
        List<InspectGroupItem> refList = new ArrayList<>();
        for (InspectGroup group : groups) {
            inspectGroupRepository.save(group);
            for (Long itemId : group.getItemIds()) {
                InspectGroupItem ref = new InspectGroupItem();
                ref.setInspectId(inspect.getId());
                ref.setItemId(itemId);
                ref.setGroupId(group.getId());
                refList.add(ref);
            }
        }
        boolean result = inspectGroupItemRepository.batchSave(refList, refList.size());
        if (!result) {
            throw new RuntimeException("保存失败");
        }
        return inspect;
    }

    private GeneralResult<Void> run(Long id, Long userId) {
        String path = Inspect.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            GeneralResult<Inspect> GeneralResult = this.get(id);
            if ((!GeneralResult.isSuccess()) || GeneralResult.getData() == null) {
                return new GeneralResult<>(false, "未查询到数据");
            }
            Inspect inspect = GeneralResult.getData();
            if (!ApplyStatus.APPROVED.name().equals(inspect.getStatus())) {
                return new GeneralResult<>(false, "选择的数据未发布，请发布后再执行！");
            }
            String name = "常规巡检-" + inspect.getName() + "-" + DateTools.formatTime2String(new Date(), "yyyyMMddHHmmss");
            ExecuteTaskService.RunData runData = this.getRunData(inspect, name);
            runData.setType(InspectRecord.Type.ROUTINE.ordinal());
            runData.setCode(Catalog.Task.STANDARD.getValue());
            return execute(runData, userId, "C-" + UUID.randomUUID());
        } catch (Exception e) {
            log.error("run inspect routine fail:", e);
            return new GeneralResult<>(false, "运行失败");
        }
    }

    @Override
    public GeneralResult<Void> release(final Long id, Long userId) {
        String path = Inspect.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            inspectRepository.updateStatus(id, ApplyStatus.APPROVED.name(), userId);
            return new GeneralResult<>(true, "修改常规作业状态成功");
        } catch (Exception e) {
            log.error("run inspect routine fail:", e);
            return new GeneralResult<>(false, "修改常规作业状态失败");
        }
    }

    @Override
    public GeneralResult<Void> recall(final Long id, Long userId) {
        String path = Inspect.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            if (isRelated(id)) {
                return new GeneralResult<>(false, "已关联了定时检查，请解除关联后重试");
            }
            inspectRepository.updateStatus(id, ApplyStatus.DEVELOPING.name(), userId);
            return new GeneralResult<>(true, "修改常规作业状态成功");
        } catch (Exception e) {
            log.error("run inspect routine fail:", e);
            return new GeneralResult<>(false, "修改常规作业状态失败");
        }
    }

    private boolean isRelated(Long id) {
        return inspectTimingRepository.countAllByRoutineId(id) > 0;
    }

    @Override
    public GeneralResult<GridBean<Inspect>> list(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = inspectRepository.count(pager.getParams());
            List<Inspect> items = inspectRepository.list(pager);
            for (Inspect item : items) {
                item.setCreator(redisTemplate.opsForValue().get("User_Name_" + item.getCreatorId()));
                if (null != item.getMenderId()) {
                    item.setMender(redisTemplate.opsForValue().get("User_Name_" + item.getMenderId()));
                }
                String category = item.getCategory();
                if (StringUtils.isEmpty(category)) {
                    continue;
                }
                if (InspectTypeEnums.INSPECTION.name().equalsIgnoreCase(item.getType())) {
                    item.setCategory(InspectCategory.valueOf(category).getName());
                } else {
                    item.setCategory(ComplianceCategory.valueOf(category).getName());
                }
            }
            GridBean<Inspect> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, items);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("List inspect routine fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public ExecuteTaskService.RunData getRunData(Inspect inspect, String name) {
        ExecuteTaskService.RunData runData = new ExecuteTaskService.RunData();
        runData.setName(name);
        runData.setGroups(inspect.getGroups().stream().map(this::getRunDataGroup).collect(Collectors.toList()));
        runData.setRoutineId(inspect.getId());
        runData.setCategory(inspect.getCategory());
        runData.setSource(inspect.getType());
        return runData;
    }

    private ExecuteTaskService.RunDataGroup getRunDataGroup(InspectGroup group) {
        ExecuteTaskService.RunDataGroup dataGroup = new ExecuteTaskService.RunDataGroup();
        dataGroup.setName(group.getName());
        dataGroup.setItemIds(group.getItemIds());
        dataGroup.setTarget(JSON.parseObject(group.getTarget(), Target.class));
        dataGroup.setExecuteType(group.getExecuteType());
        dataGroup.setCustomParams(group.getCustomParams());
        return dataGroup;
    }

    @Override
    public GeneralResult<Void> routineRun(Long id, String action, OperationParam param, RequestContext user) {
        if (action == null) {
            action = param.getAction();
        }
        Inspect routine = inspectRepository.findOne(id);
        if (null == routine) {
            return new GeneralResult<>(false, "数据不存在");
        }
        if (ApplyStatus.APPROVED.name().equals(routine.getStatus())) {
            if ("run".equals(action)) {
                return this.run(id, user.getTarget());
            }
            return new GeneralResult<>(false, "不支持的操作类型");
        }
        return new GeneralResult<>(false, "请先发布再执行!");
    }

    @Override
    public GeneralResult<List<StatResult>> statsNum(RequestContext context, String type) {
        Pager pager = new Pager();
        pager.setRows(Integer.MAX_VALUE);
        List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
        Param param = new Param(MapTools.simpleMap("type", type), Sign.EQ);
        params.add(param);
        pager.setParams(params);
        List<Inspect> records = inspectRepository.list(pager);
        int created = 0;
        int unapply = 0;
        int approving = 0;
        int approved = 0;
        int refused = 0;
        int developing = 0;
        for (Inspect routine : records) {
            if (ApplyStatus.CREATED.name().equalsIgnoreCase(routine.getStatus())) {
                created++;
                continue;
            } else if (ApplyStatus.UNAPPLY.name().equalsIgnoreCase(routine.getStatus())) {
                unapply++;
                continue;
            } else if (ApplyStatus.APPROVING.name().equalsIgnoreCase(routine.getStatus())) {
                approving++;
                continue;
            } else if (ApplyStatus.APPROVED.name().equalsIgnoreCase(routine.getStatus())) {
                approved++;
                continue;
            } else if (ApplyStatus.REFUSED.name().equalsIgnoreCase(routine.getStatus())) {
                refused++;
                continue;
            } else if (ApplyStatus.DEVELOPING.name().equalsIgnoreCase(routine.getStatus())) {
                developing++;
                continue;
            }
        }
        List<StatResult> results = new ArrayList<>();
        // results.add(new StatResult("新创建", created));
        // results.add(new StatResult("未申请", unapply));
        // results.add(new StatResult("审批中", approving));
        results.add(new StatResult("已发布", approved));
        // results.add(new StatResult("已拒绝", refused));
        results.add(new StatResult("开发中", developing));
        return new GeneralResult<>(true, results, "获取巡检/合规统计数据成功");
    }

    @Override
    public GeneralResult<Void> execute(ExecuteTaskService.RunData data, Long userId, String keyStamp) {
        try {
            // 插入一条记录
            verify(data);
            InspectRecord record = new InspectRecord();
            record.setKeyStamp(keyStamp);
            record.setName(data.getName());
            record.setExecuteType(data.getType());
            record.setInspectId(data.getRoutineId());
            record.setTimingId(data.getTimingId());
            record.setStatus(InspectRecord.Status.STARTING.name());
            record.setCategory(data.getCategory());
            record.setType(data.getSource());

            if (!data.getType().equals(InspectRecord.Type.FAST.ordinal())) {
                Inspect inspect = get(data.getRoutineId()).getData();
                if (null == inspect) {
                    return new GeneralResult<>(false, "启动失败：记录不存在或记录对应主机组不存在");
                }
                List<Long> items = new ArrayList<>();
                List<String> hosts = new ArrayList<>();
                inspect.getGroups().forEach(a -> {
                    items.addAll(a.getItemIds());
                    JSONArray os = JSONObject.parseObject(a.getTarget()).getJSONArray("data");
                    for (int i = 0; i < os.size(); i++) {
                        hosts.add(os.getJSONObject(i).getString("host"));
                    }
                });
                int itemNum = (int) items.stream().distinct().count(); // 基线去重
                int hostNum = (int) hosts.stream().distinct().count(); // 主机去重
                record.setItemNum(itemNum);
                record.setHostNum(hostNum);
            }
            // 1.返回结果保存-instanceId
            GenericEntityUtil.fillCreateData(record, userId);
            inspectRecordRepository.save(record);
            // 2.全部完成后 修改插入记录的状态
            taskService.executeItems(data, record.getId(), userId);
            return new GeneralResult<>(true, "启动成功");
        } catch (Exception e) {
            log.error("Inspect execute fail：", e);
            return new GeneralResult<>(false, "启动失败");
        }
    }

    /**
     * 校验数据
     *
     * @param data
     */
    private void verify(ExecuteTaskService.RunData data) {
        if (data == null) {
            throw new RuntimeException("runData不能为空");
        }
        if (StringUtils.isBlank(data.getName())) {
            throw new RuntimeException("name不能为空");
        }
        if (ListTool.isEmpty(data.getGroups())) {
            throw new RuntimeException("主机组不能为空");
        }
        for (ExecuteTaskService.RunDataGroup group : data.getGroups()) {
            if (group.getItemIds() == null || group.getItemIds().size() <= 0) {
                throw new RuntimeException(String.format("主机组【%s】巡检项不能为空", group.getName()));
            }
            List<WorkerHost> hosts = group.getTarget().getData();
            if (ListTool.isEmpty(hosts)) {
                throw new RuntimeException(String.format("主机组【%s】主机不能为空", group.getName()));
            }
            for (WorkerHost host : hosts) {
                if (host == null || "".equals(host.getHost())) {
                    throw new RuntimeException(String.format("主机组【%s】主机不能为空", group.getName()));
                }
            }
            List<InspectCustomParams> customParams =
                JSON.parseArray(group.getCustomParams(), InspectCustomParams.class);
            if (ListTool.isEmpty(customParams)) {
                continue;
            }
            Map<String, Map<String, String>> dataMap = new HashMap<>();
            for (InspectCustomParams param : customParams) {
                // 检查空数据
                if (param.getResourceId() == null) {
                    throw new RuntimeException(String.format("主机组【%s】自定义参数【资源不能为空】", group.getName()));
                }
                if (param.getItemId() == null) {
                    throw new RuntimeException(String.format("主机组【%s】自定义参数【巡检项】", group.getName()));
                }
                // if (param.getCustomParams() == null || param.getCustomParams().size() == 0) {
                // throw new RuntimeException(String.format("主机组【%s】自定义参数【参数项】不能为空", group.getName()));
                // }
                String key = param.getResourceId() + "-" + param.getItemId();
                // 检查重复数据
                Map<String, String> innerMap = dataMap.get(key);
                if (innerMap == null) {
                    innerMap = new HashMap<>();
                    List<ParamEntry> entries = param.getCustomParams();
                    for (ParamEntry entry : entries) {
                        if (innerMap.putIfAbsent(entry.getKey(), entry.getValue()) != null) {
                            throw new RuntimeException(String.format("主机组【%s】自定义参数【参数项%s-%s】中有重复的参数", group.getName(),
                                param.getResourceName(), param.getItemName()));
                        }
                    }
                    dataMap.put(key, innerMap);
                } else {
                    throw new RuntimeException(String.format("主机组【%s】自定义参数【资源】和【巡检项】选择重复", group.getName()));
                }
            }
        }
    }

}
