package com.bocloud.cop.service.inspection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cop.entity.inspection.InspectItem;
import com.bocloud.cop.entity.inspection.InspectTemplate;
import com.bocloud.cop.entity.inspection.InspectTemplateItem;
import com.bocloud.cop.intf.inspection.InspectTemplateService;
import com.bocloud.cop.repository.inspection.InspectItemRepository;
import com.bocloud.cop.repository.inspection.InspectTemplateItemRepository;
import com.bocloud.cop.repository.inspection.InspectTemplateRepository;
import com.bocloud.cop.service.utils.GenericEntityUtil;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 巡检或合规模板
 */
@Slf4j
@Service
public class InspectTemplateServiceImpl implements InspectTemplateService {

    @Autowired
    private InspectTemplateItemRepository inspectTemplateItemRepository;
    @Autowired
    private InspectTemplateRepository inspectTemplateRepository;
    @Autowired
    private InspectItemRepository inspectItemRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public GeneralResult<Void> create(CreateParam param, Long userId) {
        try {
            boolean flag = inspectTemplateRepository.exists(param.getName());
            if (flag) {
                return new GeneralResult<>(false, "模板名称已经存在，请重新输入！");
            }
            InspectTemplate template = new InspectTemplate();
            template.setName(param.getName());
            template.setType(param.getType());
            GenericEntityUtil.fillCreateData(template, userId);
            inspectTemplateRepository.save(template);
            for (CreateParam.Item item : param.getItem()) {
                Long itemId = item.getId();
                InspectTemplateItem templateItem = new InspectTemplateItem();
                templateItem.setItemId(itemId);
                templateItem.setTemplateId(template.getId());
                inspectTemplateItemRepository.save(templateItem);
            }
            return new GeneralResult<>(true, "新建模板成功");
        } catch (Exception e) {
            log.error(" Create template fail:", e);
            return new GeneralResult<>(false, "新建模板失败");
        }
    }

    @Override
    public GeneralResult<Void> update(UpdateParam param, Long userId) {
        String path = InspectTemplate.class.getSimpleName() + "_" + param.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            inspectTemplateItemRepository.deleteByTemplateId(param.getId()); // 先删除记录，再更新记录
            InspectTemplate template = inspectTemplateRepository.findById(param.getId());
            if (null == template) {
                return new GeneralResult<>(false, "模板数据不存在");
            }
            if (!template.getName().equals(param.getName())) {
                boolean flag = inspectTemplateRepository.exists(param.getName());
                if (flag) {
                    return new GeneralResult<>(false, "模板名称已经存在，请重新输入！");
                }
            }
            template.setName(param.getName());
            GenericEntityUtil.fillModifyData(template, userId);
            inspectTemplateRepository.update(template);
            for (UpdateParam.Item item : param.getItem()) {
                InspectTemplateItem templateItem = new InspectTemplateItem();
                templateItem.setItemId(item.getId());
                templateItem.setTemplateId(template.getId());
                inspectTemplateItemRepository.save(templateItem);
            }
            return new GeneralResult<>(true, "更新模板成功");
        } catch (Exception e) {
            log.error("Update template fail:", e);
            return new GeneralResult<>(false, "更新模板失败");
        }
    }

    @Override
    public GeneralResult<GridBean<InspectTemplate>> list(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<InspectTemplate> templates = inspectTemplateRepository.list(pager);
            for (InspectTemplate template : templates) {
                template.setCreator(redisTemplate.opsForValue().get("User_Name_" + template.getCreatorId()));
                if (null != template.getMenderId()) {
                    template.setMender(redisTemplate.opsForValue().get("User_Name_" + template.getMenderId()));
                }
            }
            int total = inspectTemplateRepository.count(pager.getParams());
            GridBean<InspectTemplate> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, templates);
            return new GeneralResult<>(true, bean, "查询模板成功");
        } catch (Exception e) {
            log.error("List template fail:", e);
            return new GeneralResult<>(false, "查询模板失败");
        }
    }

    @Override
    public GeneralResult<List<InspectItem>> listItems(Long id) {
        try {
            List<InspectItem> items = inspectItemRepository.listItems(id);
            return new GeneralResult<>(true, items, "查询模板成功");
        } catch (Exception e) {
            log.error("List template fail:", e);
            return new GeneralResult<>(false, "查询模板失败");
        }
    }

    @Override
    public GeneralResult<Void> delete(Long id, Long userId) {
        String path = InspectTemplate.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            inspectTemplateRepository.delete(id, userId);
            inspectTemplateItemRepository.deleteByTemplateId(id); // 删除关联表记录
            return new GeneralResult<>(true, "删除模板成功");
        } catch (Exception e) {
            log.error("Delete template fail:", e);
            return new GeneralResult<>(false, "删除模板失败");
        }
    }

}
