package com.bocloud.cop.service.inspection;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.apache.curator.framework.recipes.leader.LeaderLatchListener;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.Trigger;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.Inspect;
import com.bocloud.cop.entity.inspection.InspectRecord;
import com.bocloud.cop.entity.inspection.InspectTiming;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.ComplianceCategory;
import com.bocloud.cop.enums.InspectCategory;
import com.bocloud.cop.enums.InspectTypeEnums;
import com.bocloud.cop.intf.inspection.ExecuteTaskService;
import com.bocloud.cop.intf.inspection.InspectRoutineService;
import com.bocloud.cop.intf.inspection.InspectTimingService;
import com.bocloud.cop.model.OperationParam;
import com.bocloud.cop.repository.inspection.InspectReportRepository;
import com.bocloud.cop.repository.inspection.InspectRepository;
import com.bocloud.cop.repository.inspection.InspectTimingRepository;
import com.bocloud.cop.service.internal.CopInternalService;
import com.bocloud.cop.service.utils.GenericEntityUtil;
import com.bocloud.cop.service.utils.IgnorePropertiesUtil;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class InspectTimingServiceImpl implements InspectTimingService {

    private static final String FIXED = "fixed";
    private static final String SKIPED = "skiped";
    private static final Map<Long, ScheduledFuture<?>> SCHEDULED_FUTURES = new ConcurrentHashMap<>();
    private ThreadPoolTaskScheduler taskScheduler = new ThreadPoolTaskScheduler();
    @Autowired
    private CurrentService currentService;
    @Autowired
    private InspectTimingRepository inspectTimingRepository;
    @Autowired
    private InspectRepository inspectRepository;
    @Autowired
    private InspectRoutineService inspectRoutineService;
    @Autowired
    private CopInternalService copInternalService;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private InspectReportRepository inspectReportRepository;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public GeneralResult<InspectTiming> get(Long id) {
        try {
            InspectTiming timing = inspectTimingRepository.findOne(id);
            if (timing != null) {
                Inspect routine = inspectRepository.findOne(timing.getInspectId());
                timing.setRoutine(routine);
            }
            return new GeneralResult<>(true, timing, "查询成功");
        } catch (Exception e) {
            log.error("Query inspectTiming fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<InspectTiming> save(InspectTiming timing, Long userId) {
        try {
            boolean flag = inspectTimingRepository.exists(timing.getName(), timing.getType());
            if (flag) {
                return new GeneralResult<>(false, "该名称已存在，请重新输入!");
            }
            Inspect inspect = inspectRepository.findOne(timing.getInspectId());
            if (inspect == null) {
                return new GeneralResult<>(false, "常规检查不存在：" + timing.getInspectId());
            }
            boolean running = timing.getRunning() == null ? false : timing.getRunning();
            timing.setRunning(false);
            timing.setCategory(inspect.getCategory());
            // 处理一次性任务cron
            settingCron(timing);
            // 基础数据填充
            GenericEntityUtil.fillCreateData(timing, userId);
            inspectTimingRepository.save(timing);
            if (running) {
                GeneralResult<Void> result = this.run(timing.getId(), userId);
                return new GeneralResult<>(result.isSuccess(), timing, "保存成功，" + result.getMessage());
            }
            return new GeneralResult<>(true, timing, "保存成功");
        } catch (Exception e) {
            log.error("Save inspectTiming fail:", e);
            return new GeneralResult<>(false, "保存失败");
        }
    }

    @Transactional
    @Override
    public GeneralResult<Void> delete(Long id, Long userId) {
        String path = InspectTiming.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            InspectTiming timing = inspectTimingRepository.findOne(id);
            if (timing == null) {
                return new GeneralResult<>(false, "要删除的定时检查不存在");
            }
            // 巡检关联报表判断
            if (timing.getType().equalsIgnoreCase(InspectTiming.Type.INSPECTION.name()) && this.isRelated(id)) {
                return new GeneralResult<>(false, "该定时巡检已关联了巡检报表，请解除关联后重试");
            }

            if (null != timing.getRunning() && timing.getRunning()) {
                return new GeneralResult<>(false, "删除失败：定时检查任务正在执行中");
            }
            // this.stop(id, userId);
            inspectTimingRepository.delete(id, userId);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("Delete inspectTiming fail:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }

    /**
     * 判断定时巡检是否关联巡检报表
     *
     * @param inspectId
     * @return
     */
    private Boolean isRelated(Long inspectId) {
        return inspectReportRepository.countAllByInspectId(inspectId) > 0;
    }

    @Override
    @Transactional
    public GeneralResult<Void> update(InspectTiming timing, Long userId) {
        String path = InspectTiming.class.getSimpleName() + "_" + timing.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            if (inspectRepository.findOne(timing.getInspectId()) == null) {
                return new GeneralResult<>(false, "数据不存在：" + timing.getInspectId());
            }
            InspectTiming bean = inspectTimingRepository.findOne(timing.getId());
            if (bean == null) {
                return new GeneralResult<>(false, "未查询到要修改的定时任务");
            }
            if (null != bean.getRunning() && bean.getRunning()) {
                return new GeneralResult<>(false, "修改失败：定时任务正在执行中");
            }
            if (!bean.getName().equals(timing.getName())) {
                boolean flag = inspectTimingRepository.exists(timing.getName(), timing.getType());
                if (flag) {
                    return new GeneralResult<>(false, "该名称已存在，请重新输入!");
                }
            }
            boolean running = timing.getRunning() == null ? false : timing.getRunning();
            timing.setRunning(false);
            BeanUtils.copyProperties(timing, bean, IgnorePropertiesUtil.getNullPropertyNames(timing));
            GenericEntityUtil.fillModifyData(bean, userId);
            bean.setMender(redisTemplate.opsForValue().get("User_Name_" + userId));
            // 处理一次性任务cron
            settingCron(bean);
            inspectTimingRepository.update(bean);
            if (running) {
                GeneralResult<Void> result = this.run(timing.getId(), userId);
                return new GeneralResult<>(result.isSuccess(), "更新成功，" + result.getMessage());
            }
            return new GeneralResult<>(true, "更新成功");
        } catch (Exception e) {
            log.error("Update inspectTiming  fail:", e);
            return new GeneralResult<>(false, "更新失败");
        }
    }

    @Override
    public GeneralResult<GridBean<InspectTiming>> list(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = inspectTimingRepository.count(pager.getParams());
            List<InspectTiming> timings = inspectTimingRepository.list(pager);
            for (InspectTiming timing : timings) {
                timing.setCreator(redisTemplate.opsForValue().get("User_Name_" + timing.getCreatorId()));
                if (null != timing.getMenderId()) {
                    timing.setMender(redisTemplate.opsForValue().get("User_Name_" + timing.getMenderId()));
                }
                String category = timing.getCategory();
                if (StringUtils.isEmpty(category)) {
                    continue;
                }
                if (InspectTypeEnums.INSPECTION.name().equalsIgnoreCase(timing.getType())) {
                    timing.setCategory(InspectCategory.valueOf(category).getName());
                } else {
                    timing.setCategory(ComplianceCategory.valueOf(category).getName());
                }
            }
            GridBean<InspectTiming> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, timings);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("List inspectTiming fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<String> removeScheduleIfLeader(Long id) {
        if (!currentService.isLeader()) {
            log.info("收到定时检查[id={}]删除定时任务请求，当前服务非主服务，已经忽略", id);
            return new GeneralResult<>(true, SKIPED, "success");
        }
        try {
            InspectTiming timing = inspectTimingRepository.findOne(id);
            Assert.notNull(timing, "定时任务数据不存在");
            this.removeSchedule(timing);
            return new GeneralResult<>(true, FIXED, "success");
        } catch (Exception e) {
            log.error("RemoveScheduleIfLeader fail:", e);
            return new GeneralResult<>(false, "fail");
        }
    }

    @Override
    public GeneralResult<String> addScheduleIfLeader(Long id) {
        if (!currentService.isLeader()) {
            log.info("收到定时巡检[id={}]添加定时任务请求，当前服务非主服务，已经忽略", id);
            return new GeneralResult<>(true, SKIPED, "success");
        }
        try {
            InspectTiming timing = inspectTimingRepository.findOne(id);
            Assert.notNull(timing, "定时任务数据不存在");
            this.addSchedule(timing);
            return new GeneralResult<>(true, FIXED, "success");
        } catch (Exception e) {
            log.error("AddScheduleIfLeader fail:", e);
            return new GeneralResult<>(false, "fail");
        }
    }

    private void addSchedule(InspectTiming timing) {
        if (SCHEDULED_FUTURES.containsKey(timing.getId())) {
            this.removeSchedule(timing);
        }
        GeneralResult<Inspect> result = inspectRoutineService.get(timing.getInspectId());
        Assert.isTrue(result.isSuccess(), result.getMessage());
        ExecuteTaskService.RunData runData = inspectRoutineService.getRunData(result.getData(), timing.getName());
        runData.setTimingId(timing.getId());
        runData.setType(InspectRecord.Type.TIMING.ordinal());

        Trigger trigger = new CronTrigger(timing.getCron());
        ScheduledFuture<?> future = taskScheduler.schedule(() -> {
            String keyStamp = "D-" + timing.getId() + "-" + System.currentTimeMillis();
            runData.setCode(Catalog.Task.SCHEDULED.getValue());
            inspectRoutineService.execute(runData, timing.getMenderId(), keyStamp);
        }, trigger);
        SCHEDULED_FUTURES.put(timing.getId(), future);
        log.info("定时检查[{}]已经添加到定时任务，调度策略：{}", timing.getName(), timing.getCron());
        log.info("定时检查任务：" + JSONObject.toJSONString(SCHEDULED_FUTURES.keySet()));
    }

    private void removeSchedule(InspectTiming timing) {
        ScheduledFuture<?> future = SCHEDULED_FUTURES.remove(timing.getId());
        if (future != null) {
            future.cancel(true);
        }
        log.info("定时巡检[{}]已经从定时任务中移除", timing.getName());
    }

    public GeneralResult<Void> run(Long id, Long userId) {
        boolean success = copInternalService.scheduleInspectBroadcast(id, HttpMethod.POST);
        if (success) {
            inspectTimingRepository.changeRunning(id, userId, true);
        }
        return new GeneralResult<>(success, success ? "启动定时任务成功" : "启动定时任务失败");
    }

    public GeneralResult<Void> stop(Long id, Long userId) {
        boolean success = copInternalService.scheduleInspectBroadcast(id, HttpMethod.DELETE);
        if (success) {
            inspectTimingRepository.changeRunning(id, userId, false);
        }
        return new GeneralResult<>(success, success ? "停止定时任务成功" : "停止定时任务失败");
    }

    @Override
    public void onApplicationReadyEvent() {
        taskScheduler.setPoolSize(Runtime.getRuntime().availableProcessors());
        taskScheduler.setThreadNamePrefix("TimeInspect-");
        taskScheduler.initialize();
        InspectTimingServiceImpl service = this;
        LeaderLatchListener listener = new LeaderLatchListener() {
            @Override
            public void isLeader() {
                List<InspectTiming> list = inspectTimingRepository.findAllByRunning(true);
                list.forEach(service::addSchedule);
            }

            @Override
            public void notLeader() {
                List<InspectTiming> list = inspectTimingRepository.findAllByRunning(true);
                list.forEach(service::removeSchedule);
            }
        };
        try {
            currentService.addListener(listener);
        } catch (Exception e) {
            log.error("定时巡检任务初始化失败：", e);
        }
        if (currentService.isLeader()) {
            listener.isLeader();
        }
    }

    @Override
    public GeneralResult<Void> operate(Long id, String action, OperationParam param, RequestContext user) {
        String path = InspectTiming.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            if (action == null) {
                action = param.getAction();
            }
            if ("run".equals(action)) {
                return this.run(id, user.getTarget());
            } else if ("stop".equals(action)) {
                return this.stop(id, user.getTarget());
            }
            return new GeneralResult<>(false, "不支持的操作类型");
        } catch (Exception e) {
            log.error("operate inspect timing fail:", e);
            return new GeneralResult<>(false, "操作定时巡检失败");
        }

    }

    private void settingCron(InspectTiming timing) {
        if (null != timing.getIsLoop() && !timing.getIsLoop()) {
            DateFormat format = new SimpleDateFormat("ss mm HH dd MM ? yyyy");
            timing.setCron(format.format(timing.getRunTime()));
        }
    }

}
