package com.bocloud.cop.service.inspection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cop.entity.inspection.InspectItem;
import com.bocloud.cop.entity.inspection.InspectWarnLevel;
import com.bocloud.cop.intf.inspection.InspectWarnService;
import com.bocloud.cop.repository.inspection.InspectItemRepository;
import com.bocloud.cop.repository.inspection.InspectItemWarnRepository;
import com.bocloud.cop.repository.inspection.InspectWarnRepository;
import com.bocloud.cop.service.utils.GenericEntityUtil;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 告警模板
 */
@Slf4j
@Service
public class InspectWarnServiceImpl implements InspectWarnService {

    @Autowired
    private InspectWarnRepository inspectWarnRepository;
    @Autowired
    private InspectItemRepository inspectItemRepository;
    @Autowired
    private InspectItemWarnRepository inspectItemWarnRepository;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public GeneralResult<Void> create(InspectWarnLevel level, Long userId) {
        try {
            boolean flag = inspectWarnRepository.exists(level.getWarnLevel());
            if (flag) {
                return new GeneralResult<>(false, "告警等级已存在，请重新选择");
            }
            if (StringUtils.isEmpty(level.getColor())) {
                return new GeneralResult<>(false, "告警颜色不能为空");
            }
            GenericEntityUtil.fillCreateData(level, userId);
            inspectWarnRepository.save(level);
            return new GeneralResult<>(true, "新建告警模板成功");
        } catch (Exception e) {
            log.error("Create inspectWarnLevel fail:", e);
            return new GeneralResult<>(false, "新建告警模板失败");
        }
    }

    @Override
    public GeneralResult<Void> update(InspectWarnLevel level, Long userId) {
        String path = InspectWarnLevel.class.getSimpleName() + "_" + level.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            if (StringUtils.isEmpty(level.getColor())) {
                return new GeneralResult<>(false, "告警颜色不能为空");
            }
            InspectWarnLevel bean = inspectWarnRepository.findById(level.getId());
            if (bean == null) {
                return new GeneralResult<>(false, "告警信息不存在");
            }
            if (!bean.getWarnLevel().equals(level.getWarnLevel())) {
                boolean flag = inspectWarnRepository.exists(level.getWarnLevel());
                if (flag) {
                    return new GeneralResult<>(false, "告警等级已存在，请重新选择");
                }
            }
            GenericEntityUtil.fillModifyData(level, userId);
            inspectWarnRepository.update(level);
            return new GeneralResult<>(true, "更新告警模板成功");
        } catch (Exception e) {
            log.error("Update inspectWarnLevel fail:", e);
            return new GeneralResult<>(true, "更新告警模板失败");
        }
    }

    @Override
    public GeneralResult<Void> delete(Long id, Long userId) {
        String path = InspectWarnLevel.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 判断是否有关联的
            if (isRelated(id)) {
                return new GeneralResult<>(false, "该常告警设置已关联了巡检指标，请解除关联后重试");
            }
            List<InspectItem> items = inspectItemRepository.findByLevel(id);
            if (items.size() > 0) {
                return new GeneralResult<>(false, "有指标引用了该告警模板，请先删除引用的指标");
            }
            inspectWarnRepository.delete(id, userId);
            return new GeneralResult<>(true, "删除告警模板成功");
        } catch (Exception e) {
            log.error("Delete inspectWarnLevel fail:", e);
            return new GeneralResult<>(true, "更新告警模板失败");
        }
    }

    @Override
    public GeneralResult<GridBean<InspectWarnLevel>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<InspectWarnLevel> levels = inspectWarnRepository.list(pager);
            int total = inspectWarnRepository.count(pager.getParams());
            GridBean<InspectWarnLevel> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, levels);
            return new GeneralResult<>(true, bean, "查询告警模板成功");
        } catch (Exception e) {
            log.error("List inspectWarnLevel fail:", e);
            return new GeneralResult<>(true, "查询告警模板失败");
        }
    }

    private Boolean isRelated(Long id) {
        return inspectItemWarnRepository.countAllByRoutineId(id) > 0;
    }

}
