package com.bocloud.cop.service.network;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import com.megatron.common.utils.Common;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.constant.CacheConstant;
import com.bocloud.cop.entity.NetworkSegment;
import com.bocloud.cop.intf.network.NetworkSegmentService;
import com.bocloud.cop.repository.NetworkSegmentRepository;
import com.bocloud.cop.repository.TaskHostRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service("netWorkSegmentService")
public class NetworkSegmentServiceImpl implements NetworkSegmentService {

    @Autowired
    private NetworkSegmentRepository networkSegmentRepository;
    @Autowired
    private TaskHostRepository taskHostRepository;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public GeneralResult<GridBean<NetworkSegment>> list(Pager pager) {
        try {
            if (null == pager.getParams()) {
                pager.setParams(new ArrayList<>());
            }
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = networkSegmentRepository.count(pager.getParams());
            List<NetworkSegment> list = null;
            if (pager.getSimple()) {
                list = networkSegmentRepository.list();
            } else {
                list = networkSegmentRepository.list(pager);
            }
            GridBean<NetworkSegment> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询网段成功");
        } catch (Exception e) {
            log.error("List network segment fail:", e);
            return new GeneralResult<>(false, "查询网段失败");
        }
    }

    @Override
    public GeneralResult<Void> create(NetworkSegment segment) {
        try {
            boolean exists = networkSegmentRepository.exists(segment.getName());
            if (exists) {
                return new GeneralResult<>(false, "该名称已存在，请重新输入!");
            }
            networkSegmentRepository.save(segment);
            if (null != segment.getGroupId()) {
                refreshCache(segment.getGroupId(), segment.getId(), false);
            }
            refreshSegCache(segment, false);
            return new GeneralResult<>(true, "添加网段成功");
        } catch (Exception e) {
            log.error("Create network segment fail:", e);
            return new GeneralResult<>(false, "添加网段失败");
        }
    }

    @Override
    public GeneralResult<NetworkSegment> detail(Long id) {
        try {
            NetworkSegment segment = this.networkSegmentRepository.query(id);
            Assert.notNull(segment, "网段数据不存在！");
            return new GeneralResult<>(true, segment, "查询网段详细成功");
        } catch (Exception e) {
            log.error("Query network segment fail:", e);
            return new GeneralResult<>(false, "查询网段详细失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = NetworkSegment.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            NetworkSegment segment = this.networkSegmentRepository.query(id);
            Assert.notNull(segment, "网段数据不存在！");
            boolean result = this.networkSegmentRepository.delete(id, userId);
            if (result) {
                refreshCache(segment.getGroupId(), segment.getId(), true);
            }
            refreshSegCache(segment, true);
            if (result) {
                taskHostRepository.clearSegment(id);
            }
            return result ? new GeneralResult<>(true, "删除网段成功") : new GeneralResult<>(false, "删除网段失败");
        } catch (Exception e) {
            log.error("Remove network segment fail:", e);
            return new GeneralResult<>(false, "删除网段失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(NetworkSegment segment) {
        String path = NetworkSegment.class.getSimpleName() + "_" + segment.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            NetworkSegment bean = networkSegmentRepository.query(segment.getId());
            Assert.notNull(bean, "网段数据不存在");
            if (!bean.getName().equals(segment.getName())) {
                boolean exists = networkSegmentRepository.exists(segment.getName());
                if (exists) {
                    return new GeneralResult<>(false, "该名称已存在，请重新输入!");
                }
                bean.setName(segment.getName());
            }
            bean.setGroupId(segment.getGroupId());
            bean.setRemark(segment.getRemark());
            boolean result = networkSegmentRepository.update(bean);
            if (null != bean.getGroupId()) {
                refreshCache(bean.getGroupId(), bean.getId(), false);
            }
            refreshSegCache(bean, false);
            return result ? new GeneralResult<>(true, "修改网段成功") : new GeneralResult<>(false, "修改网段失败");
        } catch (Exception e) {
            log.error("Modify network segment fail:", e);
            return new GeneralResult<>(false, "修改网段失败");
        }
    }

    private void refreshCache(Long groupId, Long segmentId, boolean remove) {
        String key = CacheConstant.SEGMENT_ + segmentId;
        try (AutoCloseLock lock = lockFactory.getACLock(key)) {
            Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
            if (remove) {
                redisTemplate.delete(key);
            } else {
                redisTemplate.opsForValue().set(key, groupId.toString());
            }
        } catch (Exception e) {
            log.error("配置网段分组缓存失败:", e);
        }
    }

    private void refreshSegCache(NetworkSegment segment, boolean remove) {
        String key = CacheConstant.SEGMENTS;
        try (AutoCloseLock lock = lockFactory.getACLock(key)) {
            Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
            String segments = this.redisTemplate.opsForValue().get(key);
            JSONObject object = StringUtils.isEmpty(segments) ? new JSONObject() : JSONObject.parseObject(segments);
            if (remove) {
                object.remove(segment.getId() + "");
            } else {
                object.put(segment.getId() + "", segment.getName());
            }
            this.redisTemplate.opsForValue().set("segments", object.toJSONString());
        } catch (Exception e) {
            log.error("配置网段分组缓存失败:", e);
        }
    }

    @Override
    public GeneralResult<Integer> getHostConut(Long id) {
        try {
            int count = this.taskHostRepository.count(id);
            return new GeneralResult<>(true, count, "查询网段主机数量成功");
        } catch (Exception e) {
            log.error("Query network segment host count fail:", e);
            return new GeneralResult<>(false, "查询网段主机数量失败");
        }
    }

}
