package com.bocloud.cop.service.utils;

import com.bocloud.cop.entity.TaskFailRecord;
import com.google.common.collect.Maps;
import com.itextpdf.text.Font;
import com.itextpdf.text.*;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.DateTools;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Base64;
import java.util.List;
import java.util.Map;

/**
 * @Author liuyuanyuan
 * @Date 2022/8/11 0011 10:56
 * @Description:
 */
@Slf4j
public class TaskInstanceFileHelper {

    /**
     * 生成pdf
     *
     * @param records 导出数据
     * @return pdf内容byte数组
     * @throws Exception 异常
     */
    public static byte[] generatePdf(List<TaskFailRecord> records) throws Exception {
        // 字体
        BaseFont chinese = BaseFont.createFont("STSongStd-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
        // 创建文档
        Document document = new Document(PageSize.A4, 50, 50, 50, 50);
        // 创建输出流
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        PdfWriter tWriter = PdfWriter.getInstance(document, baos);
        document.open();
        document.newPage();
        generateBody(document, records, chinese);
        document.close();
        tWriter.close();
        return baos.toByteArray();

    }

    /**
     * 生成任务实例信息
     *
     * @param document pdf文档
     * @param records  导出数据
     * @param font     字体
     * @throws DocumentException 异常
     */
    private static void generateBody(Document document, List<TaskFailRecord> records, BaseFont font) throws DocumentException {
        // 字体样式
        Font headerFont = new Font(font, 24, Font.BOLD);
        Font contentFont = new Font(font, 12, Font.NORMAL);
        // 生成标题
        Paragraph header = new Paragraph("执行历史分析报表", headerFont);
        header.setAlignment(1);
        // 创建表格
        PdfPTable table = new PdfPTable(7);
        table.setTotalWidth(540);
        table.setLockedWidth(true);
        table.setSpacingBefore(20);
        table.addCell(addCell("IP地址", contentFont, BaseColor.BLUE, 1, 0));
        table.addCell(addCell("失败原因", contentFont, BaseColor.BLUE, 1, 0));
        table.addCell(addCell("节点名称", contentFont, BaseColor.BLUE, 3, 0));
        table.addCell(addCell("实例名称", contentFont, BaseColor.BLUE, 1, 0));
        table.addCell(addCell("创建时间", contentFont, BaseColor.BLUE, 1, 0));
        // 循环展示数据
        for (TaskFailRecord record : records) {
            table.addCell(addCell(record.getHost(), contentFont, null, 1, 0));
            table.addCell(addCell(record.getMessage(), contentFont, null, 1, 0));
            table.addCell(addCell(record.getNodeName(), contentFont, null, 3, 0));
            table.addCell(addCell(record.getInstanceName(), contentFont, null, 1, 0));
            table.addCell(addCell(DateTools.formatTime(record.getGmtCreate()), contentFont, null, 1, 0));
        }
        document.add(header);
        document.add(table);
    }

    /**
     * 添加单元格
     *
     * @param val               单元格内容
     * @param font              单元格字体
     * @param backgroudColor    单元格背景颜色
     * @param colSpan           单元格跨列列数
     * @param disableBorderSize 单元格边框隐藏设置
     */

    private static PdfPCell addCell(String val, Font font, BaseColor backgroudColor, int colSpan, int disableBorderSize) {
        String val1 = null == val ? " " : val;
        // 新建单元格
        PdfPCell cell = new PdfPCell(new Phrase(val1, font));
        // 水平居中
        cell.setHorizontalAlignment(Element.ALIGN_LEFT);
        if (0 != disableBorderSize) {
            cell.disableBorderSide(disableBorderSize);
        }
        // 设置宽度

        // 单元格高度
        cell.setMinimumHeight(20);
        if (backgroudColor != null) {
            // 设置背景颜色
            cell.setBackgroundColor(backgroudColor);
        }
        if (colSpan != 0) {
            // 设置跨列列数
            cell.setColspan(colSpan);
        }
        return cell;
    }

    public static GeneralResult<Map<String, Object>> download(List<TaskFailRecord> records) {
        SXSSFWorkbook workbook = new SXSSFWorkbook();
        try {
            SXSSFSheet sheet = workbook.createSheet("纳管分析");
            sheet.setRandomAccessWindowSize(-1);
            // 样式一，标题数据蓝色填充
            CellStyle style = workbook.createCellStyle();
            style.setFillForegroundColor(IndexedColors.ROYAL_BLUE.getIndex());
            style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            style.setBorderBottom(BorderStyle.THIN); // 下边框
            style.setBorderLeft(BorderStyle.THIN);// 左边框
            style.setBorderTop(BorderStyle.THIN);// 上边框
            style.setBorderRight(BorderStyle.THIN);// 右边框
            style.setAlignment(HorizontalAlignment.CENTER);
            // 样式二，内容样式
            CellStyle cellStyle = workbook.createCellStyle();
            cellStyle.setBorderTop(BorderStyle.THIN);// 上边框
            cellStyle.setBorderBottom(BorderStyle.THIN); // 下边框
            cellStyle.setBorderLeft(BorderStyle.THIN);// 左边框
            cellStyle.setBorderRight(BorderStyle.THIN);// 右边框
            cellStyle.setAlignment(HorizontalAlignment.CENTER);// 水平对齐方式
            cellStyle.setVerticalAlignment(VerticalAlignment.TOP);// 垂直对齐方式
            createReport(sheet, style);
            excelExporter(sheet, cellStyle, records);
            String fileName = "执行历史分析";
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            workbook.write(byteArrayOutputStream);
            byte[] ba = byteArrayOutputStream.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", fileName + System.currentTimeMillis() + ".xlsx");
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new GeneralResult<>(true, resultMap, "下载成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e.getMessage(), e);
            return new GeneralResult<>(false, e.getMessage());
        } finally {
            if (null != workbook) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    log.error("error message：", e);
                }
            }
        }
    }

    /**
     * excel导出
     *
     * @param sheet
     * @param cellStyle
     * @param records
     * @return
     */
    public static Boolean excelExporter(SXSSFSheet sheet, CellStyle cellStyle, List<TaskFailRecord> records) {
        int row = 1;
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            for (TaskFailRecord record : records) {
                row++;
                SXSSFRow sheetRow = sheet.createRow(row);
                sheetRow.createCell(0).setCellValue(record.getHost());
                sheet.getRow(row).getCell(0).setCellStyle(cellStyle);
                sheetRow.createCell(1).setCellValue(record.getMessage());
                sheet.getRow(row).getCell(1).setCellStyle(cellStyle);
                sheetRow.createCell(2).setCellValue(record.getNodeName());
                sheet.getRow(row).getCell(2).setCellStyle(cellStyle);
                sheetRow.createCell(3).setCellValue(record.getInstanceName());
                sheet.getRow(row).getCell(3).setCellStyle(cellStyle);
                sheetRow.createCell(4).setCellValue(sdf.format(record.getGmtCreate()));
                sheet.getRow(row).getCell(4).setCellStyle(cellStyle);
            }
            return true;
        } catch (Exception e) {
            log.error("任务执行历史分析【" + row + "行写入失败", e);
            return false;
        }
    }

    /**
     * 创建标题
     *
     * @param sheet
     * @param style
     */
    private static void createReport(SXSSFSheet sheet, CellStyle style) {
        // 模板标题处理
        String[] titles = new String[]{"IP地址", "失败原因", "节点名称", "实例名称", "创建时间"};
        int number = -1;
        SXSSFRow row = sheet.createRow(0);
        for (String title : titles) {
            number++;
            row.createCell(number).setCellValue(title);
            sheet.getRow(0).getCell(number).setCellStyle(style);
            sheet.setColumnWidth(number, 30 * 256);
            sheet.addMergedRegion(new CellRangeAddress(0, 1, number, number));
        }
    }
}
