package com.bocloud.cop.service.utils.worker;

import java.util.List;

import com.megatron.framework.http.core.HttpMethod;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSON;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Result;
import com.megatron.common.utils.ListTool;
import com.bocloud.cop.model.result.WorkerHostResult;
import com.bocloud.cop.model.result.WorkerResult;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.model.RemoteService;
import com.megatron.framework.http.model.ServiceParam;

/**
 * 同步调用Worker服务专用的远程服务
 */
@Slf4j
public class WorkerRemoteService extends RemoteService {

    public WorkerRemoteService(Service service, boolean exist) {
        super(service, exist);
    }

    public WorkerRemoteService(Service service, ServiceParam param, boolean exist, String url, HttpMethod method) {
        super(service, param, exist, url, method);
    }

    @Override
    public GeneralResult<?> invoke() {
        boolean exist = super.isExist();
        String url = super.getUrl();
        HttpMethod method = super.getMethod();
        ServiceParam param = super.getParam();
        PostDataFormat dataFormat = super.getDataFormat();
        Service service = super.getService();

        if ((!exist) || (!StringUtils.hasText(url)) || (method == null)) {
            log.error("Remote service [{}] does not exist!", service.getName());
            return new GeneralResult<>(false, "【" + service.getName() + "】服务未找到！", "404", "远端服务异常，请稍后重试！");
        }
        Result response = null;
        HttpClient client = new HttpClient(10 * 60 * 1000);
        switch (method.getMethod()) {
            case GET:
                response = client.get(param.getHeaders(), param.getParams(), url);
                break;
            case POST:
                client.setDataFormat(dataFormat);
                response = client.post(param.getHeaders(), param.getParams(), url);
                break;
            case DELETE:
                response = client.delete(param.getHeaders(), param.getParams(), url);
                break;
            case PUT:
                response = client.put(param.getHeaders(), param.getParams(), url);
                break;
            case PATCH:
                response = client.patch(param.getHeaders(), param.getParams(), url);
                break;
            case TRACE:
                response = client.trace(param.getHeaders(), param.getParams(), url);
                break;
            default:
                response = client.get(param.getHeaders(), param.getParams(), url);
        }
        return getResult(response);
    }

    private GeneralResult<?> getResult(Result response) {
        String url = super.getUrl();
        if (response.isSuccess()) {
            try {
                WorkerResult result = JSON.parseObject(response.getMessage(), WorkerResult.class);
                boolean success = result.getSuccess();
                String message = result.getMessage();
                List<WorkerHostResult> hostResults = result.getData();
                if (!success) {
                    message = getDetailedErrorMsg(hostResults, message);
                }
                return new GeneralResult<>(success, hostResults, message);
            } catch (Exception e) {
                log.error("Remote service[{}] 返回结果无法转化为WorkerResult对象，返回结果为:[{}]", url, response.getMessage());
                return new GeneralResult<>(false, "远程服务返回结果无法转化为WorkerResult对象");
            }
        }
        log.error("Remote service[{}] invoke Error:[{}]", url, response.getErrorMsg());
        return new GeneralResult<>(false, response.getErrorMsg());
    }

    /**
     * 获取worker服务返回结果中更详细的提示信息
     */
    private String getDetailedErrorMsg(List<WorkerHostResult> hostResults, String defaultMessage) {
        if (ListTool.isEmpty(hostResults)) {
            return defaultMessage;
        }
        StringBuilder sb = new StringBuilder();
        String message;
        for (WorkerHostResult hostRst : hostResults) {
            if (StringUtils.hasText(hostRst.getHost()) && !hostRst.getSuccess()) {
                message = hostRst.getMessage();
                if (sb.length() > 0) {
                    sb.append("；");
                }
                sb.append("[");
                sb.append(hostRst.getHost());
                sb.append("]");
                sb.append("执行失败，原因：");
                sb.append(message);
            }
        }
        return sb.toString();
    }
}
