package com.bocloud.cop.task;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskPlatformAlert;
import com.bocloud.cop.entity.TaskPlatformConfig;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.TaskParam;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.model.TaskPlatformModel;
import com.bocloud.cop.repository.TaskInstanceRepository;
import com.bocloud.cop.service.utils.PlatformAlertConstant;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.Assert;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

@Slf4j
public class TaskPlatformConfigTask implements Runnable {

    private List<TaskPlatformConfig> configList;
    private LockFactory lockFactory;
    private StringRedisTemplate redisTemplate;
    private TaskInstanceService taskInstanceService;
    private TaskInstanceRepository taskInstanceRepository;
    private String currTime;
    private SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    public TaskPlatformConfigTask(String currTime, List<TaskPlatformConfig> configList,
                                  StringRedisTemplate redisTemplate, AmqpTemplate amqpTemplate, LockFactory lockFactory,
                                  TaskInstanceService taskInstanceService, TaskInstanceRepository taskInstanceRepository) {
        this.currTime = currTime;
        this.configList = configList;
        this.redisTemplate = redisTemplate;
        this.lockFactory = lockFactory;
        this.taskInstanceService = taskInstanceService;
        this.taskInstanceRepository = taskInstanceRepository;

    }

    @Override
    public void run() {
        TaskPlatformConfig taskPlatformConfig = configList.get(0);
        String platform = taskPlatformConfig.getPlatform();

        log.info("开始执行平台【{}】限流任务，执行时间{}", platform, currTime);
        if (configList.size() == 1) {
            if (currTime.equals(taskPlatformConfig.getEndTime())) {
                // 此次执行的时间是当前告警配置中【执行时间】的结束时间，直接删除缓存，并且将暂缓执行的任务都放行
                dealEndPeriod(taskPlatformConfig);
            } else {
                dealCurrPeriod(taskPlatformConfig);
            }

        } else {
            // 两个告警配置信息，前一个的结束时间，后一个的开始时间
            // 删除原来的缓存
            taskPlatformConfig = configList.get(1);
            redisTemplate.delete("platform_" + platform);
            dealCurrPeriod(taskPlatformConfig);
        }
        log.info("结束执行平台【{}】限流任务，执行时间{}", platform, currTime);
    }

    // 处理当前周期中的数据
    private void dealCurrPeriod(TaskPlatformConfig taskPlatformConfig) {
        try {
            String platform = taskPlatformConfig.getPlatform();
            // 统计周期(几分钟统计一次)
            Integer period = taskPlatformConfig.getPeriod();
            Date day = new Date();
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
            String startCountTime = df.format(day) + " " + taskPlatformConfig.getStartTime() + ":00";
            // mins = 开始时间到当前时间的分钟差值
            long mins = getTimeDiffMin(formatter.parse(startCountTime));
            // 计算统计周期开始和结束时间
            Map<String, String> timeMap = getTime(period, mins);
            String startTime = timeMap.get("startTime");
            String endTime = timeMap.get("endTime");
            long remainder = mins % period;
            // 获取缓存中平台对应的限流配置信息
            String platformTemp = redisTemplate.opsForValue().get("platform_" + platform);
            log.info("平台{}限流任务周期【{}】至{}，执行时间{}", platform, startTime, endTime, currTime);
            // 缓存中没有平台配置信息或进入新周期的时候才会重新加载缓存配置信息
            if (StringUtils.isEmpty(platformTemp) || remainder == 0) {
                try (AutoCloseLock lock = lockFactory.getACLock("task_instance_platform_" + platform)) {
                    Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "平台：" + platform + "正在校验限流信息");
                    TaskPlatformModel platformModel = new TaskPlatformModel();
                    BeanUtils.copyProperties(taskPlatformConfig, platformModel);
                    platformModel.setPeriodStartTime(startTime);
                    platformModel.setPeriodEndTime(endTime);
                    // 更新缓存中平台的限流信息。
                    // 查询当前周期内任务实例数及目标主机数，需要加上前面周期限流留下的任务数和目标主机数。
                    Integer instanceNum = taskInstanceRepository.countInstanceNum(platform, startTime, endTime)
                            + taskInstanceRepository.countInstanceNumLast(platform, startTime, endTime);
                    platformModel.setCurrInstanceNum(instanceNum);
                    Integer targetHostNum = taskInstanceRepository.countTargetHostNum(platform, startTime, endTime)
                            + taskInstanceRepository.countTargetHostNumLast(platform, startTime, endTime);
                    platformModel.setCurrTargetHostNum(targetHostNum);
                    platformModel.setPlatformConfigId(taskPlatformConfig.getId());

                    //是否存在历史告警数据，如果存在则按照历史告警数据中的限流操作进行处理
                    boolean limitFlag = false;
                    List<TaskPlatformAlert> alerts = taskPlatformConfig.getAlertList();
                    if (!ListTool.isEmpty(alerts)) {
                        for (TaskPlatformAlert alert : alerts) {
                            if (PlatformAlertConstant.Status.PROCESSED.name().equals(alert.getStatus())) {
                                platformModel.setPlatformAlertId(alert.getId());
                                platformModel.setDealMethod(alert.getDealMethod());
                                limitFlag = true;
                                break;
                            } else if (PlatformAlertConstant.Status.CREATED.name().equals(alert.getStatus())) {
                                platformModel.setExistCreateAlert(true);
                                // 发送时间是否在当前告警周期内，是记录告警id，否不记录-每个周期可以发送一个记录
                                if (isEffectiveDate(alert.getGmtCreate(), formatter.parse(startTime),
                                        formatter.parse(endTime))) {
                                    platformModel.setCurrPlatformAlertId(alert.getId());
                                }
                            }
                        }
                    }
                    redisTemplate.opsForValue().set("platform_" + platform, JSONObject.toJSONString(platformModel));
                    //是否存在历史告警数据，如果存在则按照历史告警数据中的限流操作进行处理
                    if (limitFlag) {
                        // 判断当前周期还能放行的任务数  需要减去本周期已经执行的主机数
                        Integer instanceNumTemp = taskInstanceRepository.countDealedInstanceNum(platform, startTime, endTime);
                        Integer targetHostNumTemp = taskInstanceRepository.countDealedTargetHostNum(platform, startTime, endTime);
                        if(instanceNumTemp>=taskPlatformConfig.getInstanceNum() || targetHostNumTemp>=taskPlatformConfig.getTargetHostNum()){
                            return;
                        }
                        // 根据配置的任务实例数及目标主机数下发处理限流多出的任务下发到job中
                        // 不能只是当前周期的数据，还有之前周期为下发的任务
                        //taskPlatformConfig.getInstanceNum() 不能直接用，应该要减去本周期已经放行的任务数
                        List<TaskInstance> instanceList =
                                taskInstanceRepository.listLimited(platform, taskPlatformConfig.getInstanceNum()-instanceNumTemp);
                        for (TaskInstance instance : instanceList) {
                            Integer instanceTargetHostNum = taskInstanceRepository.countTargetHostNum(instance.getId());
                            if (targetHostNumTemp + instanceTargetHostNum > taskPlatformConfig.getTargetHostNum()) {
                                break;
                            }
                            instance.setIsLimited(false);
                            instance.setLimitedReleaseTime(new Date());
                            instance.setMenderId(1L);
                            taskInstanceRepository.updateLimitedRelease(instance.getId(), 1L);
                            // 还要判断任务数及目标主机数是否超出限制
                            taskInstanceService.putJob(buildJob(instance, instance.getPriority()));
                            targetHostNumTemp += instanceTargetHostNum;
                        }
                    } else {
                        List<TaskInstance> instanceList = taskInstanceRepository.listLimited(platform, null);
                        for (TaskInstance instance : instanceList) {
                            instance.setIsLimited(false);
                            instance.setLimitedReleaseTime(new Date());
                            instance.setMenderId(1L);
                            taskInstanceRepository.updateLimitedRelease(instance.getId(), 1L);
                            taskInstanceService.putJob(buildJob(instance, instance.getPriority()));
                        }
                    }

                } catch (Exception e) {
                    log.error("平台：" + platform + "限流操作异常", e);
                }
            }
        } catch (Exception e) {
            log.error("告警配置限流任务出错", e);
        }
    }

    private void dealEndPeriod(TaskPlatformConfig taskPlatformConfig) {
        try {
            String platform = taskPlatformConfig.getPlatform();
            // 如果是结束时间，删除缓存，暂缓执行的放行
            redisTemplate.delete("platform_" + platform);
            List<TaskInstance> instanceList = taskInstanceRepository.listLimited(platform, null);
            for (TaskInstance instance : instanceList) {
                instance.setIsLimited(false);
                instance.setLimitedReleaseTime(new Date());
                instance.setMenderId(1L);
                taskInstanceRepository.updateLimitedRelease(instance.getId(), 1L);
                // 还要判断任务数及目标主机数是否超出限制
                taskInstanceService.putJob(buildJob(instance, instance.getPriority()));
            }
        } catch (Exception e) {
            log.error("告警配置限流任务出错", e);
        }
    }

    private Job buildJob(TaskInstance instance, Integer priority) {
        Job job = new Job();
        job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
        job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
        job.setTaskId(instance.getUuid());
        job.setParam(Common.NAME, instance.getName());
        job.setParam(TaskParam.TASKCODE.name(), instance.getCode());
        if (null != priority) {
            job.setPriority(priority);
        }
        return job;
    }

    // 获取当前时间所在周期的开始和结束时间
    private Map<String, String> getTime(Integer period, long mins) {
        long remainder = mins % period;
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        cal.add(Calendar.MINUTE, (int) (period - remainder));
        String endTime = formatter.format(cal.getTime());
        cal.add(Calendar.MINUTE, -period);
        String startTime = formatter.format(cal.getTime());
        Map<String, String> map = new HashMap<>();
        map.put("startTime", startTime);
        map.put("endTime", endTime);
        return map;
    }

    // 开始时间到当前时间的分钟差值
    private long getTimeDiffMin(Date startDate) {
        long min = 0;
        Calendar cal = Calendar.getInstance();
        cal.setTime(startDate);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        Date createDate = cal.getTime();
        cal.setTime(new Date());
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        Date currDate = cal.getTime();
        // 一分钟对应的毫秒数
        long nm = 1000 * 60;
        long diff = currDate.getTime() - createDate.getTime();
        min = diff / nm;
        return min;
    }

    /**
     * 判断当前时间是否在[startTime, endTime]区间，注意时间格式要一致
     *
     * @param nowTime   当前时间
     * @param startTime 开始时间
     * @param endTime   结束时间
     */
    public boolean isEffectiveDate(Date nowTime, Date startTime, Date endTime) {
        if (nowTime.getTime() == startTime.getTime() || nowTime.getTime() == endTime.getTime()) {
            return true;
        }
        Calendar date = Calendar.getInstance();
        date.setTime(nowTime);
        Calendar begin = Calendar.getInstance();
        begin.setTime(startTime);
        Calendar end = Calendar.getInstance();
        end.setTime(endTime);
        return date.after(begin) && date.before(end);
    }
}
